/* 
 * isc_main.c (tkAppInit.c) --
 *
 *	Provides a default version of the Tcl_AppInit procedure for
 *	use in wish and similar Tk-based applications.
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * Copyright (c) 1994 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * SCCS: @(#) tkAppInit.c 1.21 96/03/26 16:47:07
 */

#include "tk.h"
#include "tix.h"
#include <stdlib.h>

#ifdef	__cplusplus
extern "C" {
#endif

extern int		Tk_Init _ANSI_ARGS_((Tcl_Interp *interp));
extern void		Tk_Main _ANSI_ARGS_((int argc, char **argv,
			    Tcl_AppInitProc *appInitProc));

/*
 * The following variable is a special hack that is needed in order for
 * Sun shared libraries to be used for Tcl.
 */

extern int matherr _ANSI_ARGS_((void));
static int (*tclDummyMathPtr) _ANSI_ARGS_((void)) = matherr;

#ifdef	__cplusplus
}
#endif

#ifdef TK_TEST
extern int		Tktest_Init _ANSI_ARGS_((Tcl_Interp *interp));
#endif /* TK_TEST */


#if (TCL_MAJOR_VERSION == 7) && (TCL_MINOR_VERSION < 4)
/*
 * The following variable is a special hack that allows applications
 * to be linked using the procedure "main" from the Tcl7.3 library.  The
 * variable generates a reference to "main", which causes main to
 * be brought in from the library (and all of Tcl with it).
 */

extern int main _ANSI_ARGS_((int argc, char **argv));
static int (*dummyMainPtr) _ANSI_ARGS_((int argc, char **argv)) = main;

#else

/*
 *----------------------------------------------------------------------
 *
 * main --
 *
 *	This is the main program for the application.
 *
 * Results:
 *	None: Tk_Main never returns here, so this procedure never
 *	returns either.
 *
 * Side effects:
 *	Whatever the application does.
 *
 *----------------------------------------------------------------------
 */

char *prog_name;

int
#ifdef _USING_PROTOTYPES_
main (int    argc,		/* Number of command-line arguments. */
     char **argv)		/* Values of command-line arguments. */
#else
main(argc, argv)
    int argc;			/* Number of command-line arguments. */
    char **argv;                /* Values of command-line arguments. */
#endif

{
  extern int Udp_Init(Tcl_Interp *interp);
  extern int Isc_Init(Tcl_Interp *interp);
  /* save prgram name */
  prog_name = argv[0];

    Et_Init(&argc,argv);
    if (Udp_Init(Et_Interp) == TCL_ERROR) {
      return TCL_ERROR;
    }
    fprintf(stderr," Initializing MINT \n");
    if (Tixsam_Init(Et_Interp) == TCL_ERROR) {
      return TCL_ERROR;
    }
    
    fprintf(stderr,"  Still Initializing MINT \n");
    if (Isc_Init(Et_Interp) == TCL_ERROR) {
      return TCL_ERROR;
    }
#if (TCL_MAJOR_VERSION > 7) || (TCL_MINOR_VERSION > 4)
    Tcl_SetVar(Et_Interp, "tcl_rcFileName", "~/.wishrc", TCL_GLOBAL_ONLY);
#else
     tcl_RcFileName = "~/.wishrc";
#endif
   Tk_MainLoop();
    return 0;			/* Needed only to prevent compiler warning. */
}
#endif

