/* 
 * tclXclock.c --
 *
 *      Contains the TclX time and date related commands.
 *-----------------------------------------------------------------------------
 * Copyright 1991-1995 Karl Lehenbauer and Mark Diekhans.
 *
 *-----------------------------------------------------------------------------
 */

#include "tcl.h"
#include <time.h>
#include <stdlib.h>

/*
 *-----------------------------------------------------------------------------
 *
 * Tcl_strftime --
 *     Implements the TCL strftime command:
 *         strftime clockval ?format?
 *
 * Results:
 *     Standard TCL results.
 *
 *-----------------------------------------------------------------------------
 */
int Tcl_strftime(ClientData clientData, Tcl_Interp *interp, int argc,
  char **argv)
{
  time_t t;
  char   *format;
  struct tm *tm;
  Tcl_DString buffer;
  int bufsize;

  if ((argc != 2) && (argc != 3)) {
    Tcl_AppendResult (interp, "wrong # args", argv [0], 
                          " clockval ?format?", (char *) NULL);
    return TCL_ERROR;
  }

  if ((argc >= 3) && (argv [2][0] != '\0'))
    format = argv[2];
  else
    format = "%a %b %d %X %Z %Y";

  t = strtoul(argv[1], 0, 10);
  tm = localtime (&t);

  /*
   * Format the time, increasing the buffer size until strftime succeeds.
   */
  bufsize = TCL_DSTRING_STATIC_SIZE - 1;
  Tcl_DStringInit (&buffer);
  Tcl_DStringSetLength (&buffer, bufsize);

  while (strftime (buffer.string, bufsize, format, tm) == 0) {
    bufsize *= 2;
    Tcl_DStringSetLength (&buffer, bufsize);
  }

  Tcl_DStringResult(interp, &buffer);
  return TCL_OK;
}
