#
# Integrated Session Controller
#
# Global variables:
#   conf             global conference information
#     free             list of empty member display slots
#     name             name of conference (shown in window title)
#     id               name suitable for pmm messages
#     us               own CNAME (noperiod)
#     cols             current number of columns in member display
#     max_rows         max. number of rows in display
#     member_display   priority of what SDES info to show
#     sessions         names of sessions (media/seq, but not conference name)
#
#   Session          default settings for session (see session(s,))
#
#   session(s,)      session-specific information
#     active           session is active (not being deleted right now)
#     type             audio, video, ...
#     media            media parameters
#     host             host address being added
#     port             network port
#     tried            tried to start application
#     timeout          handler reference for "create" timeout
#     remote_host      remote host in a unicast session
#
#   member(m,)       member-specific information
#     cname            SDES CNAME
#     name             SDES NAME
#     email            SDES EMAIL
#     loc              SDES LOC
#     phone            SDES PHONE
#     pos              position in member display
#
#   member(m,s,)     member/session-specific information (m: CNAME w/o dots)
#     state            state of member (talk, recent, silent)
#     alive            member is alive
#     from             network source
#     source           content source
#     immediate        no playout delay
#     start            started
#     G,*              graph
#     graphs           number of active graphs
#
#   sound            information about sound files
#     dir              directory
#     suffix           file suffix
#
#   debug            list of events to generate messages (pmm)
#
#   delay( )         time to wait for a certain event in ms
#       local           maximum time between the sending a rquest to an 
#                       existing agent and receiving an answer
#      regestration     maximum time needed to start an agent and hearing his 
#                       created message
#      display          time to display a footer news before deleting it
# 
# Generally, 's' designates session (without conference name part),
# 'm' individual member (in [noperiod] format, i.e., without dots).
#
# $Id: isc.tcl,v 1.40 1998/10/06 08:54:30 dor Exp $
# 
set version {$Revision: 2.00 $}

#
# get size of the text to be displayed in chars and save it in pixels
# -as we do not know the exact size of chars in pixles we take empirical values
#
proc  iscBalloonSize { w msg} {
    global message
    set len [llength [split $msg \n]]
    set width 0
    for {set i 0} {$i <$len } {incr i} {
	if {[string length [lindex [split $msg \n] $i] ] > $width } {
	    set width [string length [lindex [split $msg \n] $i] ]
	}
    }
    set message($w,h) [expr $len *15]
    set message($w,w) [expr $width*10]
}

#
# create the actual balloon window
#
proc iscBalloonWin { w x y } {
    global message

    set x [expr $x + 5 ]
    set y [expr $y + 5 ]
    iscBalloonSize $w $message($w,msg)
    toplevel $w.balloon
    wm title $w.balloon "Help"
    wm geometry  $w.balloon $message($w,w)x$message($w,h)+$x+$y
    pack [message $w.balloon.msg -justify left -text $message($w,msg) \
		  -bg white -width $message($w,w)] -fill both -expand yes 
}

#
# create the window containing the member infor (cname name ..)
#
proc iscBalloonMember {w msg x y {m ""} } {
    global message member
    set message($w,msg) ""
    if { [info exists member($m,name)] } {append message($w,msg) [format "NAME  : %s \n" $member($m,name) ]}
    if { [info exists member($m,cname)] } {append message($w,msg) [format "CNAME : %s \n" $member($m,cname) ]}
    if { [info exists member($m,email)] } {append message($w,msg) [format "EMAIL : %s \n" $member($m,email) ]}
    if { [info exists member($m,tool)] } {append message($w,msg) [format "TOOL  : %s \n" $member($m,tool) ]}
    if { [info exists member($m,phone)] } {append message($w,msg) [format "PHONE : %s \n" $member($m,phone) ]}
    if { [info exists member($m,loc)] } {append message($w,msg) [format "LOCATION: %s \n" $member($m,loc) ]}
    
    if { $message($w,msg) == "" } {
	append message($w,msg) "No information available 
or has left session"
    }
    iscBalloonWin $w $x $y
}

#
# create the window for  the help windows
#
proc iscBalloonMsg { w msg x y {m ""}} {
    global message Session

    if { $Session(help) } {
	set message($w,msg) $msg 
	iscBalloonWin $w $x $y
    }
}

#
# bind the mouse event to a window and initialize the command to use 
#   
proc iscBalloon { w flag {command iscBalloonMsg} {m ""}}  {
    global message
    set message($w,msg) [help_balloon $flag]
    set message($w,command) $command
    set message($w,m) $m
    bind $w <Enter> {$message(%W,command) %W $message(%W,msg) %X %Y $message(%W,m)}
    bind $w <Leave> {if {[winfo exists %W.balloon]} {destroy %W.balloon} }
}
#
# Invoked when conference name is changed; changes conf(id) accordingly.
# Adjusts window titles.
#
proc confname_trace {name element op} {
  global conf

  set conf(id) [nospace $conf(name)]
  set w [concat . [winfo children .]]
  if [winfo exists .add] {
    set w [concat $w [winfo children .add]]
  }
  foreach c $w {
    if {$c == [winfo toplevel $c]} {
      wm title    $c $conf(name)
      wm iconname $c $conf(name)
    }
  }
}

set top Null

# symbolic font names
set helvetica8  "-*-helvetica-small-r-normal--*-100-*"
set helvetica10  "-*-helvetica-medium-r-normal--*-100-*"
set helvetica10b "-*-helvetica-bold-r-normal--*-100-*"
set helvetica10o "-*-helvetica-bold-o-normal--*-100-*"
set helvetica12  "-*-helvetica-medium-r-normal--*-120-*"
set helvetica12b "-*-helvetica-bold-r-normal--*-120-*"
set helvetica14  "-*-helvetica-medium-r-normal--*-140-*"
set helvetica14b "-*-helvetica-bold-r-normal--*-140-*"
set times14      "-*-times-medium-r-*--*-100-*"

#
# X options
option add $top.audioFormat      "DVI4 1 8000" startupFile
option add $top.videoFormat      "H.261"        startupFile
option add $top.muteNewSites     false         startupFile
option add $top.defaultHost      224.2.0.12    startupFile
option add $top.defaultPort      3456          startupFile
option add $top.defaultLocal     [idstring "%h"]             startupFile
option add $top.defaultVAT       0             startupFile
option add $top.defaultTTL       16            startupFile
option add $top.network          UDP           startupFile
option add $top.maxRows          2             startupFile
option add $top.priority         duplex        startupFile
option add $top.transport        VAT           startupFile
option add $top.rate             8000          startupFile
option add $top.channels         1             startupFile
option add $top.packet           0.1          startupFile
option add $top.key              ""            startupFile
option add $top.memberDisplay    "name"        startupFile
option add $top.statsPeriod      5             startupFile
option add $top.debug            ""            startupFile
option add $top.minDelay         0.0           startupFile
option add $top.maxDelay         1.0           startupFile
option add $top.initialDelay     0.1           startupFile
option add $top.delayMultiplier  4             startupFile 
option add $top.lateTs           1             startupFile
option add $top.immediate        0             startupFile
option add $top.delaySmooth      3             startupFile
# SDES values
option add $top.cname         [idstring "%u@%i"]          startupFile
option add $top.email         [idstring "%N@%3D"]         startupFile
option add $top.name          [idstring "%n@%h"]          startupFile
option add $top.loc           [idstring "%o (%Z %z)"]     startupFile
option add $top.tool          "MINT $version [exec uname -mr]" startupFile
option add $top.img           [idstring "http://www.%3D/~%u/.face"] startupFile
option add $top.note          ""                          startupFile

#delay
set delay(local)        5000
set delay(regestration) 10000
set delay(display)      10000
set delay(rsvp_service_class) 2000
# the delay after which to control if any members were inactive in order to 
# to remove their display (only for Session(conference)=1 -i.e., teleteaching)
set delay(member_update_time)      20000  

# font definitions
option add $top.config*font   $helvetica8

trace variable conf(name) w confname_trace

set conf(us)                   "[idstring "%w@%d"] [idstring %w@%i] [idstring %w] [idstring "%u@%d"]  [idstring %u@%i] [idstring %i]"
set conf(display)             "name" 
set conf(name)                 [idstring "%N@%h"]
set conf(stats_period)         5
set conf(member_display)       {note name email loc phone cname from img tool}
set conf(max_rows)             20
# chhose between conference and teleteaching mode (conference ->1)
set Session(conference) 1
set Session(type)              audio
set Session(audio)             "PCMU 1 8000"
set Session(netghost)          ""
set Session(data)              "JPEG"
set Session(whiteboard)                0
set Session(ifloor)			0
set Session(video)             "H.261"
set Session(mute_new)          false
set Session(host)              224.2.0.12
set Session(local)              [idstring "%h"] 
set Session(network)           UDP
set Session(port)              3456
set Session(ttl)               4
set Session(priority)          duplex
set Session(transport)         VAT
set Session(frame)             0.02
set Session(key)               ""
set Session(encrypt)           0
set Session(confid)            0
set Session(help)              0

# audio settings
set Session(initial_delay)     0.1
set Session(min_delay)         0.0
set Session(max_delay)         1.0
set Session(delay_multiplier)  4
set Session(late_ts)           1
set Session(immediate)         0
set Session(delay_smooth)      3
set Session(media)             "DVI4 1 8000"

# audio control
set play(port) invalid
set record(port) invalid
set play(gain) invalid
set record(gain) invalid

# video settings
set Session(video_input)       1
set Session(width)             320
set Session(height)            240
set Session(max_bandwidth)     500
set Session(max_frame_rate)    30
set Session(quality)           50
set Session(send)              0
set Session(preview)           0
set Session(sending_slides) 0
set Session(use_jpeg_for_h261) 0
set Session(dither) ordered
set Session(gamma) 0.7

# data settings
set Session(file)         "/net/u/step/dor/picture/picture.jpg"

# netghost settings
set Session(netghost_protocol) "HTTP"
set Session(netghost_host)     "host.domain.edu"
if {[info exists env(HOME)]} {
    set Session(netghost_path) "$env(HOME)/file.ps"
} else {
    set Session(netghost_path) "/path/filename.ps"
}
set Session(netghost_file)     "FILENAME"
set Session(mode)              "receiver"
set Session(netghost_id)       "123456"


# ATM Settings 
set Session(peak_bandwidth_minimum)     1000
set Session(peak_bandwidth_target)      1000
set Session(mean_bandwidth_minimum)     10
set Session(mean_bandwidth_target)      10
set Session(mean_burst_minimum)         1
set Session(mean_burst_target)          2
set Session(device)                     fa0

#
# application controller settings
#
set Session(control)                  0
set Session(alpha)                    0.30
set Session(multiplicative_decrease)  87.5
set Session(additive_increase)        30
set Session(loss_lower_threshold)     2
set Session(loss_upper_threshold)     4
set Session(congested_recv_fraction)  10
set Session(loaded_recv_fraction)     10
set Session(algorithm)                -1 ; #no adaptation
set Session(timeout)                  900000     ;# timeout in milliseconds 
set Session(graph_interval)           5000 ;# update interval for graphs

# SDES
set Session(cname)             [idstring "%u@%i"] 
set Session(email)             [idstring "%N@%3D" ]
set Session(name)              [idstring "%N"]
set Session(loc)               [idstring "%o (%Z %z)"]
set Session(tool)              "MINT"
set Session(img)               [option get . img              Isc]
set Session(note)              [option get . note             Isc]
#set Session(source)            [idstring %i]
set sound(suffix)              au
set debug                      [option get . debug            Isc]

# activity check interval
set Session(update_interval) 60000   

# set isc default to server mode
set Session(server) normal
#RSVP
#set tspec type to generic
set Tspec(type)			0	
set Tspec(b)			128
set Tspec(r)			128
set Tspec(p)			500
set Tspec(m)			576
set Tspec(M)			9188

set Flowspec(src)			""
#set flowtype to controlled load
set Flowspec(type)		1
set Flowspec(b)			128
set Flowspec(r)			128
set Flowspec(p)			500
set Flowspec(m)			576
set Flowspec(M)			9188
set Flowspec(R)			0
set Flowspec(S)			0

set Rsvp(tspec)		[concat $Tspec(type) $Tspec(b) $Tspec(r) $Tspec(p) $Tspec(m) $Tspec(M)]
set Rsvp(flowspec)		[concat $Flowspec(type) $Flowspec(b) $Flowspec(r) $Flowspec(p) $Flowspec(m) $Flowspec(M) $Flowspec(src) $Flowspec(R) $Flowspec(S)]

set Rsvp(reserve)               0
set Rsvp(source)           ""
set Rsvp(filtertype)            0
set Rsvp(path)	                0
set Rsvp(recv_unicast_path)	0
set Rsvp(serviceclasses)       "generic_TSpec Controlled_Load Guaranteed_Service 3 4 5"

set rsvp(name)		""

set Message(rsvp_msg)           ""

# SIP
set SIP(VERSION)		SIP/2.0
set SIP(PORT)			5060
set SIP(MAX_TIME)		180000
set instance 0
set invite(guest)		""
set invite(session_name)	conf_name
set invite(session_addr)	224.2.0.12 
set invite(ttl)			4
set invite(port)		3456
set invite(email)		[option get . email            Isc]
set invite(phone)		[option get . phone            Isc]

# mailcap
set mailcap_types               {}

# changed by mkr
set connect_env(type) Default
set connect_env(net) LAN
#


# define images
set img_list {picture afraid audio video control whiteboard configure invite clock ifloor mint help exit add_media warning phonebook info ok sip_handler reservation netghost}
if {[info exists env(ISC_IMG)]} {
    foreach im $img_list {
	set image($im) [image create photo -file $env(ISC_IMG)/images/$im.gif -palette 8/8/8]
    }
} else {
    foreach im $img_list {
	set image($im) ""
    }
}

#save a few colors to use as session indicators
set colors [list blue aquamarine  azure beige bisque black brown burlywood chartreuse cornsilk chocolate DarkKhaki DarkOrchid4 DarkSeaGreen DeepPink firebrick gold honeydew IndianRed ivory LemonChiffon]

# If multicast available, use it; otherwise pmm will be used automatically.
set controller     {udp 225.2.0.1 5010}

# modify to change initialization file
if {[file exists ~/.nevotrc]} {source ~/.nevotrc}

set auto_path "$isc_library $auto_path"
 

#starting values
set Default(host)  [host2ip "224.2.0.1" ]
set Default(port)  "3458"
set Default(local) [host2ip [idstring "%h"]]

# tix resources 

# symbolic font names
set helvetica10  "-*-helvetica-medium-r-normal--*-100-*"
set helvetica10b "-*-helvetica-bold-r-normal--*-100-*"
set helvetica10o "-*-helvetica-bold-o-normal--*-100-*"
set helvetica12  "-*-helvetica-medium-r-normal--*-120-*"
set helvetica12b "-*-helvetica-bold-r-normal--*-120-*"
set helvetica14  "-*-helvetica-medium-r-normal--*-140-*"
set helvetica14b "-*-helvetica-bold-r-normal--*-140-*"
set times14      "-*-times-small-r-normal--*-140-*"

# X options
# if called from nevot, its background will be used (called with -bg)
# (as class Background, thus overriding activeBackground)
option add $top.*background lightgray startupFile
option add $top.*foreground black startupFile
option add $top.*activeBackground gray50 startupFile
option add $top.*activeForeground white startupFile
option add $top.*selectBackground purple startupFile
option add $top.*Font $helvetica10o startupFile
#option add $top.*Font $times14 startupFile

wm iconbitmap . isc_icon
wm iconmask   . isc_mask

proc tkerror {err} {
  global tk_library errorInfo

  if [info exists tk_library] {tkerror.nevot $err} else {error $errorInfo}
}


#
# Show usage information.
#
proc usage {} {
  global prog_name errorFatal

  set errorFatal 1
    return "Usage: $prog_name \[-C conf-name\] \[-D \{debug\} ]\ 
\[-A \{activity check \} ]\
\[-I ignored\] \[-n\] \[-c\] 
\[-N own name\] \
\[-p udp|tcp/pmm_address/pmm_port \] 
\[-r\] \[-i] \[-s sip_port]\
address/port/confid/media/ttl \[...\]"
}


#
# Display status message at bottom of isc.
#
proc footer {msg} {
    global Session debug delay
    if {[info exists Session(f_message)] && [winfo exists $Session(f_message)]} {
	$Session(f_message) configure -text $msg
	after $delay(display) "footer {}"
    } else {
	if { [info exists debug] } {
	    if { $debug == "pmm" } {
		puts $msg
	    }
	}
    }
}


# default files: first host-specific, then generic
if {[catch {source ~/.iscrc-$env(HOST)}]} {
  catch {source ~/.iscrc}
}


# 
#if {[catch {source [file join [info library] init.tcl]}]} {
#  catch {source [file join [info library] init.tcl] }
#}
#auto_mkindex /home/koepsel/isc_cvs *.tcl

#



# add to this list to search for sound directories
foreach sound(dir) ". /usr/demo/SOUND/sounds ~/multimedia/sounds ~" {
  if {[file readable $sound(dir)/sample.au]} break
}

# other initializations
set conf(count)    0
set conf(sessions) {}
set conf(entry) {}

#
# get the size of the screen and the number of columns to use
#
proc get_col {} {
    global CONFIGURE_LEN
    set width [winfo vrootwidth .]
    set CONFIGURE_LEN [expr $width / 200]
}
get_col

    
load_config; set_config
    

control_init $controller

# command line arguments
while {[set err [getopt $argv "VPDAC:ncN:t:f:p:r:S:i" opt optarg ]]} {
    if {$err < 0} then {
	error "$prog_name: $opt not defined\n[usage]"
    } else {
	switch -exact $opt {
	    V {
		puts "isc : $version" 
	      }
	    D {
		set debug pmm
	    }
	    P {
		set Session(server) disabled
	    }
	    A {
                set Session(update_interval) -1
            }    
	    C {
		set conf(name) $optarg
	    }
	    v {
		set Session(media) $optarg
		set Session(video) $optarg
		set Session(type) video
		set SIP(started_by_invitation) 1
		bind . <Enter>\
		    "catch {unset SIP(started_by_invitation)};bind . <Enter>"
		conference_init_2
		session_create	
	    }
	    a {
		set Session(media) $optarg
		set Session(video) $optarg
		set Session(type) audio
		set SIP(started_by_invitation) 1
		bind . <Enter>\
		    "catch {unset SIP(started_by_invitation)};bind . <Enter>"
		conference_init_2
		session_create	
	    }
	    n {
		set Session(transport) VAT
	    }
	    N {
		set Session(name) $optarg
	    }
	    p {
		set controller [split $optarg /]
	    }
	    r {
		set Session(transport) RTP
	    }
	    i {
		wm iconify .
		set SIP(started_by_invitation) 1
		bind . <Enter>\
		    "catch {unset SIP(started_by_invitation)};bind . <Enter>"

		conference_init_2

	    }
            S {
                set SIP(PORT) $optarg
            }
	}
    }
}
control_send "+ $conf(name)"

audioctl_check

conference_init

# for teleteaching start the garbage collecting procedure
global conf
set conf(conference) $Session(conference)

if { $Session(conference) == 0} {
    after 10000 member_active
}
mailcap_load
set_mailcap_subtypes

if { $Session(update_interval) >0 } {
    after [expr $Session(update_interval) + 1000 ] conference_activity } 



# URL related to isc
set Hand_URL "openURL(http://www.fokus.gmd.de/research/cc/glone/products/mint)"
set Copy_URL "openURL(http://www.fokus.gmd.de/research/cc/glone/products/mint)"
