#
# mint.tcl
# general procedure for initiating isc and mint sessions
#


# translation list from sdr codes representation to mint representation
# format  mint name       sdr name          sdr audio packet size
proc mint_sdr_code_list {} {
    global codec_list
    set codec_list { \
			 {JPEG jpeg 0}\
			 {YUV  yuv 0}\
			 {H.261 h261 0}\
			 {MPEG mpeg 0}\
			 {NV nv 0}\
			 {CELLB celb 0}\
			 {"DVI4 1 8000" dvi 0.020}\
			 {"DVI4 1 8000" dvi2 0.040}\
			 {"DVI4 1 8000" dvi4 0.080}\
			 {"GSM 1 8000" gsm 0.020}\
			 {"LPC 1 8000" lpc 0.020}\
			 {"PCMU 1 8000" pcm 0.020}\
			 {"PCMU 1 8000" pcm2 0.040}\
			 {"PCMU 1 8000" pcm4 0.080}\
		     }
}

#get the mint equivalent of the sdr codec
proc mint_sdr_codec { codec} {
    global codec_list
    
    mint_sdr_code_list
    foreach line $codec_list {
	if {[lindex $line 1] == $codec } {
	    return [lindex $line 0]
	}
    }
    return
}

#get the audio packet size proposed by sdr
proc mint_sdr_frame { codec} {
    global codec_list
    
    mint_sdr_code_list
    foreach line $codec_list {
	if {[lindex $line 1 ] == $codec } {
	    return [lindex $line 2]
	}
    }
    return
}


proc mint_request {msg} {
    global Session conf

    #check if this call is for this isc
    if {![string match [lindex $msg [expr [lsearch -exact $msg session] +1]] \
	      $conf(name)]} {
	return;
    }

    control_send "*/mint/ $conf(name)"
    #find the media type and set the variable connected to it
    set type [lindex $msg [expr [lsearch -exact $msg media] +1]]
    global ${type}
    set conf(started) 0
    set Session(type) $type

    # set the code of the session
    set ${type}(media) [mint_sdr_codec [lindex $msg [expr \
	 [lsearch -exact $msg coding] +1]]]
    set Session(media) [mint_sdr_codec [lindex $msg [expr \
         [lsearch -exact $msg coding] +1]]]
    
    #set the audio frame size
    if { $type == "audio" } {
	set ${type}(frame)  [mint_sdr_frame  [lindex $msg [expr \
	 [lsearch -exact $msg coding] +1]]]
	set Session(frame)  [mint_sdr_frame  [lindex $msg [expr \
	 [lsearch -exact $msg coding] +1]]]
    }

    #set the address and port numbers of the session
    set ${type}(host) [lindex $msg [expr [lsearch -exact $msg address] +1]]
    set Session(host) [lindex $msg [expr [lsearch -exact $msg address] +1]]
    set ${type}(port) [lindex $msg [expr [lsearch -exact $msg port] +1]]
    set Session(port) [lindex $msg [expr [lsearch -exact $msg port] +1]]
    set ${type}(transport) \
	[lindex $msg [expr [lsearch -exact $msg transport] +1]]
    set Session(transport) \
	[lindex $msg [expr [lsearch -exact $msg transport] +1]]
    set Session(ttl) [lindex $msg [expr [lsearch -exact $msg ttl] +1]]

     # set the id of the session
    set ses "$Session(type)/$Session(host):$Session(port)"
    puts " Session $ses"
    set session($ses,media) $Session(media)

    # create the session 
    # start the members window 
    wm deiconify .
    catch { destroy .conference_1 }
    conference_init_2
    session_create
}
