#
# sip_handler.tcl
# simple editor to modify ~/.sip/sip_handler file which will be read by sipd
# enables automatic replies to SIP requests
#



proc sip_handler {} {
  global image sip_handler

  # create sip_handler window
  set w .sip_handler
  if [winfo exists $w] return
  toplevel $w
  set title "Invitation Handler"
  wm title $w $title
    
  frame $w.title
  pack \
    [label $w.title.im -image $image(sip_handler) -anchor center] \
    [label $w.title.title -text $title -font *-*-*-bold-i-*-18-*-*-*-*-*-*-*] \
    -side left -anchor w

  set sip_handler(reply_all_string) Decline
  set sip_handler(entries) [read_file .sipforward]
 
  frame $w.top -relief ridge -borderwidth 4
  frame $w.center -relief ridge -borderwidth 4
  frame $w.buttons

  frame $w.top.reply
  frame $w.top.temp
  frame $w.top.perm

  pack \
    [checkbutton $w.top.reply.cb -text "Reply to all:" -width 20 -anchor nw -variable sip_handler(reply_all)] \
    [menubutton $w.top.reply.mb -menu $w.top.reply.mb.menu -relief sunken -width 16 -textvariable sip_handler(reply_all_string)] \
    [label $w.top.reply.l -text Reason:] \
    [entry $w.top.reply.e -textvariable sip_handler(all_reason_phrase)] \
    -side left
    
  menu $w.top.reply.mb.menu
  $w.top.reply.mb.menu add command -label Decline -command {reason_phrase_display .sip_handler.top.reply n; set sip_handler(reply_all_string) Decline}
  $w.top.reply.mb.menu add command -label "Not Known Here" -command {reason_phrase_display .sip_handler.top.reply d; set sip_handler(reply_all_string) "Not Known Here"}
  $w.top.reply.mb.menu add command -label "Not Currently Here" -command {reason_phrase_display .sip_handler.top.reply d; set sip_handler(reply_all_string) "Not Currently Here"}

  pack \
    [checkbutton $w.top.temp.cb -text "Temporary Forward to:" -variable sip_handler(temp)] \
    [entry $w.top.temp.e -width 45 -textvariable sip_handler(temp_to)] \
    -side left -fill x

  pack \
    [checkbutton $w.top.perm.cb -text "Permanent Forward to:" -variable sip_handler(perm)] \
    [entry $w.top.perm.e -width 45 -textvariable sip_handler(perm_to)] \
    -side left

  $w.top.reply.cb configure -command "all_display reply_all"
  $w.top.temp.cb configure -command "all_display temp"
  $w.top.perm.cb configure -command "all_display perm"

  foreach s {from subject} {
    frame $w.center.$s
    pack \
      [listbox $w.center.$s.reason -height 5 -width 25 -yscrollcommand "$w.center.$s.scroll set"] \
      [scrollbar $w.center.$s.scroll -command "$w.center.$s.reason yview;$w.center.from.action yview"] \
      [listbox $w.center.$s.action -height 5 -width 40 -yscrollcommand "$w.center.$s.scroll set"] \
      -side left -fill y
  }

  foreach s {reason action} {
    bind $w.center.from.$s <ButtonRelease-1> {
      set sip_handler(selected_window) from
      set sip_handler(selected_index) [%W nearest %y]
    }
    bind $w.center.subject.$s <ButtonRelease-1> {
      set sip_handler(selected_window) subject
      set sip_handler(selected_index) [%W nearest %y]
    }
  }
    
  pack \
    [label $w.center.l_from -text "From:                       ->                              Action:                    "] \
    $w.center.from \
    [label $w.center.l_subject -text "Subject:                   ->                              Action:                    "] \
    $w.center.subject \
    -side top
    
  pack \
    [button $w.buttons.new -width 10 -text "New Handler" -command "new_handler"] \
    [button $w.buttons.remove -width 10 -text "Remove" -command "remove_handler"] \
    [button $w.buttons.save -width 10 -text "Save" -command {save_sip_handler}] \
    [button $w.buttons.dismiss -width 10 -text "Dismiss" -command "save_check sip_handler"] \
    -side left -expand 1
    
  pack $w.top.reply $w.top.temp $w.top.perm -anchor nw

  pack $w.title
  pack $w.top $w.center $w.buttons -padx 1m -pady 1m -fill x


  # get actual invitation handling configuration
  # read handler file
  set sip_handler(entries) [read_file .sipforward]

  set i 0
  set line [split [lindex $sip_handler(entries) 0] "\t"]

  if {[lindex $line 0] == "*"} {
    switch [lindex $line 2] {
      301 {
        # Permanent Forward
        set sip_handler(perm) 1
        set sip_handler(perm_to) [lindex $line 3]
        all_display perm
      }
      302 {
        # Temporary Forward
        set sip_handler(temp) 1
        set sip_handler(temp_to) [lindex $line 3]
        all_display temp
      }
      450 {
        set sip_handler(reply_all) 1
        set sip_handler(reply_all_string) Decline
        set sip_handler(all_reason_phrase) [lindex $line 3]
        reason_phrase_display .sip_handler.top.reply n
        all_display reply_all
      }
      601 -
      602 {
        set sip_handler(reply_all) 1
        set sip_handler(reply_all_string) [lindex $line 3]
        all_display reply_all
      }
    }
    set i 1
  }
  foreach line [lrange $sip_handler(entries) $i end] {
    insert_handler $line
  }  
  set sip_handler(modified) 0
} ;# sip_handler


#
# update display of sip_handler window
#
proc all_display {selected} {
  global sip_handler

  set w .sip_handler
  set sip_handler(modified) 1

  if {$sip_handler($selected)} {
    foreach t {reply_all temp perm} {
      if {$selected != $t} {
        set sip_handler($t) 0
      }
    }
    set color gray
  } else {
    set color black
  } 
  $w.center.from.reason configure -foreground $color
  $w.center.from.action configure -foreground $color
  $w.center.subject.reason configure -foreground $color
  $w.center.subject.action configure -foreground $color
} ;# all_display


#
# add handler entry
#
proc new_handler {} {
  global sip_handler image reason action 
    
  # create add_handler window
  set w .add_handler
  if [winfo exists $w] return
  toplevel $w
  set title "Add Invitation Handler"
  wm title $w $title
   
  frame $w.title
  pack \
    [label $w.title.im -image $image(sip_handler) -anchor center] \
    [label $w.title.title -text $title -font *-*-*-bold-i-*-18-*-*-*-*-*-*-*] \
    -side left -anchor w

  frame $w.top -relief ridge -borderwidth 4
  frame $w.buttons
  frame $w.top.left
  frame $w.top.right
  frame $w.top.left.from
  frame $w.top.left.subject
  frame $w.top.right.to
  frame $w.top.right.reply

  set reason from
  set action to

  pack \
    [radiobutton $w.top.left.from.rb1 -width 10 -anchor nw -text From: -variable reason -value from] \
    [entry $w.top.left.from.e -textvariable from_string] \
    -side left

  pack \
    [radiobutton $w.top.left.subject.rb1 -width 10 -anchor nw -text Subject: -variable reason -value subject] \
    [entry $w.top.left.subject.e -textvariable subject_string] \
 -side left
    
  pack \
    [radiobutton $w.top.right.reply.rb1 -width 12 -anchor nw -text Reply: -variable action -value reply] \
    [menubutton $w.top.right.reply.mb -menu $w.top.right.reply.mb.menu -relief sunken -width 16 -textvariable reply_string] \
    [label $w.top.right.reply.l -text Reason:] \
    [entry $w.top.right.reply.e -textvariable reason_phrase] \
    -side left

  menu $w.top.right.reply.mb.menu
  $w.top.right.reply.mb.menu add command -label Decline -command {reason_phrase_display .add_handler.top.right.reply n; set reply_string Decline}
  $w.top.right.reply.mb.menu add command -label "Not Known Here" -command {reason_phrase_display .add_handler.top.right.reply d; set reply_string "Not Known Here"}
  $w.top.right.reply.mb.menu add command -label "Not Currently Here" -command {reason_phrase_display .add_handler.top.right.reply d; set reply_string "Not Currently Here"}

  pack \
    [radiobutton $w.top.right.to.rb1 -width 12 -anchor nw -text "Forward to:" -variable action -value to] \
    [entry $w.top.right.to.e -width 45 -textvariable to_string] \
    -side left

  pack \
    [label $w.top.left.l -text Invitation:] \
    $w.top.left.from \
    $w.top.left.subject \
    -side top -fill both
    
  pack \
    [label $w.top.right.l -text Action:] \
    $w.top.right.to \
    $w.top.right.reply \
    -side top -fill both

  pack \
    [button $w.buttons.add -text "Add" -command {add_handler $reason [subst $[subst $reason]_string] [subst $[subst $action]_string] $reason_phrase}] \
    [button $w.buttons.dismiss -text "Dismiss" -command "destroy $w"] \
    -side left

  pack $w.top.left $w.top.right -side left
  pack $w.title 
  pack $w.top -padx 1m -pady 1m
  pack $w.buttons
} ;# new_handler



proc add_handler {r r_string a_string r_ph} {
  global sip_handler

  
  if {($r_string == "") || ($a_string == "")} {
    warning "The specified settings are incomplete!\nPlease modify your entries."
    return 
  }
    
  set sip_handler(modified) 1
  set new_entry "$r\t$r_string\t"
  switch $a_string {
    Decline {
      if {$r_ph != ""} {
        regsub -all -- "\t" $r_ph " " r_ph
        append new_entry "450\t$r_ph"
      } else {
        append new_entry "450\tDecline"
      }
    }
    "Not Known Here" {
      regsub -all -- "\t" $a_string " " a_string
      append new_entry "601\t$a_string"
    }
    "Not Currently Here" {
      regsub -all -- "\t" $a_string " " a_string
      append new_entry "602\t$a_string"
    }
    default {
      regsub -all -- "\t" $a_string " " a_string
      append new_entry "302\t$a_string"
    }
  }
  lappend sip_handler(entries) $new_entry

  # update display
  insert_handler $new_entry
} ;# add_handler



# remove handler entries
proc remove_handler {} {
  global sip_handler

  if ![info exists sip_handler(selected_index)] return

  set sip_handler(modified) 1
  # find entry in list
  set index [lsearch $sip_handler(entries) "$sip_handler(selected_window)\t[.sip_handler.center.$sip_handler(selected_window).reason get $sip_handler(selected_index)]\t*"]
  set sip_handler(entries) [lreplace $sip_handler(entries) $index $index]

  # update display
  .sip_handler.center.$sip_handler(selected_window).reason delete $sip_handler(selected_index)
  .sip_handler.center.$sip_handler(selected_window).action delete $sip_handler(selected_index)
  unset sip_handler(selected_index)
} ;# remove_handler


    
proc reason_phrase_display {w flag} {
  global sip_handler

  set sip_handler(modified) 1

  if {$flag == "d"} {
    $w.l configure -foreground grey 
    $w.e configure -foreground grey 
    $w.e configure -state disabled
  } else {
    $w.l configure -foreground black 
    $w.e configure -foreground black 
    $w.e configure -state normal
  }
} ;# reason_phrase_display



# insert new handler to Invitation Handler window
proc insert_handler {line} {

  set line [split $line "\t"]
  set w .sip_handler.center.[lindex $line 0]
 
  $w.reason insert end [lindex $line 1]
  switch [lindex $line 2] {
    450 {
      $w.action insert end "Reply: Decline: [lindex $line 3]"
    }
    601 -
    602 {
      $w.action insert end "Reply: [lindex $line 3]"
    }
    302 {
      $w.action insert end "Forward to:[lindex $line 3]"
    }
  }
} ;# insert_handler



proc save_sip_handler {} {
  global sip_handler

  set line [split [lindex $sip_handler(entries) 0] "\t"]

  if {[lindex $line 0] == "*"} {
    set sip_handler(entries) [lreplace $sip_handler(entries) 0 0]
  }
  if {$sip_handler(reply_all)} {
    switch $sip_handler(reply_all_string) {
      Decline {
        if {$sip_handler(all_reason_phrase) != ""} {
          regsub -all -- "\t" $sip_handler(all_reason_phrase) " " sip_handler(all_reason_phrase)
          set new_entry "*\t*\t450\t$sip_handler(all_reason_phrase)"
        } else {
          set new_entry "*\t*\t450\tDecline"
        }
      }
      "Not Known Here" {
        regsub -all -- "\t" $sip_handler(reply_all_string) " " sip_handler(reply_all_string)
        set new_entry "*\t*\t601\t$sip_handler(reply_all_string)"
      }
      "Not Currently Here" {
        regsub -all -- "\t" $sip_handler(reply_all_string) " " sip_handler(reply_all_string)
        set new_entry "*\t*\t602\t$sip_handler(reply_all_string)"
      }
    }
  }
    
  if {$sip_handler(temp)} {
    if {$sip_handler(temp_to) == ""} {
      warning "You have selected \"Temporary Forward to:\"\nbut you haven't specified a location.\nPlease modify your entries."
      sip_handler
      return 
    }
    regsub -all -- "\t" $sip_handler(temp_to) " " sip_handler(temp_to)
    set new_entry "*\t*\t302\t$sip_handler(temp_to)"
  }
    
  if {$sip_handler(perm)} {
    if {$sip_handler(perm_to) == ""} {
      warning "You have selected \"Permanent Forward to:\"\nbut you haven't specified a location.\nPlease modify your entries."
      sip_handler
      return 
    }
    regsub -all -- "\t" $sip_handler(perm_to) " " sip_handler(perm_to)
    set new_entry "*\t*\t301\t$sip_handler(perm_to)"
  }
    
  if [info exists new_entry] {
    set sip_handler(entries) [linsert $sip_handler(entries) 0 $new_entry]
  }
        
  save_file .sipforward $sip_handler(entries)
  set sip_handler(modified) 0
  catch {destroy .add_handler}
  destroy .sip_handler
} ;# save_sip_handler








