#
# sip_media_list.tcl
# procedures to handle media_lists
# - create checkbutton frame to specify media encodings
# - add media descriptions to media_list
# - convert a media_list or media descriptions to an invite array
#
# media_list: 
# - is a part of an invite, request or response array
# - consists of several media descriptions
# media description:
# "media port session_address ttl code_list information_string"
# e.g.:
# "audio 3456 224.2.0.12 16 {pcmu dvi} {german audio stream}"
#



proc create_media_list {} {
    global invite
    
    set invite(media_list) ""
    for {set row 1} {$row < [lindex [grid size .invite.body] 1]} {incr row} {
	if {$invite($row) == 1} {
	    # dor: update all invite(row,ttl) to the ttl value choosen for 
	    # the session
	    set invite($row,ttl) $invite(ttl)
	    set ml $invite($row,type)
	    foreach tag {port session_addr ttl code_list} {
		if {$invite($row,$tag) == ""} {
		    warning "The settings for media type $invite($row,type) are incomplete!\nPlease modify your entries."
		    return 1
		}
		lappend ml $invite($row,$tag)
	    }
	    lappend ml $invite($row,information)
	    foreach m $invite(media_list) {
		if {[lindex $m 1] == $invite($row,port) && [lindex $m 2] == $invite($row,session_addr)} {
		    warning "You have specified several sessions using the same network address and port!\nPlease modify your entries."
		    return 1
		}
	    }
	    lappend invite(media_list) $ml
	}
    }
    if {$invite(media_list) == ""} {
	warning "No media selected!\nPlease modify your entries."
	return 1
    }
return 0

}




#
# convert a media description to an invite array
# - needed for the display in standard and advanced invite windows
#
proc media2invite {media} {
  global invite
      
  set m [lindex $media 0]
  set invite($m) 1
  set invite($m,port) [lindex $media 1]
  set invite($m,session_addr) [lindex $media 2]
  set invite($m,ttl) [lindex $media 3]
  set invite($m,code_list) [lindex $media 4]
  set invite($m,profile_list) ""
  foreach c $invite($m,code_list) {
    set invite($c) 1
    lappend invite($m,profile_list) [get_profile $c]
  }
  set invite($m,information) [lindex $media 5]
} ;# media2invite


