/*
 * gv.h -- Main include file for ghostview.
 * Copyright (C) 1992  Timothy O. Theisen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   Author: Tim Theisen           Systems Programmer
 * Internet: tim@cs.wisc.edu       Department of Computer Sciences
 *     UUCP: uwvax!tim             University of Wisconsin-Madison
 *    Phone: (608)262-0438         1210 West Dayton Street
 *      FAX: (608)262-9777         Madison, WI   53706
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include <netdb.h>
#include <pwd.h>         /* struct passwd */ 
#include <X11/Xos.h>
#include <sys/stat.h>
#include <sys/utsname.h> 
#include <X11/Intrinsic.h>
#define XtSetFloatArg(arg, n, d) \
    if (sizeof(float) > sizeof(XtArgVal)) { \
	XtSetArg(arg, n, &(d)); \
    } else { \
	XtArgVal *ld = (XtArgVal *)&(d); \
	XtSetArg(arg, n, *ld); \
    }
#include "Ghostview.h"

/*
 * Definitions for remote callbacks
 */


/* net_mode = NORMAL:   normal use of ghostview
 * net_mode = SENDER:   ghostview transmits its own user-generated events
 * net_mode = RECEIVER: ghostviewe acts as a receiver of events
 */

#define SENDER 2
#define RECEIVER 1
#define NORMAL 0

/* packet structure for transmitting events to other net-users */

struct statuspacket_t{
  int type;
  long int connection_ID;
  long int page;
  short int orientation;
  unsigned char swap_landscape;
  int x;
  int y;
  Boolean visible;
  int optlen;
  char host[128];
  char file[128];
  char* options;
};


/* everything we need to know about our socket */

struct network_descriptor{
    int connection_ID;
    String address;         /* Network adress in ASCII notation */
    unsigned short port;              /* port number */
    unsigned char ttl;     /* time to life value for multicast */
    int mode;              /* SENDER,RECEIVER or NORMAL */
    struct sockaddr_in addr;  /* address,port,address family for syscalls */
    int sd;                /* socket descriptor */
    int interval;          /* periodically send of status packets */
    XtIntervalId interval_id;
    int pmmsd;             /* socket descriptor for local pmm */
    unsigned long update_interval; /* interval to send an active message */
};
    


/* Application resources */
typedef struct _AppResources {
  Boolean show_title;		/* whether to show title */
  Boolean show_date;		/* whether to show date */
  Boolean show_locator;	/* whether to show locator */
  Boolean auto_center;	/* whether to automatically center the page */
  int wm_horiz_margin;	/* Space taken by window manager */
  int wm_vert_margin;		/* Space taken by window manager */
  int minimum_magstep;	/* smallest magstep allowed */
  int maximum_magstep;	/* largest magstep allowed */
  int magstep;		/* default magstep */
  XtPageOrientation orientation;	/* default orientation */
  String page;		/* first page to show */
  String pagemedia;		/* default page media */
  Boolean force_orientation;	/* use default to override document comments */
  Boolean force_pagemedia;	/* use default to override document comments */
  Boolean swap_landscape;	/* Landscape comment maps to Seascape */
  String print_command;	/* command used to print doc, usually "lpr" */
  String printer_variable;	/* env varaible to use, usually "PRINTER" */
  String default_printer;	/* printer to use if no PRINTER is not set*/
  String print_prompt;	/* string to prompt user for printer name */
  String print_fail;		/* string to inform user that print failed */
  String open_prompt;		/* string to prompt for file name to open */
  String open_fail;		/* string to inform user that open failed */
  String save_prompt;		/* string to prompt for file name to save */
  String save_fail;		/* string to inform user that save failed */
  String address_prompt;         /* string to prompt for internet address */
  String port_prompt;            /* string to promt for port number */
  String ttl_prompt;             /* string to prompt for time to life (mcast) */
  String connection_ID_prompt;
  String net_fail;                /* string inform the user about network failure */ 
  /* Work arounds for others' bugs */
  Boolean openwindows;	/* whether to work around openwindow bug */
  Boolean ncdwm;		/* whether to work around ncdwm bug */
  int net_mode;                 /* whether the programm should act as sender,recv */
  String address;               /* the address for sending statusmessages */
  int port;                     /* simply the port number */
  int ttl;                      /* hopcount for multicast packets */
  int connection_ID;
  int interval;                  /* interval for transmitting status packets */
} AppResources;

extern float	default_xdpi;
extern float	default_ydpi;

extern int	num_ghosts;
extern FILE	*psfile;
extern String	filename;
extern String	oldfilename;
extern int	current_page;
extern int	current_magstep;
extern XtPageOrientation	current_orientation;
extern int	default_pagemedia;
extern int	current_pagemedia;
extern Boolean	force_document_media;
extern int	document_media;
extern int	current_llx;
extern int	current_lly;
extern int	current_urx;
extern int	current_ury;
extern int	base_papersize;
extern Boolean	info_up;
extern int	force_setting;
extern Pixmap	dot_bitmap;
extern Pixmap	menu16_bitmap;
extern Pixmap	tie_fighter_bitmap;
extern String	toc_text;
extern int	toc_length;
extern int	toc_entry_length;
extern int	info_length;
extern time_t	mtime;
extern struct document *doc;
extern struct document *olddoc;
extern Atom	wm_delete_window;
extern int	catch_Xerror();
extern XErrorHandler	old_Xerror;
extern Boolean	dying;
extern XErrorEvent	bomb;


enum {OPEN, PRINT_WHOLE, PRINT_MARKED, SAVE, NET};
extern int      mode;
extern struct network_descriptor nwd;
extern XtInputId net_id;
extern char pmmmessage[];
extern XtAppContext app_con;
extern AppResources app_res;

/* Widgets */
extern Widget toplevel;
extern Widget     form;
extern Widget         titlebutton;
extern Widget             titlemenu;
extern Widget         datebutton;
extern Widget             datemenu;
extern Widget         locator;
extern Widget         box;
extern Widget             filebutton;
extern Widget                 filemenu;
extern Widget                     openbutton;
extern Widget                     reopenbutton;
extern Widget                     netbutton;
extern Widget                     printwholebutton;
extern Widget                     printmarkedbutton;
extern Widget                     savebutton;
extern Widget                     copyrightbutton;
extern Widget                     quitbutton;
extern Widget             pagebutton;
extern Widget                 pagemenu;
extern Widget                     nextbutton;
extern Widget                     showbutton;
extern Widget                     prevbutton;
extern Widget                     centerbutton;
extern Widget                     markbutton;
extern Widget                     unmarkbutton;
extern Widget             magstepbutton;
extern Widget                 magstepmenu;
extern Widget                     *magstepentry;
extern Widget             orientationbutton;
extern Widget                 orientationmenu;
extern Widget                     portraitbutton;
extern Widget                     landscapebutton;
extern Widget                     upsidedownbutton;
extern Widget                     seascapebutton;
extern Widget                     swapbutton;
extern Widget             pagemediabutton;
extern Widget                 pagemediamenu;
extern Widget                     *pagemediaentry;
extern Widget         toc;
extern Widget         pageview;
extern Widget             page;

/* Popup widgets */
extern Widget infopopup;
extern Widget     infoform;
extern Widget         infotext;
extern Widget         infobutton;
extern Widget copyrightpopup;
extern Widget     copyrightform;
extern Widget         copyrighttext;
extern Widget         copyrightbutton;
extern Widget dialogpopup;
extern Widget     dialog;
extern Widget netdialogpopup;
extern Widget     netdialog;
extern Widget sender;  

/* Dialogs */
extern Widget CreateDialog();
extern String GetDialogPrompt();
extern void SetDialogPrompt();
extern String GetDialogResponse();
extern void SetDialogResponse();
extern void ClearDialogResponse();

extern Widget CreateNetDialog();
extern String GetAddressDialogPrompt();
extern void SetAddressDialogPrompt();
extern String GetAddressDialogResponse();
extern void SetAddressDialogResponse();
extern void ClearAddressDialogResponse();
extern String GetPortDialogPrompt();
extern void SetPortDialogPrompt();
extern String GetPortDialogResponse();
extern void SetPortDialogResponse();
extern void ClearPortDialogResponse();
extern String GetTtlDialogPrompt();
extern void SetTtlDialogPrompt();
extern String GetTtlDialogResponse();
extern void SetTtlDialogResponse();
extern void ClearTtlDialogResponse();
extern String GetIdDialogPrompt();
extern void SetIdDialogPrompt();
extern String GetIdDialogResponse();
extern void SetIdDialogResponse();
extern void ClearIdDialogResponse();

/* Callbacks */
extern void quit_ghostview();
extern void popup();
extern void net_dialog();
extern void popup_dialog();
extern void reopen_file();
extern void prev_page();
extern void this_page();
extern void next_page();
extern void center_page();
extern void mark_page();
extern void unmark_page();
extern void set_magstep();
extern void set_orientation();
extern void swap_landscape();
extern void set_pagemedia();
extern void track_and_zoom();
extern void message();
extern void output();
extern void okay();
extern void dismiss();
extern void destroy();
extern void destroy_ghost();
extern void pointer_change();
extern void receive_status();
extern void send_status();
extern void receive_pmm();
/* Actions */
extern void gv_copyright();
extern void gv_quit();
extern void gv_open();
extern void gv_reopen();
extern void gv_save();
extern void gv_print_whole();
extern void gv_print_marked();
extern void gv_prev();
extern void gv_show();
extern void gv_next();
extern void gv_center();
extern void gv_mark();
extern void gv_unmark();
extern void gv_set_magstep();
extern void gv_increase_magstep();
extern void gv_decrease_magstep();
extern void gv_set_orientation();
extern void gv_swap_landscape();
extern void gv_set_pagemedia();
extern void gv_default();
extern void gv_force();
extern void gv_delete_window();
extern void gv_delete_zoom();
extern void gv_dismiss();
extern void gv_scroll_up();
extern void gv_scroll_down();
extern void gv_scroll_left();
extern void gv_scroll_right();
extern void gv_erase_locator();
extern void gv_check_file();

/* Misc */
extern void show_page();
extern Boolean setup_ghostview();
extern void layout_ghostview();
extern void magnify();
extern String open_file();
extern String save_file();
extern String print_file();
extern void pscopydoc();
extern void positionpopup();
extern Boolean set_new_magstep();
extern Boolean set_new_orientation();
extern Boolean set_new_pagemedia();
extern void build_pagemedia_menu();
extern Widget build_label_menu();
extern void new_file();
extern String setup_send_socket();
extern String setup_receive_socket();
extern int send_pmm();
extern int setup_pmm_socket();
extern void register_timer();
extern void update_timer();
