/*
 *  nevit_main.c --
 * 
 * This file mainly handles the openning and closing of the network interface,
 * the video device and files.
 * 
 * The configuration parameters are received from nevit.tcl and from the
 * standard input in case of a standalone nevit without ISC. 
 *       Main function of nevit
 *
 *  (C) 1995 Dorgham Sisalem GMD Fokus
 *
 *  See the copyright file for copyright information
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "/usr/include/netinet/in.h"  /* struct sockaddr_in */
#include <arpa/inet.h>   /* inet_ntoa() */
#include <string.h>
#include <errno.h>
#include <values.h>
#include <unistd.h>      /* select(), perror() */
#include <signal.h>
#include <assert.h>
#include <tcl.h>
#include <tk.h>
#include "types.h"
#include "ansi.h"
#include "rtp.h"
#include "nevit.h"
#include "util.h"
#include "debug.h"
#include "udp.h"
#include "ac.h"
#include <math.h>
#include "vic_jpeg.h"
#include "yuv.h"
#include "xv.h"
#include "transmit_atm.h"

int DEBUG=0;
static char rcsid[]  = "$Id: nevit_main.c,v 1.9 1998/08/18 15:25:00 dor Exp $";


extern Tcl_Interp *Et_Interp;

/*************  global variables declaration ************************/
char *picture=NULL;
static Tk_TimerToken Data_TimerToken[NUM_LEVELS];  
static session_t session;     /* global session structure */
session_t *s = &session;      /* pointer to the global session structure */
Tcl_Interp *interp_;          /* global interpreter for this application */

static int Levels_Count=0;   /* keeps track of the number of opened levels */
static struct sockaddr_in *rmt_addr[NUM_LEVELS]; /*addresses of the different*/
                                                 /* multicast levels */
static struct sockaddr_in  my_addr;             /* my own address */


/************* Local procedures declaration ************************/
int open_connection(int level, int port, int ttl, char *addr);
int calc_time_interval(int level);
int Exit_Level(int level);

extern int rtcp_timer_handler(ClientData client);
extern void RTCP_activity_handler(ClientData session);

/****************************************************************************/
/*   how to use nevit */
/****************************************************************************/

void usage()
{
  fprintf(stderr, 
	  "Usage: nevit [-b kbit/s] [-f frames/s] [-h] [-C [JPEG|YUV]] [-q quality] [-I input] host/port/ttl\n or \n nevit -s  (for remote control)\n");
}

/****************************************************************************/
/*
 * check if we are doing controlled nevit -i.e from isc or are using online 
 * parameters
 */
/****************************************************************************/

int parse_online(int argc, char *argv[])
{
  extern char *optarg;
  extern int optind;

  char c;
  while ((c = getopt(argc, argv, "sb:hf:R:DC:Gp:Pm:a:I:q:t:")) != EOF) {

    switch(c) {
    case 'D':
      session.debug=1;
      DEBUG=1;
      break;

    case 's':
      session.passive = 1;
      break;
    case 'h':
      usage();
      Exit(); 
    }
  }
  return 1;
}

void parse(int argc, char *argv[])
{
  extern char *optarg;
  extern int optind;
  char c;
  char command[100];
  int send=0;

  optind=1;
  while ((c = getopt(argc, argv, "sb:hf:R:d:GC:p:Pm:a:I:q:t:")) != EOF) {
    switch(c) {
    case 'G' :
      sprintf(command,"nevit_config grab");
      Tcl_Eval(interp_,command);
      break;
       
    case 's':
      session.passive = 1;
      break;
    case 'b':
      send=1;
      sprintf(command,"nevit_config max_bandwidth %s",optarg),
      Tcl_Eval(interp_,command);
      break;
    case 'P':
      sprintf(command,"nevit_config  preview 1"),
      Tcl_Eval(interp_,command);
      break; 
    case 'h':
      usage();
      Exit();
      break;
    case 'q':
      sprintf(command,"nevit_config  quality %s",optarg),
      Tcl_Eval(interp_,command);
      break;
    case 'f':
      sprintf(command,"nevit_config max_frame_rate  %s",optarg),
      Tcl_Eval(interp_,command);
      break;
    case 'I':
      sprintf(command,"nevit_config video_input  %s",optarg),
      Tcl_Eval(interp_,command);
      break;
    case 'C':
      if (!strcasecmp(optarg, "JPEG")) {
	Tcl_Eval(interp_,"nevit_config media JPEG");
      } else if (!strcasecmp(optarg, "YUV")) {
	Tcl_Eval(interp_,"nevit_config media YUV");
      } else {
	message_output("Code Not Supported",1);
	fprintf(stderr,"Code Not Supported");
      }
      break;
    default :
      usage();
      Exit();
    }
  }
  if(send)
    Tcl_Eval(interp_,"nevit_config send 1 ");
}

/****************************************************************************/
/* read the online arguments, retreive the address and port numbers
 * and open the necessary connections
 */
/****************************************************************************/

int open_online(int argc, char *argv[])
{
  int port=0;
  int ttl = 16;
  char *s;
 
  if (!argv[optind]) {
    usage();
    Exit();
  } 
  
  /* parses host description */
  s = strchr(argv[optind], '/');
  if (!s)
    message_output(" should give address in host/port format",0);
  else {
    *s = '\0';
    port = htons(atoi(s+1));
    s = strchr(s+1, '/');
    if (s) {
      ttl = atoi(s+1);
    }
  }
  session.net.scattered_write_possible = 1;
  /* 
   *  we save one memcpy from the grabber buffer if scattered write
   *  operations exist (no need to place the packet header in front
   *  of the buffer)
   */
  Levels_Count+=open_connection(FIRST_LEVEL,port,ttl,argv[optind]);
  return 1;
}


/****************************************************************************/
/*
 * get the local address of the calling host, or the address of the interface
 * from which the user whiches to send its data.
 * retun :  1 success
 *          0 something went wrong
 */
/****************************************************************************/

int get_myaddr(struct sockaddr_in* my_addr,char* local)
{
  my_addr->sin_family = AF_INET;
  if(local[0]!='{') { 
    struct hostent *name;
    name= gethostbyname(local);
    if (name == NULL) {
      fprintf(stderr, "Can't resolve host %s\n", local);
      return (0);
    } 
    else {
      memcpy((char *)&(my_addr->sin_addr),name->h_addr,name->h_length);
    }
  }
  else {
    /* no interface was specified so use the default one */    
    char *name;
    name=idstring("%d",100);
    my_addr->sin_addr=host2ip(name);
  }
  return 1;
}

/****************************** NeViT Simple *************************
 * with this version a file is opened and repeatedly sent. It was implemented
 * just as for testing and as a backgroung load generator.
 * With this version it is possible to repeatedly send an image and thereby
 * produce a background load.
 * 
 * To compile such a version one must comment the XIL macros in the makefile
 * and replace " if {![regexp -nocase video $st]} {" with
 *    " if {![regexp -nocase data $st]} {" in nevit.tcl proc join_session
 *
 */

/*
 * open a file containing a jpeg picture and return a buffer with the contents
 * of this picture
 */
char*  open_File()
{
  static char file[10000];
  static char* last_picture=NULL;
  FILE* fp;
  static size_t length;

  if(picture!=NULL&&(last_picture==NULL||strcmp(picture,last_picture))) {
    s->local->width=320;
    s->local->height=240;
    s->codec.quality=50;
    s->send[FIRST_LEVEL]=1;
    fp =fopen(picture,"r"); 
    if(fp==NULL) {
      perror("can not open file: try another name");
      if(picture){
	free(picture);
      }
      picture=last_picture=NULL;
      return NULL;
    }
    length=fread(file,(size_t)1,(size_t)10000,fp);
    fclose(fp);
    last_picture=picture;
  }
  session.local->rtp_s[FIRST_LEVEL].nbytes=length;
  return picture? file:picture;
}
/****************************** End NeViT Simple *********************/

/*
 * low-pass filter to compute the short time statistics                 
 * the smaller a, the greater is the influence of the old value 
 */
float filter(float old, float new, float a)
{
  return ( (1-a)*old + a*new );
}

int dummy(char** image)
{
  return 1;
}
int dummy1() 
{
  return 1;
}

/****************************************************************************/
/*
 * initialise xil library 
 */
int xil_init(session_t *s) 
{
#ifdef XIL
  if(xil_open_device()==NULL){
    message_output("could not open Sun video grabber",1);
    s->grabber.grab  = dummy;
    s->grabber.close = dummy1;
    
    return 0;
  }
  s->local->width=s->local->height=0;
  if(xil_init_grab(&s->local->width,&s->local->height,
		   s->codec.quality, 
		   s->grabber.whichinput,
		   s->codec.compressor)==NULL) {
    perror(" XIL Stuff error, repeat later after adjusting the parameters ");
  }

  s->grabber.grab  = xil_grab_image;
  s->grabber.close = xil_end_grab;
  return 1;
#endif
}



/****************************************************************************/
/*
 * grab and compress one frame from the video board, call fragmentation and
 * transmit function, update bandwidth and frame rate statistics and 
 * adjust frame rate to the upper bounds if necessary
*/
void transmit_frame(int level)
{
  int size;
  char *image;  /* pointer to image buffer */
  extern  int display_frame(char* image,int len,char * decoder);
      
#ifdef XIL
  session.local->rtp_s[level].nbytes = session.grabber.grab(&image);
#else
  if((image=open_File())==NULL){
    return;
  }
#endif
  if(session.local->rtp_s[level].thumbnail==1){
    write_thumbnail(session.local,image);
    session.local->rtp_s[level].thumbnail=0;
    s->local->token=
      Tk_CreateTimerHandler(1000,(Tk_TimerProc *)thumbnail_handler,
			  (ClientData) s->local);
  } 


  if ((session.local->preview==1)&&session.send[FIRST_LEVEL]) {
    if(display_frame(image,session.local->rtp_s[level].nbytes,
		      session.local->decoder)<0){
       perror(" - could not display frame");
       Exit();
       }
     
  }
  else {
    if(session.local->preview==0){
      destroy_display();
      session.local->preview=-1;
    }
  }
  /* this is a ##hack## for some reason the tool looses contact to isc when
   * displaying its own output with a high frame rate.
   * Therefore we check the pmm socket after each grab operation to see
   * if some messages were received. The PMM socket is non blocking. 
   */
  session.net.read_pmm((ClientData)PMM); 
  /*
   *  if scattered write is not possible -e.g, ATM-, we copy the image bytes 
   *  into our buffer starting after the packet header
   */
  if (!session.net.scattered_write_possible) {
    memcpy(session.local->image + session.codec.hdr_len, image, 
	   session.local->rtp_s[level].nbytes);
    session.local->image+=session.codec.hdr_len;
  }
  else
    session.local->image = image;

  /* check if we need to send and RTCP packet first */
  /*if(s->delay_rtcp!=1000) { 
    if(s->delay_rtcp==level) { 
      rtcp_timer_handler_delayed(s->delay_rtcp); 
      s->delay_rtcp=1000; 
    } 
  }*/
  /* fragment and transmit frame */
  if((size = session.codec.encode(&session, level))<0) {
    perror(" Problems in writing on network ");
    Exit(); 
  }
  session.local->rtp_s[level].active = 1;
  session.stats.frame_size =filter(session.stats.frame_size, size,
				    FILTER_PARAMETER);
  session.local->rtp_s[level].frames_sent++;
  session.local->rtp_s[level].octets_sent+= size;
  /* make sepearte measurements for the statistics */
  session.local->rtp_s[level].packets_last++;
  session.local->rtp_s[level].octets_last+=size;
}

/****************************************************************************/
/* round the numbers */
int round(float value)
{
  if(value-ceil(value)>=0.5)
    return((int)ceil(value)+1);
  return((int)ceil(value));
}
/****************************************************************************/
/*
 * calc_level -> which level to send the next frame on
 * 
 * keep an array of the size of the frame rate -it is initialized to the 
 * max_frame _rate, but we only use actual frame rate
 * all entries of the array are initialized to the value of the heighest level
 * for number of levels starting at the heighest level 
 *     set each array[frame rate/bw_level] to level
 *   
 * note that bw_level here denotes the bandwdith of this level plus the 
 * bandwidth of underlaying levels
 *
 * this operation is is repeated every sent frame rate frames
 */
/* conside that we are sending in 3 layers with an overall rate of 800 Kb/s  */
/* with a frame_size of 10000 Bytes -i.e. a frame rate of 10 frames/s */ 
/* we want to send on the first level 240 kb/s, the second level 240 kb/s */
/* and the third one 320 kb/s -i.e, 3, 3, 4 frames /sec */
 
/* we start by: */
/* setting level_bw[0]->3 */
/*         level_bw[1]->6 */
/*         level_bw[2]->10 */
 
/* now we do 2 for loops */
/* the first one is for the levels */
/* for example: */
/* k=2 */
/* the second loop: */
/*         l=0 */
/*                 levels[0]= 3 */
/*                 l=0+ round(10/10)=1 */
/*         l=1 */
/*                 levels[1] = 3 */
/*                 l==1+round(10/10)=2 */
/*         l=9 */
/*                 levels[9] = 3 */
 
/* levels= 3 3 3 3 3 3 3 3 3 3 */
 
/* k=1 -second level */
 
/*         l=0 */
/*                 levels[0]= 2 */
/*                 l=0+ round(10/6)=2 */
/*         l=2 */
/*                 levels[2]= 2 */
/*                 l=2+ round(10/6)=5 */
/*         l=4 */
/*                 levels[4]= 2 */
/*                 l=2+ round(10/6)=6 */
/*         l=6 */
/*                 levels[6]= 2 */
/*                 l=6+ round(10/6)=8 */
/*         l=8 */
/*                 levels[8]= 2 */
 
/* k=0 -base layer */
 
/*         l=0 */
/*                 levels[0]= 1 */
/*                 l=0+ round(10/3)=3 */
 
/*         l=3 */
/*                 levels[3] = 1 */
/*                 l=3+round(10/3)=6 */
 
/*         l=6 */
/*                 levels[6] = 1 */
/*                 l=6+round(10/3)=9 */
 
/*         l=9 */
/*                 levels[9]=1 */
 
 
/* levels= 3 3 3 3 3 3 3 3 3 3 after the first loop */
/* levels= 2 3 2 3 2 3 2 3 2 3 after the second loop */
/* levels= 1 3 2 1 2 3 1 3 2 1 after the final loop */
 
/* due to the rounding problems the base level gets 4 frames/s and the  */
/* upper level only 3.  We aim at achieveing a rather periodic distribution */
/* of the frames -i.e,  sombody getting frames up to the second level would 
/* get a frame something like every 0.2 second.  */

int calc_level(float frame_size,int *rate)
{
  static int levels[3000];
  static int i=0,l,k;
  static int levels_bw[NUM_LEVELS];
  static int Rate;
  static int frame_rate=1;

  if((i=i%(int)frame_rate)<1) {
    levels_bw[0]=(session.local->rtp_s[FIRST_LEVEL].bandwidth*1000)/
      (frame_size*8);
    if(levels_bw[0]>session.grabber.frames)
      levels_bw[0]=session.grabber.frames;
    if(levels_bw[0]<1)
      levels_bw[0]=1;
    for(k=1;k<Levels_Count;k++){
      levels_bw[k]=levels_bw[k-1]+
	(session.local->rtp_s[k+FIRST_LEVEL].bandwidth*1000)/(frame_size*8);
      if(levels_bw[k]<1)
	levels_bw[k]=1;
    }
    frame_rate=levels_bw[Levels_Count-1];
    Rate = 1.0 /frame_rate * 1000;
    *rate=Rate;
    for(k=Levels_Count-1; k>=0;k--){
      for(l=0;l<frame_rate;){
	levels[l]=k+1;
	l+=round(frame_rate/levels_bw[k]);
      }
    }
    i=1;
  }
  else{
    ++i;
    *rate=Rate;
    return (levels[i-1]);
  }
  return(levels[0]);
  
}



/****************************************************************************/
/*
 * Timer handler: 
 * call transmit_frame function and schedule next timer event
 * according to the actual frame rate
 *
 */
static int transmit_handler(ClientData level)
{
  static int first[] = {1,1,1,1,1,1,1,1,1,1};

  if ((int)level <=0) {   
    if(level==0){
      int i;
      for(i=FIRST_LEVEL;i<=Levels_Count;i++){
	/* stop timer */
	Tk_DeleteTimerHandler(Data_TimerToken[i]);
	first[i] = 1;
      }
      return 1;
    }
    /* stop timer */
    Tk_DeleteTimerHandler(Data_TimerToken[abs((int)level)]);
    first[abs((int)level)] = 1;
    return 1;
  }
  if (first[(int)level]) {
    /* initialize the actual frame rate with the maximum frame rate */
    session.grabber.act_frame_rate = session.grabber.frames;
    first[(int)level] =0;
  }
  calc_time_interval((int)level);/* start the event handling for the session*/
  transmit_frame((int)level);
  return 1;
}


/****************************************************************************/
/* calc_time_interval
 * 
 * calculate the time intervals between two sent frames for each level
 * and start their event handlers with those intervals
 */
int calc_time_interval(int level)
{
  int rate;
  /* Hack: to take account of calling the event, doing some initialization
   * and some yet unknown bugs set the rate a little bit higher than 
   * specified -here it is set to intervals shorter of 3 ms */
  level=calc_level(session.stats.frame_size,&rate);
  Data_TimerToken[level]= 
    Tk_CreateTimerHandler((int) rate-3 ,(Tk_TimerProc *)transmit_handler,
			  (ClientData) level);
  return 1;
}



/****************************************************************************/
/* 
 *  sends statistcs for the cname specified to the conference bus.
 *
 *  Usage: stats [cname]
 */
int stats_cmd(ClientData clientData, Tcl_Interp *interp, 
		     int argc, char *argv[])
{
  Tcl_HashEntry *entryPtr;
  Tcl_HashSearch search; 
  member_t *m;
  char buf[1024];
  if (argc != 2) {
    sprintf(interp->result, "stats_cmd: wrong # of args: should be %s cname",
	    argv[0]);
    return TCL_ERROR;
  }
  /* search for cname */
  for (entryPtr = Tcl_FirstHashEntry(&s->member_table, &search); 
       entryPtr != NULL;
       entryPtr = Tcl_NextHashEntry(&search)) {
    m = (member_t *) Tcl_GetHashValue(entryPtr);
    if(m->cname) {
      struct timeval now;           /* current time */
      gettimeofday(&now,0);
      if (!strcmp(argv[1], m->cname)) {
      if (m->ssrc == s->local->ssrc) {  
	int i;
	session.stats.bw=session.stats.frame_rate=0;
	for(i=FIRST_LEVEL;i<=Levels_Count;i++){
	
	  /* statistics for local site */
	  if(now.tv_sec-session.local->rtp_s[i].update_time.tv_sec>1){
	    /* session.stats.bw+=
	       (double)(8*(session.local->rtp_s[i].octets_sent -
	       session.local->rtp_s[i].octets_last))/
	       (double) (1024*(now.tv_sec-
	       session.local->rtp_s[i].update_time.tv_sec));*/
	    session.stats.bw+=(double)(8*session.local->rtp_s[i].octets_last)/
	                          (double) (1024*(now.tv_sec-
			          session.local->rtp_s[i].update_time.tv_sec));
	    session.stats.frame_rate+=(double) 
	      (session.local->rtp_s[i].packets_last)/
	      (double)(now.tv_sec-
		       session.local->rtp_s[i].update_time.tv_sec);
	    session.local->rtp_s[i].frames_sent=0;
	    gettimeofday(&(session.local->rtp_s[i].update_time),0);
	    session.local->rtp_s[i].packets_last=0;
	    session.local->rtp_s[i].octets_last=0;
	  }
	}
	  /* build the command for sending the pmm message containing the 
	   * updated statistics */
	sprintf(buf, 
		"control_send \"$session_name/%s statistics "
		"{ssrc %lu} {created %lu} {last_data %lu.%lu} "
		" {last_control %lu.%lu}"
		" {actual_bandwidth %f} {actual_framerate %f} \" ",
		m->cname, 
		(unsigned long)m->ssrc,
		m->rtp_s[FIRST_LEVEL].created,
		m->rtp_s[FIRST_LEVEL].last[0].tv_sec,m->rtp_s[FIRST_LEVEL].last[0].tv_usec,
		m->rtp_s[FIRST_LEVEL].last[1].tv_sec,m->rtp_s[FIRST_LEVEL].last[1].tv_usec,
		session.stats.bw,
		session.stats.frame_rate);
	/*	fprintf(stderr," %s \n" ,buf);*/
	session.stats.bw=0;
	session.stats.frame_rate=0;
      } else {
	double bw=0, framerate=0,rr_loss_fraction=0,loss_fraction=0;
	double rr_lost=0, lost=0;  
	/* set the minimal update time to 1 sec, we might get into touble if 
	   it was set to 0 for example */
	if(now.tv_sec - m->rtp_s[FIRST_LEVEL].update_time.tv_sec>0.001){
	  int level;
	  for(level=1;level<=Levels_Count;level++){
	    rtp_calc_loss(&(m->rtp_s[level]));
	    lost+=m->rtp_s[level].lost;
	    rr_lost+=m->rtp_s[level].rr_lost;
	    loss_fraction+=(double)(m->rtp_s[level].fraction)/(double)256;
	    rr_loss_fraction+=(double)(m->rtp_s[level].rr_fraction)/(double)256;

	    /* statistics for remote site */
	    bw+=(double) (8*m->rtp_s[level].octets_sent)/
	      (double)(1024*(now.tv_sec-m->rtp_s[level].update_time.tv_sec));
	    m->rtp_s[level].octets_sent=0;
	    m->rtp_s[level].octets_last=m->rtp_s[level].octets_sent;
	    framerate+=(double) (m->rtp_s[level].frames_sent)/
	      (double)(now.tv_sec-m->rtp_s[level].update_time.tv_sec);
	    
	    
	    m->rtp_s[level].frames_sent=0;
	    m->rtp_s[level].packets_last=m->rtp_s[level].frames_sent;
	    gettimeofday(&(m->rtp_s[level].update_time),0);
	  }
	  /* build the command for sending the pmm message containing the 
	   * updated statistics */
	  
	  sprintf ( buf, 
	     "control_send \"$session_name/%s statistics "
	     "{ssrc %lu} {created %lu} {last_data %lu} "
	     " {last_control %lu} "
             " {lost %lu} {jitter %lu} "
	     " {rr_fraction %f} {received %li} {rr_lost %lu} {rr_jitter %lu} "
	     " {actual_bandwidth %f} {actual_framerate %f} "
	     "  {duplicate %lu} \"",
	     m->cname, 
	     m->ssrc,
	     m->rtp_s[FIRST_LEVEL].created,
	     m->rtp_s[FIRST_LEVEL].last[0].tv_sec, 
	     m->rtp_s[FIRST_LEVEL].last[1].tv_sec,
	     m->rtp_s[FIRST_LEVEL].lost,
	     (u_int32) m->rtp_s[FIRST_LEVEL].jitter,
	     rr_loss_fraction,
	     m->rtp_s[FIRST_LEVEL].received,
	     m->rtp_s[FIRST_LEVEL].rr_lost,
	     (u_int32) m->rtp_s[FIRST_LEVEL].rr_jitter,
	      bw,framerate, m->rtp_s[FIRST_LEVEL].duplicate);
	  m->rtp_s[FIRST_LEVEL].duplicate=0;
	  
	  /*fprintf(stderr," %s \n",buf); */
	}
      }
      if (Tcl_GlobalEval(interp, buf) != TCL_OK) {
	printf("error: %s\n", interp->result);
      }
      return TCL_OK;
    } /* if strcmp */
    }/* if cname */
  } /* for */
  /* entry not found */
  return TCL_ERROR;
}

/****************************************************************************/
/* 
 * open the rtp and RTCP connections 
 */

int open_connection(int level, int port, int ttl, char *addr)
{
  char *local;

  if((local=malloc(100))==NULL)
    message_output("NOt Enough Memory",1);
  if (session.net.connected[level]) {
    return 0;
  }
  my_addr.sin_port=port;
  local=inet_ntoa(my_addr.sin_addr);
  /* 
   *  we save one memcpy from the grabber buffer if scattered write
   *  operations exist (no need to place the packet header in front
   *  of the buffer)
   */
  if(session.net.connect((char *)&session, RTCP+level-1, addr, local, port + 1,
			 ttl, 1,RTCP_receive)==NULL) {
    perror(" could not open RTCP socket ");
    Exit();
  }
  if((rmt_addr[level]=(struct sockaddr_in*) 
      session.net.connect((char *)&session,RTP+level-1,addr,
			  local,port,ttl,1,rtp_handler))==NULL){
    perror(" could not open RTP socket");
    Exit();
  }
  session.net.connected[level] = 1;
  rtcp_timer_handler((ClientData) level);     /* start RTCP timer */
  if(level>FIRST_LEVEL) /* we have already initialized the session structure 
			   for the basis layer in the main part */
    RTP_init(&session,level);
  return 1;
}
  
/****************************************************************************/
/* Determine the type of the media -video or picture*/
/****************************************************************************/

int media_type_cmd(ClientData clientData, Tcl_Interp *interp,
  int argc, char **argv)
{
#ifdef XIL
    Tcl_SetResult(interp,"video", TCL_STATIC);
#else
    Tcl_SetResult(interp,"picture", TCL_STATIC);
#endif
  
  return TCL_OK;
}

/****************************************************************************/
/* close the connection */
/****************************************************************************/

int nevit_leave_cmd(ClientData clientData, Tcl_Interp *interp,
  int argc, char **argv)
{
  fprintf(stderr," Calling Exit from nevit_leave_cmd\n");
  Exit();
  return TCL_OK;
}



/****************************************************************************/
/* 
 *  invoked after receiving a PMM open command. Creates the actual
 *  network connections and the first timer event.
 *  Usage: nevit_open host port ttl
 */
int nevit_open_cmd(ClientData clientData, Tcl_Interp *interp,
  int argc, char **argv)
{
  int port, ttl;
  extern int rtcp_timer_handler(ClientData client);
  char *local;
  if((local=malloc(100))==NULL)
    message_output("NOt Enough Memory",1);

  if (session.net.connected[FIRST_LEVEL]) {
    sprintf(interp->result, "%s S error: session already open",
	    argv[0]);
    return TCL_ERROR;
  }
  if (argc != 4) {
    sprintf(interp->result, "nevit_open_cmd: wrong # of args: should be %s host port ttl",argv[0]);
    return TCL_ERROR;
  }
  port = atoi(argv[2]);
  ttl  = atoi(argv[3]);
  my_addr.sin_port=port;
  local=inet_ntoa(my_addr.sin_addr);
  Levels_Count+=open_connection(FIRST_LEVEL,port,ttl,argv[1]);
  if(session.send[FIRST_LEVEL]) {
    transmit_handler((ClientData )FIRST_LEVEL);/*create first event for 
						 session 1*/
  }
  return TCL_OK;
}

/****************************************************************************/
/* 
 *  invoked after receiving a PMM open command. Creates the actual
 *  network connections and the first timer event.
 *  Usage: nevit_open host port ttl
 */

/* get address of local host*/
int nevit_local_cmd(ClientData clientData, Tcl_Interp *interp,
		    int argc, char **argv)
{
  get_myaddr(&my_addr,argv[1]);
  return TCL_OK;
}


/****************************************************************************/
/* invoked after receiving a PMM close command */
int nevit_close_cmd(ClientData clientData, Tcl_Interp *interp,
  int argc, char **argv)
{
  if (argc != 1) {
    sprintf(interp->result, "nevit_close_cmd: wrong # of Args: should be %s",
	    argv[0]);
    return TCL_ERROR;
  }
  Exit();
  return TCL_OK;
  
}

/****************************************************************************/
/*
 * Utility function to store a string at a target pointer. 
 */
void set_string(char *target, char *source)
{
  int len;
  
  len = strlen(source);
  /*free(target);*/
  if (len == 0)
    target = NULL;
  else {
    if((target = malloc(len + 1))==NULL)
          message_output("NOt Enough Memory",1);

    strcpy(target, source);
  }
}



/****************************************************************************/
/* put a message on the screen with 
 * the button Exit Ignore for param 1
 * the button Exit        for param 0
 */
void message_output (char * message,int param)
{

  fprintf(stderr,"----------------------------------------------\n");
  fprintf(stderr,"        %s \n", message);
  fprintf(stderr,"----------------------------------------------\n");
  /* if (param){
  sprintf(command,"set b [tk_dialog .tkerrorDialog \"Warning in NeViT\" \"%s\" warning 0 \"Ignore\" \"Exit\"];if { $b==1} nevit_leave ; if { $b==0} return ;",message);
  }
  else
    sprintf(command,"set b [tk_dialog .tkerrorDialog \"Warning in NeViT\" \"%s\" warning 0 \"Exit\"];nevit_leave ;",message );
    
  Tcl_Eval(interp,command);*/
}

/****************************************************************************/
/* 
 * interprete PMM configure session messages
 * 
 * Usage:
 *   nevit_config parameter value 
 *
 *  make sure that this function corresponds with the tcl session procedure
 */
int nevit_config_cmd(ClientData clientData, Tcl_Interp *interp,
  int argc, char **argv)
{
  if (!strcasecmp(argv[1], "media")) {
    if (!strcasecmp(argv[2], "JPEG")) {
      if(s->codec.quality>100||s->codec.quality<=0)
	s->codec.quality=50;      /* set a default value for quality */
      if(!JPEG_init(&session))  /* already JPEG, so do nothing */
	return 1;
      s->local->init_decompression=Init_JpegDecoder;
      s->local->delete_decompression=Delete_JpegDecoder;
      s->local->decompress=JpegDecoder_411;
      s->local->redecode=Jpeg_ReInit;
      s->local->alloc_colors=Jpeg_AllocColors;
      s->local->draw=drawWindow;
      s->local->xil_stuff=ignore_XilStuff;
	
    } else if (!strcasecmp(argv[2], "YUV")) {
      if(!YUV_init(&session)){ /* already YUV, so do nothing */
	return 1;
      }
#ifdef XIL
      s->local->init_decompression=Init_YUVDecoder;
      s->local->decompress=YUVDecoder;
      s->local->redecode=YUV_ReInit;
      s->local->alloc_colors=YUV_AllocColors;
      s->local->draw=YUV_draw;
      s->local->xil_stuff=do_XilStuff;
#endif
    } else  {
      sprintf(interp->result, 
	      "encoding %s not supported. should be JPEG/YUV \nusing default JPEG",argv[2]);
      s->codec.quality=50;      /* set a default value for quality */
      if(!JPEG_init(&session))  /* already JPEG, so do nothing */
	return 1;
      s->local->init_decompression=Init_JpegDecoder;
      s->local->decompress=JpegDecoder_411;
      s->local->redecode=Jpeg_ReInit;
      s->local->alloc_colors=Jpeg_AllocColors;
      s->local->draw=drawWindow;
      s->local->xil_stuff=ignore_XilStuff;
      return TCL_ERROR;
    }
#ifdef XIL
    if(s->send[FIRST_LEVEL]){
      xil_end_grab();      
      xil_init(&session);
    }
#endif
    if(s->local->decoder!=NULL){
      free(s->local->decoder);
      s->local->decoder=NULL;
      destroy_display();
      s->local->decoder=init_stream(s->local->decoder,s->local);
    }
  } /* media */
  /******************************************************/
  /* add or modify new video transmission layers */
  else if (!strcasecmp(argv[1], "Levels")) {
    char **list;
    int num;

    Tcl_SplitList(interp,argv[2],&num,&list);
    if (!strcasecmp(list[1], "remove")) {
      Levels_Count-=Exit_Level(atoi(list[0])+FIRST_LEVEL);
      session.local->rtp_s[atoi(list[0])+FIRST_LEVEL].bandwidth=0;
    }
    else {
      int i=1;
      char addr[100];
      int port=0;
      int ttl=0;
      for(;i<num;i+=2){
	if(!strcasecmp(list[i], "host")){
	  strcpy(addr,list[i+1]);
	} else if(!strcasecmp(list[i], "port")){
	  port=atoi(list[i+1]);
	} else if(!strcasecmp(list[i], "ttl")){
	  ttl=atoi(list[i+1]);
	} else if(!strcasecmp(list[i], "bandwidth")){
	  int bandwidth;
	  bandwidth=atoi(list[i+1]);
	  session.local->rtp_s[atoi(list[0])+FIRST_LEVEL].bandwidth=bandwidth;
	}   
      }
      Levels_Count+=open_connection(atoi(list[0])+FIRST_LEVEL,port,ttl,addr);
    }
    /* Tcl_Free((char *) list);*/
    
  }/* Levels */
  
  /******************************************************/
  else if (!strcasecmp(argv[1], "network")) {
    if(!strcasecmp(argv[2], "ATM")) { /* use native ATM */
      session.net.scattered_write_possible = 0; /* as the ATM send procedures 
						 * do not support scattered 
						 * writing one more copy
						 * opertaion will be required*/
      session.net.network=1;
#ifdef ATM
      ATM_init(&session);
#else
      perror(" ATM not supported on this host please restart the application with an IP address");
      Exit();
#endif
    }
    else {
      session.net.scattered_write_possible = 1;
      session.net.network=1;
      UDP_init(&session);
    }
  } /* network */

  /******************************************************/
  else if (!strcasecmp(argv[1], "peak_bandwidth_minimum")) {
#ifdef ATM
    ATM_qos_init(PEAK_BANDWIDTH_MINIMUM,argv[2]);
#endif
  } /*  peak_bandwidth_minimum*/
  /******************************************************/
  else if (!strcasecmp(argv[1], "peak_bandwidth_target")) {
#ifdef ATM
    ATM_qos_init(PEAK_BANDWIDTH_TARGET,argv[2]);
#endif
  } /* peak_bandwidth_target */
  /******************************************************/
  else if (!strcasecmp(argv[1], "mean_bandwidth_minimum")) {
#ifdef ATM
    ATM_qos_init(MEAN_BANDWIDTH_MINIMUM, argv[2]);
#endif
  } /* mean_bandwidth_minimum */
  /******************************************************/
  else if (!strcasecmp(argv[1], "mean_bandwidth_target")) {
#ifdef ATM
    ATM_qos_init(MEAN_BANDWIDTH_TARGET, argv[2]);
#endif
  } /* mean_bandwidth_target */
  /******************************************************/
  else if (!strcasecmp(argv[1], "mean_burst_minimum")) {
#ifdef ATM
    ATM_qos_init(MEAN_BURST_MINIMUM, argv[2]);
#endif
  } /* mean_burst_minimum */
  /******************************************************/
  else if (!strcasecmp(argv[1], "mean_burst_target")) {
#ifdef ATM
    ATM_qos_init(MEAN_BURST_TARGET, argv[2]);
#endif
  } /*mean_burst_target */
  /******************************************************/
  else if (!strcasecmp(argv[1], "device")) {
#ifdef ATM
    ATM_qos_init(DEVICE,argv[2]);
#endif
  } /* device*/
  /******************************************************/
  else if (!strcasecmp(argv[1], "max_bandwidth")) {
    ac_update( MAX_BANDWIDTH, atoi(argv[2]));
    ac_update( MIN_BANDWIDTH, 10);
    session.grabber.bw = atoi(argv[2]);
    if(Levels_Count<=FIRST_LEVEL) /*we are only using one level so give the 
				    basis level all the available bandwidth */ 
       session.local->rtp_s[FIRST_LEVEL].bandwidth=atoi(argv[2]);
  } /* max_bandwidth */
  /******************************************************/
  else if (!strcasecmp(argv[1], "max_frame_rate")) {
    s->grabber.frames  = atoi(argv[2]);
  } /* max_frame_rate */
  /******************************************************/
  else if (!strcasecmp(argv[1], "width")) {
    /*s->local->width  = atoi(argv[2]);*/
  } /* width */
  /******************************************************/
  else if (!strcasecmp(argv[1], "height")) {
    /*s->local->height = atoi(argv[2]);*/
  } /* height */
  /******************************************************/
  else if (!strcasecmp(argv[1], "file")) { /* which file to open */
    if(picture==NULL|| strcmp(picture,argv[2])){ /* consider the filename only
						  * when it has changed */
      if((picture=malloc(strlen(argv[2])))==NULL)
	    message_output("NOt Enough Memory",1);

      strcpy(picture,argv[2]);
    }
  } /* file */
  /******************************************************/
#ifdef XIL
  /******************************************************/
  else if (!strcasecmp(argv[1], "quality")) { /* q factor for jpeg */
    session.codec.quality = atoi(argv[2]);
    if(s->local->payload_type==RTP_PT_JPEG)
      setQ(session.codec.quality);
  } /* quality */
  /******************************************************/
  else if (!strcasecmp(argv[1], "video_input")) {
    if(s->grabber.whichinput!=atoi(argv[2])){
      s->grabber.whichinput=atoi(argv[2]);
      if(s->send[FIRST_LEVEL]){
	xil_open_input(atoi(argv[2]));
	xil_init(&session);
	restart_TKStuff();
      }
    }
     
  } /* video_input */
  /******************************************************/
#endif
  /* ac related parameters */
  else if (!strcasecmp(argv[1], "control")) {
    ac_update( CONTROL, atoi(argv[2]));
    session.control= atoi(argv[2]);
    if(atoi(argv[2])>0) {
      ac_init(session.local->ssrc);
    }
    else {
      ac_delete_entry(-1);
    }
  }
  else if (!strcasecmp(argv[1], "alpha")) {
    ac_update( ALPHA, atoi(argv[2]));
  }
  else if (!strcasecmp(argv[1], "multiplicative_decrease")) {
    ac_update( MULTIPLICATIVE_DECREASE, atoi(argv[2]));
  }
  else if (!strcasecmp(argv[1], "loss_upper_threshold")) {
    ac_update( LOSS_UPPER_THRESHOLD, atoi(argv[2]));
  }
  else if (!strcasecmp(argv[1], "additive_increase")) {
    ac_update( ADDITATIVE_DECREASE, atoi(argv[2]));
  }
  else if (!strcasecmp(argv[1], "loss_lower_threshold")) {
    ac_update(LOSS_LOWER_THRESHOLD, atoi(argv[2]));
  }
  else if (!strcasecmp(argv[1], "congested_recv_fraction")) {
    ac_update(CONGESTED_RECV_FRACTION, atoi(argv[2]));
  }
  else if (!strcasecmp(argv[1], "loaded_recv_fraction")) {
    ac_update(LOADED_RECV_FRACTION, atoi(argv[2]));
  }
  else if (!strcasecmp(argv[1], "algorithm")) {
    ac_update(ALGORITHM, atoi(argv[2]));
  }
  /* end ac related parameters */
  /******************************************************/
  else if (!strcasecmp(argv[1], "receive")) {
    Tcl_HashEntry *entryPtr;
    Tcl_HashSearch search;
    member_t *m=NULL;
    if(argc>3) {

      for (entryPtr = Tcl_FirstHashEntry(&s->member_table, &search); 
	   entryPtr != NULL;
	   entryPtr = Tcl_NextHashEntry(&search)) {
	m = (member_t *) Tcl_GetHashValue(entryPtr);
	if(m->cname) {
	  if(!strcmp(argv[2],m->cname)) {
	    if((atoi(argv[3])>0)&&(argc>=4)) { /* display image? if a thumbnail image is given */
	      if(m->width!=0) { /* dimensions of stream already known? */
		if(strtoul(argv[4],NULL,16)>0)
		  m->thumbnail_window=strtoul(argv[4],NULL,16);
		if(init_thumbnail(m,m->thumbnail_window)==NULL) {
		  perror("could not initialize thumbnail image ");
		} 
	      }
	     /* dimensions not known so delay initialization to rtp_handle.c */
	      else {
		m->rtp_s[FIRST_LEVEL].thumbnail=1;
		m->thumbnail_window=strtoul(argv[4],NULL,16);
	      }	
	    }
	    else { /* release image */
	      if(m->theImage!=NULL)  /* a big image is displayed */
		receive_display(s,argv[2],atoi(argv[3]));
	      delete_thumbnail(m);/*release the resources of thumb image*/
	    }
	  }
	} 
      }/* end for*/
    }
  } /* receive */
  /******************************************************/
  else if (!strcasecmp(argv[1], "enlarge")) {
    member_t *m=NULL;
    if(argc>3) {
       m=receive_display(s,argv[2],atoi(argv[3]));
      if((m!=NULL)&&(atoi((char *)m)==-1)){
	perror(" receiver initialization error :receive");
	Exit();
      }
    }
  } /* receive */

  /******************************************************/
  else if   (!strcasecmp(argv[1], "grab")) { /* grab data */ 
    x11_grab("testing",s->local->ssrc);
  }

  /******************************************************/
  else if (!strcasecmp(argv[1], "send")) { /* send data */
    static int first=1;
    session.send[FIRST_LEVEL] = atoi(argv[2]);
    if(argc>=4 )
      if(strtoul(argv[3],NULL,16)>0)
	s->local->thumbnail_window=strtoul(argv[3],NULL,16);
    if (atoi(argv[2])) {
      if(first) {
	first=0;
#ifdef XIL    /* use the usual nevit */
	if(xil_init(&session)==0) { /*initialise the xil library */
	  session.send[FIRST_LEVEL]=0;
	  first =1;
	  return 1;
	}
#else         /* use nevit.simple */
	open_File();
#endif
	

      }
      if(session.net.connected[FIRST_LEVEL]) {
	if(init_thumbnail(s->local,s->local->thumbnail_window)==NULL) {
	  perror("could not initialize thumbnail image ");
	} 
	transmit_handler(0);   /* stop the timer of the old rate */
	transmit_handler((ClientData) FIRST_LEVEL);/* create first 
						      transmit_frame event */
      }
    } else {
      delete_thumbnail(s->local); /* release the resources of the thumb image*/
      transmit_handler(0);       /* stop timer */
      if(s->local->theImage!=NULL) { /* a big image is displayed */
	destroy_display();
	session.local->preview=-1;
      }
      /* release the video card */
#ifdef XIL
      xil_end_grab();
      first=1;
#endif
    }
  } /* send */
  /******************************************************/
  else if (!strcasecmp(argv[1], "preview")) { /* display video stream*/
   member_t *m;
   if(s->local->width) { /* width and height video card are initialized??
			    as currently the system takes the appropriate
			    diplay dimensions from the card, the sender has
			    to start sending data before being able to 
			    view local data */
       if(s->local->preview!= atoi(argv[2])) {
	 m=receive_display(s,s->local->cname,atoi(argv[2]));
	 if((m!=NULL)&&(atoi((char *)m)==-1)) {
	   perror(" receiver initialization error: preview");
	   Exit();
	 }
       }
   }
   else {
	s->local->preview=atoi(argv[2]);  /* remember the value of preview
					    so we can start the necessary
					    actions just after initializing the
					    display dimensions
					  */
   }
  } /* preview */
  /******************************************************/
  else if (!strcasecmp(argv[1], "debug")) {
    int i, mask = 0;
    for (i = 2; i < argc; i++) {  
      if (!strcasecmp(argv[i], "RTP")) mask |= DEBUG_RTP;
      if (!strcasecmp(argv[i], "RTCP")) mask |= DEBUG_RTCP;
      if (!strcasecmp(argv[i], "PMM")) mask |= DEBUG_PMM;
      if (!strcasecmp(argv[i], "TRANSMIT")) mask |= DEBUG_TRANSMIT;
      if (!strcasecmp(argv[i], "STATS")) mask |= DEBUG_STATS;
    }
    debug_mask(mask);
  } /* debug */
  /******************************************************/
  /* identification parameters */
  else if (!strcasecmp(argv[1], "cname"))
    set_string(s->local->cname, argv[2]);
  else if (!strcasecmp(argv[1], "name"))
    set_string(s->local->name, argv[2]);
  else if (!strcasecmp(argv[1], "email"))
    set_string(s->local->email, argv[2]);
  else if (!strcasecmp(argv[1], "loc"))
    set_string(s->local->loc, argv[2]);
  else if (!strcasecmp(argv[1], "phone"))
    set_string(s->local->phone, argv[2]);
  else if (!strcasecmp(argv[1], "tool"))
    set_string(s->local->tool, argv[2]);
  else if (!strcasecmp(argv[1], "note"))
    set_string(s->local->note, argv[2]);
  /* sdes items */
  
  /* all other messages are ignored  */
  return TCL_OK;
}



/****************************************************************************/
#include "nevit.c"

extern int Tcl_idstring(ClientData clientData, Tcl_Interp *interp,
			int argc, char **argv);


void Event_Proc_main(ClientData data,XEvent *eventPtr)
{
  if(eventPtr->type == DestroyNotify ) {
    raise(SIGINT);
  }
  if(eventPtr->type == ConfigureNotify ) {
  }
}
    

/****************************************************************************/
int ARGC;
char **ARGV;

int Tcl_AppInit(Tcl_Interp *interp)
{
 
  Et_Init(&ARGC,ARGV);
  interp_ = Et_Interp;     /* set global pointer to tcl interpreter */
  
  
  /* set up commands */
  
  Tcl_CreateCommand(interp_, "media_type", media_type_cmd, 0, 0);
  Tcl_CreateCommand(interp_, "idstring", Tcl_idstring, 0, 0);
  Tcl_CreateCommand(interp_, "control_send", control_send_cmd, 0, 0);
  Tcl_CreateCommand(interp_, "stats", stats_cmd, 0, 0);
  Tcl_CreateCommand(interp_, "nevit_open", nevit_open_cmd, 0, 0);
  Tcl_CreateCommand(interp_, "nevit_local", nevit_local_cmd, 0, 0);
  Tcl_CreateCommand(interp_, "nevit_close", nevit_close_cmd, 0, 0);
  Tcl_CreateCommand(interp_, "nevit_config", nevit_config_cmd, 0, 0);
  Tcl_CreateCommand(interp_, "nevit_leave", nevit_leave_cmd, 0, 0);

  Tcl_SetVar(interp_, "cname", s->local->cname, TCL_GLOBAL_ONLY);
  /*
   * open UDP connection for the session controller
   * the function pmm_receive is called if something is
   * received on this multicast address
   */
  if(session.net.connect((char *)&session,PMM,"225.2.0.1",NULL,5010, 0, 1, 
			 pmm_receive)==NULL){
    perror (" could not open PMM socket");
      Exit();
  }
  if(session.passive){
    /* execute inlined tcl script */
    if (Tcl_Eval(interp_, nevit_tcl) != TCL_OK) {
      fprintf(stderr, interp->result);
      Exit();
    }
  }
  else {
    open_online(ARGC,ARGV);
    parse(ARGC,ARGV);
  }
  /* start a timer that after expiring checks for the activity of all members*/
  Tcl_CreateTimerHandler((int) 3000,
      (Tcl_TimerProc *)RTCP_activity_handler,s); 

  Tk_CreateEventHandler(Tk_MainWindow(interp_),
	      ConfigureNotify|DestroyNotify, 
	      Event_Proc_main, (ClientData) -1);

  if(init_video_map()<0)
    Exit();

  while (1) { /* start event handler */
    /*
     * Tk doesn't give priority to the X server so file handlers
     * (i.e., packets from the network) can starve the window
     * system (technically, tk lets one X event through per file
     * event, but a file event can trigger more than one X event
     * so the X server loses to a high-rate video source).
     * The solution is to let the X server go first by only
     * processing file handlers when their are no pending
     * X events.
     */
    if (Tcl_DoOneEvent(TCL_WINDOW_EVENTS|TCL_IDLE_EVENTS|TCL_DONT_WAIT)==0)
      Tcl_DoOneEvent(0);
    }

  return TCL_OK;
}


/****************************************************************************/
/* signal handler for SIGINT */
void nevit_exit(int sig) 
{
  /* say goodbye */
  fprintf(stderr," calling Exit from nevit_exit \n");
  Exit();
}

/****************************************************************************/
/* leave the multicast session for one and all above multicast levels when 
 * doing hierarchical transmission 
 */
int Exit_Level(int level)
{
  if(session.net.connected[level]){
    rtcp_timer_handler((ClientData) (level*-1));
    transmit_handler((ClientData )(level*-1));
    session.net.close(RTP+level-1); 
    session.net.close(RTCP+level-1);
    session.net.connected[level] = 0;
    return 1;
  }
  return 0;
}


/****************************************************************************/
/* make a clean exit */
int Exit()
{
  int i;

  /* stop timers */
  rtcp_timer_handler(0);
  transmit_handler(0);

  /* send RTCP and PMM BYE */
  RTP_transmit_control(&session, P_traffic|P_bye, " PMM close message", 
		       FIRST_LEVEL);
  if (session.passive)
    dotcl(interp_, "control_send \"$session_name closed\"");
  /* close connections */
  for(i=FIRST_LEVEL;i<=Levels_Count;i++){
    Exit_Level(i);
  } 
#ifdef XIL
  xil_end_grab();
#endif
  
  /* delete all member entries except our own entry?? */
  delete_entry(&session, s->local->ssrc, DELETE_OTHERS);
  /* close the display for the thumbnail images */

  exit(-1);
}





/*
 * Main function
 */
void main(int argc, char *argv[])
{
  extern char *optarg;
  extern int optind;
  extern int rtcp_timer_handler(ClientData client);
  ARGC=argc;
  ARGV=argv;
  
  
  session.local=RTP_init(&session, FIRST_LEVEL);
  session.local->width=0;
  session.local->height=0;
  s->codec.compressor=-1;
  s->codec.quality=50;
  s->debug=0;
  UDP_init(&session);
  
  debug_file("");  /* default: stderr */
  debug_mask(0);
  
  
  /* install exit handler */
  signal(SIGINT, nevit_exit);


  parse_online(argc,argv);

  /* 
   * in the passive mode the session is opened by the conference 
   * controller. We ignore all command line parameters except 's'
   * and wait for a session open message. 
   */

  if (session.passive) {/* if !session.passive */

    debug(DEBUG_TRANSMIT, "Waiting for PMM messages");
    session.net.scattered_write_possible = 1;
    /*
     *  now we have to wait for a PMM open message. The 
     *  function nevit_open_cmd (called from tcl) then 
     *  opens the network connection and generates the transmit_frame
     *  event that starts sending
     */
  }
  
    
  Tk_Main(argc, argv, Tcl_AppInit);
  
  session.grabber.close();
  RTP_close(&session);
  exit(0);
}


