/*
 * rtp_handle.c ---
 *
 * Here we process the incoming data packets, call the RTP processing parts
 * initialize the decoder if necessary and hand the complete frames to the
 * display routines
 *
 *  (C) 1995 Dorgham Sisalem
 *
 *  See the copyright file for copyright information
 */

#include "signal.h"
#include "tcl.h"
#include "types.h"
#include "rtp.h"
#include "nevit.h"
#include "xil_capture.h"
#include "xv.h"
#include "vic_jpeg.h"
#include "yuv.h"

/**** definition of static variables   ***/
static u_int32 ssrc=0;
static u_int16 seq=0;

static char rcsid[]  = "$Id: rtp_handle.c,v 1.5 1998/04/20 14:16:32 dor Exp $";



/* call the decoder and display the image */

void decode(session_t* s,char* head,int level)
{
  int q=0;
  if(s->receive->decompressor==RTP_PT_JPEG) {
    jpeg_hdr_t *video_head;
    video_head= head+sizeof(rtp_hdr_t);
    if(s->receive->session_rec.encap_head.jpeg_head.q!=video_head->q){
      /* in case we only watch thumbnails */
      if(s->receive->decoder!=NULL)
	init_jpeg_q(s->receive->decoder,video_head->q);
      init_jpeg_q(s->receive->decoder_small,video_head->q);
      q=s->receive->session_rec.encap_head.jpeg_head.q=video_head->q;
    }
    if((video_head->width<<3)!=s->receive->width){
      s->receive->width=video_head->width<<3;
      s->receive->height=video_head->height<<3;
      restart_TKStuff();
      delete_thumbnail(s->receive);
      if(init_thumbnail(s->receive,s->receive->thumbnail_window)==NULL) {
	perror("could not initialize thumbnail image ");
      } 
    }
  }
  if(s->receive->decompressor==RTP_PT_YUV) {
    yuv_hdr_t *video_head;
    video_head= head+sizeof(rtp_hdr_t);
    if((video_head->width)!=s->receive->width){
      s->receive->width=video_head->width;
      s->receive->height=video_head->height;
      restart_TKStuff();
      if(init_thumbnail(s->receive,s->receive->thumbnail_window)==NULL) {
	perror("could not initialize thumbnail image ");
      } 
    }
  }
  if(s->receive->preview==1) {
    s->receive->decompress(s->receive->rtp_s[level].buf,
			   s->receive->rtp_s[level].nbytes, q,
			   s->receive->decoder,(char *)s->receive->theImage);
    s->receive->draw(0,0, s->receive->width, s->receive->height,
		     (char *)s->receive->theImage,(char *)s->receive->theDisp,
		     (char *)s->receive->theGC,(char *)s->receive->mainW,
		     (char *)s->receive->cname,(char *)s->receive->shminfo);
  }
  if(s->receive->rtp_s[level].thumbnail==1){
    write_thumbnail(s->receive,s->receive->rtp_s[level].buf);
    s->receive->rtp_s[level].thumbnail=0;
    s->receive->token=
      Tk_CreateTimerHandler(1000,(Tk_TimerProc *)thumbnail_handler,
			    (ClientData) s->receive);
  } 
    
}



/* convert timeval to double */
static double tdbl(struct timeval *a)
{
  return a->tv_sec + a->tv_usec/1e6;
}

/*********************************************************/
/* make necessary preperation for displaying large image */
/*********************************************************/
void init_preview(member_t *m) 
{
  if(m->theImage) {
    XFlush(m->theDisp);
    XSync(m->theDisp,TRUE);
    end_TkStuff(s->receive->cname);
  }
  if((m->decoder=init_stream(m->decoder,m))==NULL)
    Exit();
}



/*
 * the main procedure for processing incoming packets 
 * RTP and RTCP packets are handed over to the RTP processing procedures
 * in rtp.c. 
 * if the data is not being displayed it is simple dropped
 */
void rtp_handler(session_t *s,char *buf,int len, int level)
{
  pack_header  header;
  unsigned int type,q=0;
  static int hdr_len;
  header.rtp_head=(rtp_hdr_t *)buf;

  /*check if we are getting gif files */
  if(gif_receive(buf,header.rtp_head,len)==1)
    return;


  /* process RTP header */
  if((s->receive=RTP_receive(s,header.rtp_head, level))==NULL)
    return ;  
  /* set the activity bit */
  s->receive->active+=1;
  type=header.rtp_head->pt;
  if(s->local->ssrc==s->receive->ssrc) /* check if it is our own data -actually
					* obsolete now as we turned the loop
					* back off for RTP/RTCP data */
    return;
  RTCP_calc_bbw(s->receive,ntohs(header.rtp_head->seq),len,level); 
  s->receive->rtp_s[level].octets_sent+=len;/*update the received bytes value*/
  if(display_ssrc(s,s->receive->ssrc)>0){
    if(type!=s->receive->decompressor) {
      switch(type) {
      case RTP_PT_YUV:/*YUV is currently not supported yet but hopefully soon*/
	{
	  yuv_hdr_t *video_head;
	  video_head= buf+sizeof(rtp_hdr_t);
	  s->receive->decompressor=RTP_PT_YUV;
	  s->receive->width=video_head->width;
	  s->receive->height=video_head->height;
#ifdef XIL
	  hdr_len=sizeof(rtp_hdr_t)+sizeof(yuv_hdr_t);
	  seq=ntohs(header.rtp_head->seq);
	  ssrc=ntohl(header.rtp_head->ssrc);
	  s->receive->init_decompression=Init_YUVDecoder;
	  s->receive->decompress=YUVDecoder;
	  s->receive->redecode=YUV_ReInit;
	  s->receive->alloc_colors=YUV_AllocColors;
	  s->receive->draw=YUV_draw;
	  s->receive->xil_stuff=do_XilStuff;
#endif
 	  break;
	}
      case RTP_PT_JPEG:
	{
	  jpeg_hdr_t *video_head;
	  video_head= buf+sizeof(rtp_hdr_t);
	  s->receive->decompressor=RTP_PT_JPEG;
	  s->receive->width=video_head->width<<3;
	  s->receive->height=video_head->height<<3;
	  /* s->receive->session_rec.encap_head.jpeg_head.mbz=
	    video_head->mbz;
	  s->receive->session_rec.encap_head.jpeg_head.off=
	    video_head->off;*/
	  s->receive->session_rec.encap_head.jpeg_head.type=
	    video_head->type;
	  q=s->receive->session_rec.encap_head.jpeg_head.q=
	    video_head->q;
	  s->receive->init_decompression=Init_JpegDecoder;
	  s->receive->decompress=JpegDecoder_411;
	  s->receive->delete_decompression=Delete_JpegDecoder;
	  s->receive->redecode=Jpeg_ReInit;
	  s->receive->alloc_colors=Jpeg_AllocColors;
	  s->receive->draw=drawWindow;
	  s->receive->xil_stuff=ignore_XilStuff;
	  hdr_len=sizeof(rtp_hdr_t)+sizeof(jpeg_hdr_t);
	  seq=ntohs(header.rtp_head->seq);
	  ssrc=ntohl(header.rtp_head->ssrc);
	  break;
	}
      default:
	fprintf(stderr," Payload type unknown ");
	return;
      }
      if(s->receive->preview==1){
	init_preview(s->receive);
      }
      if(s->receive->thumbnail_image!=NULL){/* a thumb image already exists*/
	delete_thumbnail(s->receive);/*release resources of thumb image*/
      }	
      if(init_thumbnail(s->receive,s->receive->thumbnail_window)==NULL) {
	perror("could not initialize thumbnail image ");
      } 
    }
    /* ignore duplicate packets */
/*     if(ntohs(header.rtp_head->seq)<=s->receive->rtp_s[level].seq) { */
/*       s->receive->rtp_s[level].duplicate++; */
/*       fprintf(stderr," dup =%i seq=%i\n",ntohs(header.rtp_head->seq),s->receive->rtp_s[level].seq); */
/*       return; */
/*     } */
    /* We only consider complete frames at this stage. That is, if any part of
     * of a frame got lost the entire frame is thrown away
     */
    if(ntohs(header.rtp_head->seq)!=s->receive->rtp_s[level].seq+1) {
      if(header.rtp_head->m) {
	s->receive->rtp_s[level].seq =ntohs(header.rtp_head->seq);
	s->receive->rtp_s[level].nbytes=0;
      }
      return;
    }
    s->receive->rtp_s[level].seq=ntohs(header.rtp_head->seq);
    memcpy(s->receive->rtp_s[level].buf+s->receive->rtp_s[level].nbytes,
					  buf+hdr_len,len-hdr_len);
    s->receive->rtp_s[level].nbytes+=len-hdr_len;
    if(header.rtp_head->m) { /* a complete frame was seen */
      decode(s,buf,level);     /* decode the frame and display it */
      s->receive->rtp_s[level].frames_sent++;
      s->receive->rtp_s[level].nbytes=0;
    }
  }
  else {
    if(header.rtp_head->m) 
      s->receive->rtp_s[level].frames_sent++;
  }
}


