# nevit.tcl --
#
#    pmm related tcl-procedures for the sender 
# 
# (C) 1995 Bernd Deffner
# (c) 1996 Dorgham Sisalem
#

# build an error window 
proc control_error {arg} {
    global var
    toplevel .window 
    label .window.message -text $arg
    frame .window.frame
    button .window.frame.ok -text Continue -command {;set var 1;destroy .window}  
    button .window.frame.exit -text Exit -command {set var 1;nevit_leave}
    pack .window.frame.ok .window.frame.exit -side left \
	-padx 5 -pady 5 -ipadx 5 -ipady 5
    pack  .window.message .window.frame -side top \
	-padx 5 -pady 5 -ipadx 5 -ipady 5
}
    


# receive and process pmm message
proc control_receive {cmd} {
    global session_name
    puts stderr "received PMM msg    $cmd"
    if {[string length $cmd] > 0} {
	if {[lsearch {proc exec rename} [lindex $cmd 0]] >= 0} {
	} elseif {[lindex $cmd 1] == "no_interest"} {
	    puts stderr "[pid] $cmd"
	} else {
	    if {[lindex $cmd 0] == "session"} {
		join_session [lindex $cmd 1]
	    }  else {
		if {[catch {eval $cmd} msg]} {
		    if {[string length $cmd] > 0} {
			if {[info exists session_name]>0} {
			    if {![regexp -nocase audio  $cmd]} {
				#only audio and video should be synchronized
				return
			    }
			    #audio_video $cmd
			}
			#puts "control receive error: $msg executing $cmd"
		    }   
		}
	    }
	} 
    }
}

proc audio_video {cmd} {
    global session_name myself
    if {![regexp -nocase [lindex [split $session_name /] 0]  $cmd]} {
	return
    }
    if {[regexp -nocase send  $cmd]} {
	puts "sending "
	nevit_config send  [lindex [split $cmd] 2] 
	return
    }
    #if {[regexp -nocase active  $cmd]} {
#	set cname [lindex [split [lindex [split $cmd] 0] /] 3]
#	if { $cname == $myself } {
#	    nevit_config preview 1
#	} else {
	#    nevit_config receive $cname 1
	 #   }
#	return
 #   }
  #  if {[regexp -nocase !active  $cmd]} {
	#set cname [lindex [split [lindex [split $cmd] 0] /] 3]
	#if { $cname == $myself } {
#	    nevit_config preview -1
	#} else {
	#    nevit_config receive $cname -1
	#    }
#	return
#    }    
}

#
# join a new session, issue created and hello packet, create procedures
# for member specific pmm messages
#
proc join_session {st} {
    global session_name myself site update_int
    # session name must have the form *video* for the video application and 
    # picture for the picture  application
    if {[info exists session_name]>0} {
	    return;	
    }
    
    set type [media_type]
    if {![regexp -nocase $type $st]} {
  	return
    }
    set session_name $st
    set myself [idstring %u@%i]
    set site($myself,cname) $myself
    eval "proc $st/$myself {c args} {member $st $myself \$c \$args}"
    nevit_config cname $myself
    eval "proc $st {c args} {session $st \$c \$args}"
    control_send "$st created"
    control_send "$st hello $myself"
    # schedule the activity messages
    after $update_int update_active 
}

#
# signal the activity of the tool
#
proc update_active { } {
    global session_name update_int
    control_send "$session_name active"
    if { $update_int > 0 } {
       after $update_int update_active
    }
}
#
# ignore pmm "+" message
#
proc + {args} {
}
 
#
# ignore pmm "agent" message
#
proc agent {args} {
}



#
# process member specific pmm messages: issue statistics, set bandwidth
# and frame rate limits
#
proc member {st cname c args} {
    global myself session_name


    if {[info exists session_name]>0} {
	if {![regexp -nocase $session_name $st]} {
	    return
	}
    }
    switch $c {
	statistics? {
	    stats $cname
	}
	enlarge {
		nevit_config enlarge $cname $args
	}	    
	receive {
	    if { $cname == $myself } {
		nevit_config preview $args
	    } else {
		regsub -all "{" $args "" args
		regsub -all "}" $args "" args	
		nevit_config receive $cname [lindex $args 0] [lindex $args 1]
	    }
	}
	bye {
	    puts quit
	}
    }
}


#
# invoked when session message is received
#
# note that the default action of the switch command is to pass
# the messages directly to nevit_config
#

proc session {st c args} {
    global site myself session_name update_int

    if {![regexp -nocase $session_name $st]} {
	return
    }
    switch $c {
	
	update_interval {
	    set update_int [lindex $args 0]
	}
	cname {
	}
	note {
	    # we ignore note because a "st note {}" lets the program crash 
	}
	created {
	}
	opened {
	}
	hello {
	    set cname [lindex $args 0]
	    if {$cname != $myself} {
		eval "proc $st/$cname {c args} {member $st $cname \$c \$args}"
	    }
	}
	network {
	    nevit_config network $args
	}
	peak_bandwidth_minimum {
	    nevit_config  peak_bandwidth_minimum $args
	}
	peak_bandwidth_target {
	    nevit_config  peak_bandwidth_target $args
	}   
	mean_bandwidth_minimum {
	    nevit_config  mean_bandwidth_minimum $args
	}
	mean_bandwidth_target {
	    nevit_config  mean_bandwidth_target $args
	}   
	mean_burst_minimum {
	    nevit_config  mean_burst_minimum $args
	}
	mean_burst_target {
	    nevit_config  mean_burst_target $args
	}   
	device {
	    nevit_config  device $args
	}
	transport {
	    # ignored
	    set site($myself,transport) $args
	}
	port {
	    set site($myself,port) $args
	}
	ttl {
	    set site($myself,ttl) $args
	}
	media {
	    set site($myself,media) $args
	    nevit_config media  $args
	}
	quality {
	    nevit_config quality $args
	}
	Levels {
	    set args [string trim [lindex $args 0] ]
	    set len [llength [split $args]]
	    set i 0
	    while {$i <$len } {
		    lappend param_list [lindex [split $args] $i]
		incr i
		}
	    nevit_config Levels $args	
	}
	reserve {
	    nevit_config reserve   $args	
	}
	path {
	    nevit_config path  $args	
	}
	max_bandwidth {
	    nevit_config max_bandwidth  $args	
	}
	max_frame_rate {
	    nevit_config max_frame_rate $args	
	}
	width {
	    nevit_config width $args
	}
	height {
	    nevit_config height $args
	}
	send {
	    regsub -all "{" $args "" args
	    regsub -all "}" $args "" args	
	    nevit_config send  [lindex $args 0]	[lindex $args 1]
	    #if {[regexp -nocase audio $st]} {
		#nevit_config send $args	
	    #}
	    
	}
	video_input {
	    nevit_config video_input $args
	}
	file {
	    nevit_config file $args 
	}
	preview {
	    nevit_config preview $args
	}
	debug {
	    # args is a list in a list
	    eval nevit_config debug [lindex $args 0]
	}
	open {
	    set host $args
	    nevit_open $host $site($myself,port) $site($myself,ttl)
	    control_send "$st opened"
	}
	
	local {
	    set local $args
	    nevit_local $local
	}
	close {
	    nevit_close
	    # close connections, grabber, display window
	    control_send "$st closed"
	    exit 1;
	}
	closed {
	    # remove procedures
	    # we have to do this after receiving our own "st closed"
	    # message to avoid an error message
	    rename $st {}
	    rename $st/$myself {}
	}	
	control {
	    nevit_config control   $args
	}	
	alpha {
	    nevit_config alpha   $args
	}	
	multiplicative_decrease {
	    nevit_config multiplicative_decrease   $args
	}	
	loss_upper_threshold {
	    nevit_config loss_upper_threshold   $args
	}	
	additive_increase {
	    nevit_config additive_increase   $args
	}	
	loss_lower_threshold {
	    nevit_config loss_lower_threshold   $args
	}	
	congested_recv_fraction {
	    nevit_config congested_recv_fraction   $args
	}
	loaded_recv_fraction {
	    nevit_config loaded_recv_fraction   $args
	}
	algorithm {
	    nevit_config algorithm   $args
	}	   
	default {
	    # pass message directly to the nevit_config procedure
	    nevit_config $c [lindex $args 0]
	}
    }
}

control_send "agent nevit attached [pid]"

#
# initial value of activity inirtval reports is set to 2000, the actual value
# to be used will then be delivered by isc
# 
set update_int 2000
