/*
 * procedures for handling the initialization, display and termination of 
 * thumbnail images 
 */

#include "types.h"
#include "rtp.h"
#include "nevit.h"
extern Tcl_Interp      *interp_;

void delete_thumbnail(member_t* m)
{
  XImage *image=(XImage *)m->thumbnail_image;
  if(m->thumbnail_image!=NULL) {
    free(image->data);
    free(m->thumbnail_image);
    m->thumbnail_image=NULL;
  }
  if(m->decoder_small!=NULL){
    m->decoder_small=m->delete_decompression(m->decoder_small);
    m->decoder_small=NULL;
  }
  Tk_DeleteTimerHandler(m->token);
  m->rtp_s[FIRST_LEVEL].thumbnail=0;
	    

}
/****************************************************************************
 * init the needed parameters for the thumbnail images
 * create the ximage and initialize the colors
 * return image or NULL
 */
char* init_thumbnail_window(int width, int height)
{
  XImage *ximage=NULL;
  static int counter=0;
  char name[12];

  Tk_Window  mainWindow=init_video_map();

  sprintf(name,".container.window%i",counter++);
  /* make sure the main window exists and is mapped */
  if(!Tk_IsMapped(mainWindow))
    Tk_MakeWindowExist(mainWindow);
  Tk_MapWindow(mainWindow);
  /* initialize the image for the snapshot */
  if(ximage==NULL) {
    if((ximage= XCreateImage(Tk_Display(mainWindow), 
			    Tk_Visual(mainWindow), 
			    Tk_Depth(mainWindow), 
			    ZPixmap, 0,NULL,width,height,8,0))==NULL) {
      perror("could not create thumbnail XImage \n");
      return NULL;
    }
    if((ximage->data=(u_char *)malloc((size_t)(height*ximage->bytes_per_line)))
       ==NULL) {
      perror("could not allocate data for thumbnail XImage \n");
      return NULL;
    }
  }
  return (char *)ximage;
}

/****************************************************************************
 * init the needed paramters for decompressing an image into a thumbnail image
 ****************************************************************************/
char* init_thumbnail(member_t *m,Window win)
{
  int scale=0.25;
  Tk_Window   mainWindow=init_video_map();
  Visual *theVisual=Tk_Visual(mainWindow);
  if((m->thumbnail_image= 
      init_thumbnail_window(m->width/4,m->height/4))==NULL)
    return NULL;

  m->thumbnail_window=win;

  if(!m->decoder_small) {
    /* initialize the decoder structure*/
    m->decoder_small=m->init_decompression(m->width,m->height,
				     s->codec.quality,
				     (char *)Tk_Display(mainWindow), 
				     (int )m->mainW, 
				     (char *)theVisual );
    m->alloc_colors((char *)m->decoder_small,(char *)theVisual,
			  (char *)Tk_Display(mainWindow),scale);
  }
  m->token=  Tk_CreateTimerHandler(10,(Tk_TimerProc *)thumbnail_handler,
				(ClientData) m);
  return (char *)m;
}  
  

/****************************************************************
 * handle the event of displaying a thumbnail image 
 */
 
void thumbnail_handler(ClientData member)
{
  member_t* m=(member_t *)member;
  /* set the flag for displaying the thumbnail image for the basic level*/
  m->rtp_s[FIRST_LEVEL].thumbnail=1;
}
/****************************************************************/

void write_thumbnail(member_t* m,char* image) 
{
  Tk_Window   mainWindow=init_video_map();
  
  m->decompress(image,m->rtp_s[FIRST_LEVEL].nbytes,
		s->codec.quality,m->decoder_small,(char *)m->thumbnail_image);

  if(m->theGC==NULL){
    XGCValues v;
    u_long mask;
    v.background = 0x000;
    v.foreground = 0x999999;
    mask = GCForeground|GCBackground;
    m->theGC= Tk_GetGC(mainWindow, mask, &v);
  }

  m->draw(0,0,m->width/4,
	  m->height/4,
	  (char *)m->thumbnail_image,
	  (char *)Tk_Display(mainWindow),(char *)m->theGC,
	  (char *)m->thumbnail_window,NULL,NULL);
}
