/* 
 * vic_jpeg.c ----
 * 
 * This file handles the decompression steps of jpeg images
 *
 * Copyright (c) 1996 by Dorgham Sisalem GMD fokus all rights reserved.
 *
 * The code presented here is ported into C by the author from
 * the code developed by the University of California in C++. The code
 * is part of the vic video tool developed by the University.
 *
 * Copyright (c) 1993-1995 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Dorgham Sisalem
 *       GMD Fokus
 * 4. Neither the name of the GMD Fokus nor of the author may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */



/*
 * This code is derived from the Independent JPEG Group's JPEG software:
 *
 * Copyright (C) 1991, 1992, Thomas G. Lane.
 * This file is part of the Independent JPEG Group's software.
 * For conditions of distribution and use, see the accompanying
 * README.IJPG file.
 */

#include "vic_jpeg.h"
#include "dct.h"
#include <stdlib.h>
#include <stdio.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <string.h>
#include "color_map.h"
#include "bsd-endian.h"

#define NCC 6	/* # of components to check for CR */
#define	BMB 6	/* # of (8x8) blocks in a "macroblock" [2xY+U+V] */
#define	DCT_GRAY	(0x80 * 8)	/8 DCT gray value */

#define	EVEN(x)	((x & 1) == 0)
#define	ODD(x)	((x & 1) == 1)

static char rcsid[]  = "$Id: vic_jpeg.c,v 1.3 1998/07/28 15:52:55 dor Exp $";

/**********************VIC******************************************/

static int
mtos(int mask)
{
  int shift = 0;
  if (mask) {
    while ((mask & 1) == 0) {
      mask >>= 1;
      ++shift;
    }
  }
  return (shift);
}

config *config_;
/***********************************/

/*
 * initialise the color stuff for a 24 bit true color display
 */

int alloc_true_colors(char* c,char* the_Visual, char* the_Disp)
{
  u_int rmask, gmask, bmask,rshft, rlose, gshft, glose, bshft, blose;
  int u, v, r, b, g;
  register double uf, vf;
  Visual* theVisual;
  config_=(config *)c;
  theVisual=(Visual *)the_Visual;
  rmask = theVisual->red_mask;
  gmask = theVisual->green_mask;
  bmask = theVisual->blue_mask;
  /* XXX
   * we would expect the masks we get back from the server to
   * reflect the byte position of the color *from the server's
   * point of view* but, for a dec mips or alpha talking to
   * an sgi, the masks appear to have been byte-swapped to host
   * int form.  This is either an sgi bug, a dec bug or an
   * X protocol bug.  For right now, we put the masks in correct
   * order when a little-endian machine talks to a 24-bit
   * big-endian display.
   * Note that if this is an X protocol bug, there should be
   * an equivalent operation for big-to-little that byte swaps
   * the masks but, unfortunately, htonl() on a big-endian won't
   * do the job & there's no standard library that does.  For
   * now we ignore this problem.
   */
#if BYTE_ORDER == LITTLE_ENDIAN
  if (ImageByteOrder(theDisp) == MSBFirst) {
    rmask = htonl(rmask);
    gmask = htonl(gmask);
    bmask = htonl(bmask);
    }
#endif
  rshft = mtos(rmask);
  rlose = 8 - mtos(~(rmask >> rshft));
  gshft = mtos(gmask);
  glose = 8 - mtos(~(gmask >> gshft));
  bshft = mtos(bmask);
  blose = 8 - mtos(~(bmask >> bshft));
  
  config_->omask_ = 0x80 >> rlose << rshft;
  config_->omask_ |= 0x80 >> glose << gshft;
  config_->omask_ |= 0x80 >> blose << bshft;
  
  config_->pmask_ = 0xff >> rlose << rshft;
  config_->pmask_ |= 0xff >> glose << gshft;
  config_->pmask_ |= 0xff >> blose << bshft;
  
  for (u = 0; u < 256; ++u) {
    uf = (double)(u - 128);
    for (v = 0; v < 256; ++v) {
      vf = (double)(v - 128);
      r = (int)(vf * 1.402);
      r = (r < -128)? -128 : (r > 127)? 127 : r;
      b = (int)(uf * 1.772);
      b = (b < -128)? -128 : (b > 127)? 127 : b;
      g = (int)(uf * -0.34414 - vf * 0.71414);
      g = (g < -128)? -128 : (g > 127)? 127 : g;
      config_->uvtab_[(u << 8)|v] =
	(r & 0xff) >> rlose << rshft |
	(g & 0xff) >> glose << gshft |
	(b & 0xff) >> blose << bshft;
    }			
  }
  return (0);
}


/***********************************************************************/

/*
 * This routine sums the luma & chroma components of one pixel &
 * constructs an rgb output.  It does all three r g b components
 * in parallel.  The one complication is that it has to
 * deal with overflow (sum > 255) and underflow (sum < 0).  Underflow
 * & overflow are only possible if both terms have the same sign and
 * are indicated by the result having a different sign than the terms.
 * Note that we ignore the carry into the next byte's lsb that happens
 * on an overflow/underflow on the grounds that it's probably invisible.
 * The luma term and sum are biased by 128 so a negative number has the
 * 2^7 bit = 0.  The chroma term is not biased so a negative number has
 * the 2^7 bit = 1.  So underflow is indicated by (L & C & sum) != 0;
 */
#define ONEPIX(src, dst) { \
  l = src; \
  l |= l << 8; l |= l << 16; \
  sum = l + uv; \
  uflo = (l ^ uv) & (l ^ sum) & omask; \
  if (uflo) { \
		if ((l = uflo & l) != 0) { \
					     /* saturate overflow(s) */ \
					     l |= l >> 1; \
					     l |= l >> 2; \
					     l |= l >> 4; \
					     sum |= l; \
					     uflo &=~ l; \
		} \
		if (uflo != 0) { \
				   /* zero underflow(s) */ \
				   uflo |= uflo >> 1; \
				   uflo |= uflo >> 2; \
				   uflo |= uflo >> 4; \
				   sum &=~ uflo; \
		} \
  } \
	dst = sum & pmask; \
}

#define RV_PAST(now, ts) ((((now) - (ts)) & 0x80) == 0)

/*********************************************************/
/* dither the decoded output to be suitable for pseudo color display */
#if BYTE_ORDER == LITTLE_ENDIAN
#define PIX0(i) ((i) << 2)
#define PIX1(i) ((i) >> 6)
#define PIX2(i) ((i) >> 14)
#define PIX3(i) ((i) >> 22)
#define UV0 ((v << 7) & 0xffc00)
#define UV1 ((u << 4) & 0xffc00)
#define UV2 ((v >> 9) & 0xffc00)
#define UV3 ((u >> 12) & 0xffc00)
#else
#define PIX0(i) ((i) >> 22)
#define PIX1(i) ((i) >> 14)
#define PIX2(i) ((i) >> 6)
#define PIX3(i) ((i) << 2)
#define UV0 ((u >> 12) & 0xffc00)
#define UV1 ((v >> 9) & 0xffc00)
#define UV2 ((u << 4) & 0xffc00)
#define UV3 ((v << 7) & 0xffc00)
#endif
/*
 * dither small picture for a 8 bit graphic cards
 */

void dither_down_411(config* c, u_int off, u_int x, u_int width, u_int height,char* the_Image) 
{
  XImage*  theImage=(XImage *)the_Image;
  register u_int iw = width;
  register const u_char* yp = c->frm_ + off;
  register const u_char* up = c->frm_ + c->framesize_ + ((off - x) >> 2) + (x >> 1);
  register const u_char* vp = up + (c->framesize_ >> 2);
  register u_int doff = ((off - x) >> 4) + (x >> 2);
  register int w = width;
  register int zl = 0;
  register const u_char* dither = lut_ + zl;
  register int len = w * height >> 2;
  if ((w & 0xf) != 0 || (len & 0xf) != 0 || (doff & 3) != 0) {
    register u_short* xip = (u_short*)(theImage->data + doff);
    register int z = 0;
    for (; len > 0; len -= 8) {
      register u_int u = *(const u_int*)up;
      register u_int v = *(const u_int*)vp;
      register u_int y1;
      register u_short out = 0;
#if BYTE_ORDER == LITTLE_ENDIAN
      v = ((u & 0x00f800f8) << 5) | (v & 0x00f800f8);
#else
      u = (u & 0xf800f800) | ((v & 0xf800f800) >> 5);
#endif
      y1 = (u_int)(yp[0] & 0xfc) << 2;
      SPLICE(out, dither[y1 + UV0 + z], 8);
      ++z;
      y1 = (u_int)(yp[4] & 0xfc) << 2;
      SPLICE(out, dither[y1 + UV2 + z], 0);
      *xip++ = out;
      z = (z + 1) & 0x3;
      yp += 8;
      up += 4;
      vp += 4;
      
      w -= 8;
      if (w <= 0) {
	register int pstride,cstride;
	w = width;
	pstride = 4 * iw - w;
	cstride = iw - (w >> 1);
	yp += pstride;
	up += cstride;
	vp += cstride;
	xip += (iw - w) >> 3;
	zl = (zl + 4) & 0xf;
	dither = lut_ + zl;
	z = 0;
      }
    }
  } else {
    register u_int* xip = (u_int*)(theImage->data+ doff);
    for (; len > 0; len -= 16) {
      register u_int u = *(const u_int*)up;
      register u_int v = *(const u_int*)vp;
      register u_int y1, out = 0;
#if BYTE_ORDER == LITTLE_ENDIAN
      v = ((u & 0x00f800f8) << 5) | (v & 0x00f800f8);
#else
      u = (u & 0xf800f800) | ((v & 0xf800f800) >> 5);
#endif
      y1 = (u_int)(yp[0] & 0xfc) << 2;
      SPLICE(out, dither[y1 + UV0 + 0], 24);
      y1 = (u_int)(yp[4] & 0xfc) << 2;
      SPLICE(out, dither[y1 + UV2 + 1], 16);
      up += 4;
      vp += 4;
      u = *(const u_int*)up;
      v = *(const u_int*)vp;
#if BYTE_ORDER == LITTLE_ENDIAN
      v = ((u & 0x00f800f8) << 5) | (v & 0x00f800f8);
#else
      u = (u & 0xf800f800) | ((v & 0xf800f800) >> 5);
#endif
      y1 = (u_int)(yp[8] & 0xfc) << 2;
      SPLICE(out, dither[y1 + UV0 + 2], 8);
      y1 = (u_int)(yp[12] & 0xfc) << 2;
      SPLICE(out, dither[y1 + UV2 + 3], 0);
      *xip++ = out;
      
      yp += 16;
      up += 4;
      vp += 4;
      
      w -= 16;
      if (w <= 0) {
	register int pstride,cstride;
	w = width;
	pstride = 4 * iw - w;
	cstride = iw - (w >> 1);
	yp += pstride;
	up += cstride;
	vp += cstride;
	xip += (iw - w) >> 4;
	zl = (zl + 4) & 0xf;
	dither = lut_ + zl;
      }
    }
  }  
  



}


/*******************************************************************/
/*
 *  create a small picture for true color screens
 *
 */
/*******************************************************************/
void map_411_down(config* c,u_int off,u_int x,u_int width,
		   u_int height,char* the_Image)
{
  register u_int iw = width;
  XImage*  theImage=(XImage *)the_Image;
  register u_char* yp, *up, *vp;
  register u_int* xip = (u_int*)theImage->data + ((off - x) >> 4) + (x >> 2);
  register int w = width;
  register const u_int* yuv2rgb = c->uvtab_;
  register u_int omask = c->omask_;
  register u_int pmask = c->pmask_;
  register int len;

  yp = c->frm_ + off;
  up = c->frm_ + c->framesize_ +((off - x) >> 2) + (x >> 1);
  vp = up + (c->framesize_ >> 2);
 
  for ( len = w * height >> 2; len > 0; len -= 8) {
    register u_int l, uv;
    register u_int uflo, sum;
    
    uv = yuv2rgb[(up[0] << 8) | vp[0]];
    ONEPIX(yp[0], xip[0])
      uv = yuv2rgb[(up[2] << 8) | vp[2]];
    ONEPIX(yp[4], xip[1])
      
      xip += 2;
    yp += 8;
    up += 4;
    vp += 4;
    
    w -= 8;
    if (w <= 0) {
      register int pstride,cstride;
      w = width;
      pstride = 4 * iw - w;
      cstride = iw - (w >> 1);
      yp += pstride;
      up += cstride;
      vp += cstride;
      xip += (iw - w) >> 2;
    }
  }
}


/*******************************************************************/
/*
 *  create a large picture for true color screens
 *
 */
/*******************************************************************/
void map_411_up(config* c,u_int off,u_int x,u_int width,u_int height,
		char* the_Image)
{
  register u_int iw = width;
  XImage*  theImage=(XImage *)the_Image;
  register u_char* yp, *up, *vp;
  register u_int* xip = (u_int*)theImage->data + ((off - x) << 2) + (x << 1);
  register int w = width;
  register u_int e1, o1 ;
  register u_int* yuv2rgb = c->uvtab_;
  register u_int omask = c->omask_;
  register u_int pmask = c->pmask_;
  register int len;
  yp= c->frm_ + off;
  up= c->frm_ + c->framesize_ + ((off - x) >> 2) + (x >> 1);
  vp= up + (c->framesize_ >> 2);
  e1 = yp[0], o1 = yp[iw];

  for (len = w * height; len > 0; len -= 4) {
    register u_int l, uv;
    register u_int uflo, sum;
    register u_int e2, o2;
    register u_char* yp2 = yp + iw;
    register u_int* xip2 = xip + (iw << 1);
    register u_int* xip3 = xip2 + (iw << 1);
    register u_int* xip4 = xip3 + (iw << 1);
    
    uv = yuv2rgb[(up[0] << 8) | vp[0]];
    e2 = yp[0];
    ONEPIX((e1 + e2) >> 1, xip[0])
      xip2[0] = sum;
    ONEPIX(e2, xip[1])
      xip2[1] = sum;
    e1 = yp[1];
    ONEPIX((e1 + e2) >> 1, xip[2])
      xip2[2] = sum;
    ONEPIX(e1, xip[3])
      xip2[3] = sum;
    
    o2 = yp2[0];
    ONEPIX((o1 + o2) >> 1, xip3[0])
      xip4[0] = sum;
    ONEPIX(o2, xip3[1])
      xip4[1] = sum;
    o1 = yp2[1];
    ONEPIX((o1 + o2) >> 1, xip3[2])
      xip4[2] = sum;
    ONEPIX(o1, xip3[3])
      xip4[3] = sum;
    
    xip += 4;
    yp += 2;
    up += 1;
    vp += 1;
    
    w -= 2;
    if (w <= 0) {
      register u_int pstride, cstride;
      w = width;
      pstride = 2 * iw - w;
      cstride = (iw - w) >> 1;
      yp += pstride;
      e1 = yp[0];
      o1 = yp[iw];
      up += cstride;
      vp += cstride;
      xip += 8 * iw - 2 * w;
    }
  }
}

/*******************************************************************/
/*
 *  create a normal picture for true color screens
 *
 */
/*******************************************************************/
void map_411(config* c,u_int off,u_int x,u_int width,u_int height,
	     char* the_Image)
{
  register u_char *yp,*up,*vp;
  register u_int iw = c->width;
  XImage*  theImage=(XImage *)the_Image;
  register u_int* xip = (u_int*)theImage->data + off;
  register int w = width;
  register u_int *yuv2rgb;
  register u_int omask = c->omask_;
  register u_int pmask = c->pmask_;
  register int len;
  yp = c->frm_ + off;
  yuv2rgb=c->uvtab_;
  up = c->frm_ + c->framesize_+((off - x) >> 2)+(x >> 1);
  vp = up + (c->framesize_ >> 2);
  for (len = w * height; len > 0; len -= 8) {
    register u_int l, uv;
    register u_int uflo, sum;
    register u_int* xip2 = xip + iw;
    register u_char* yp2 = yp + iw;
#define FOUR411(n) \
    uv = yuv2rgb[(up[(n)/2] << 8) | vp[(n)/2]];\
    ONEPIX(yp[(n)], xip[(n)]) \
    ONEPIX(yp[(n)+1],xip[(n)+1]) \
    ONEPIX(yp2[(n)], xip2[(n)]) \
    ONEPIX(yp2[(n)+1],xip2[(n)+1])

    FOUR411(0)
    FOUR411(2)
    xip += 4;
    yp += 4;
    up += 2;
    vp += 2;
    w -= 4;
    if (w <= 0) {
      register int pstride, cstride;
      w = width;
      pstride = 2 * iw - w;
      cstride = (iw - w) >> 1;
      yp += pstride;
      up += cstride;
      vp += cstride;
      xip += pstride;
    }
  }
}


/****************************************************************/
/*
 * Dither function for jpeg 411 format (in large size
 *
 */
/****************************************************************/
void dither_411_up(config* c,
	   u_int off, u_int x,
	   u_int width, u_int height,char* the_Image)
{
  register u_int iw = c->width;
  XImage* theImage=(XImage *)the_Image;
  register u_char* yp = c->frm_ + off;
  register u_char* up = c->frm_ + c->framesize_ + ((off - x) >> 2) + (x >> 1);
  register u_char* vp = up + (c->framesize_ >> 2);
  register u_int* xip = (u_int*)(theImage->data + ((off - x) << 2) + (x << 1));
  register int w = width;
  register int z = 0;
  register int len;
  register u_char* dither = lut_ + z;
  register u_int e1 = (yp[0] & 0xfc) << 2, o1 = (yp[iw] & 0xfc) << 2;
  for (len = width * height; len > 0; len -= 16) {
    register u_int u = *(u_int*)up;
    register u_int v = *(u_int*)vp;
    register u_int y1, y2, e2, o2, out1, out2, p;
    register u_char* y2re;
    register u_char* y2ro;
    register u_int* xip2 = xip + (iw >> 1);
    
    y1 = (u & 0xf800f800) | ((v & 0xf800f800) >> 5);
    v = ((u & 0x00f800f8) << 5) | (v & 0x00f800f8);
    u = y1;
    
    /* double pixels 0 & 1 of the `even' row */
    y1 = *(u_int*)yp;
    out1 = 0;
    out2 = 0;
    y2re = dither + UV0;
    y2ro = y2re + 8;
    e2 = PIX0(y1) & 0x3f0;
    p = ((e1 + e2) >> 1) & 0x3f0;
    SPLICE(out1, y2re[p + 0], 24);
    SPLICE(out2, y2re[p + 4], 24);
    SPLICE(out1, y2re[e2 + 1], 16);
    SPLICE(out2, y2re[e2 + 5], 16);
    e1 = PIX1(y1) & 0x3f0;
    p = ((e1 + e2) >> 1) & 0x3f0;
    SPLICE(out1, y2re[p + 2], 8);
    SPLICE(out2, y2re[p + 6], 8);
    SPLICE(out1, y2re[e1 + 3], 0);
    SPLICE(out2, y2re[e1 + 7], 0);
    *xip = out1;
    *xip2 = out2;
    
    /* double pixels 0 & 1 of the `odd' row */
    y2 = *(u_int*)(yp + iw);
    out1 = 0;
    out2 = 0;
    o2 = PIX0(y1) & 0x3f0;
    p = ((o1 + o2) >> 1) & 0x3f0;
    SPLICE(out1, y2ro[p + 0], 24);
    SPLICE(out2, y2ro[p + 4], 24);
    SPLICE(out1, y2ro[o2 + 1], 16);
    SPLICE(out2, y2ro[o2 + 5], 16);
    o1 = PIX1(y1) & 0x3f0;
    p = ((o1 + o2) >> 1) & 0x3f0;
    SPLICE(out1, y2ro[p + 2], 8);
    SPLICE(out2, y2ro[p + 6], 8);
    SPLICE(out1, y2ro[o1 + 3], 0);
    SPLICE(out2, y2ro[o1 + 7], 0);
    xip[iw] = out1;
    xip2[iw] = out2;
    ++xip;
    ++xip2;
    
    /* double pixels 2 & 3 of the `even' row */
    out1 = 0;
    out2 = 0;
    y2re = dither + UV1;
    y2ro = y2re + 8;
    e2 = PIX2(y1) & 0x3f0;
    p = ((e1 + e2) >> 1) & 0x3f0;
    SPLICE(out1, y2re[p + 0], 24);
    SPLICE(out2, y2re[p + 4], 24);
    SPLICE(out1, y2re[e2 + 1], 16);
    SPLICE(out2, y2re[e2 + 5], 16);
    e1 = PIX3(y1) & 0x3f0;
    p = ((e1 + e2) >> 1) & 0x3f0;
    SPLICE(out1, y2re[p + 2], 8);
    SPLICE(out2, y2re[p + 6], 8);
    SPLICE(out1, y2re[e1 + 3], 0);
    SPLICE(out2, y2re[e1 + 7], 0);
    *xip = out1;
    *xip2 = out2;
    
    /* double pixels 2 & 3 of the `odd' row */
    out1 = 0;
    out2 = 0;
    o2 = PIX2(y1) & 0x3f0;
    p = ((o1 + o2) >> 1) & 0x3f0;
    SPLICE(out1, y2ro[p + 0], 24);
    SPLICE(out2, y2ro[p + 4], 24);
    SPLICE(out1, y2ro[o2 + 1], 16);
    SPLICE(out2, y2ro[o2 + 5], 16);
    o1 = PIX3(y1) & 0x3f0;
    p = ((o1 + o2) >> 1) & 0x3f0;
    SPLICE(out1, y2ro[p + 2], 8);
    SPLICE(out2, y2ro[p + 6], 8);
    SPLICE(out1, y2ro[o1 + 3], 0);
    SPLICE(out2, y2ro[o1 + 7], 0);
    xip[iw] = out1;
    xip2[iw] = out2;
    ++xip;
    ++xip2;
    
    /* double pixels 4 & 5 of the `even' row */
    yp += 4;
    y1 = *(u_int*)yp;
    out1 = 0;
    out2 = 0;
    y2re = dither + UV2;
    y2ro = y2re + 8;
    e2 = PIX0(y1) & 0x3f0;
    p = ((e1 + e2) >> 1) & 0x3f0;
    SPLICE(out1, y2re[p + 0], 24);
    SPLICE(out2, y2re[p + 4], 24);
    SPLICE(out1, y2re[e2 + 1], 16);
    SPLICE(out2, y2re[e2 + 5], 16);
    e1 = PIX1(y1) & 0x3f0;
    p = ((e1 + e2) >> 1) & 0x3f0;
    SPLICE(out1, y2re[p + 2], 8);
    SPLICE(out2, y2re[p + 6], 8);
    SPLICE(out1, y2re[e1 + 3], 0);
    SPLICE(out2, y2re[e1 + 7], 0);
    *xip = out1;
    *xip2 = out2;
    
    /* double pixels 4 & 5 of the `odd' row */
    y2 = *(u_int*)(yp + iw);
    out1 = 0;
    out2 = 0;
    o2 = PIX0(y1) & 0x3f0;
    p = ((o1 + o2) >> 1) & 0x3f0;
    SPLICE(out1, y2ro[p + 0], 24);
    SPLICE(out2, y2ro[p + 4], 24);
    SPLICE(out1, y2ro[o2 + 1], 16);
    SPLICE(out2, y2ro[o2 + 5], 16);
    o1 = PIX1(y1) & 0x3f0;
    p = ((o1 + o2) >> 1) & 0x3f0;
    SPLICE(out1, y2ro[p + 2], 8);
    SPLICE(out2, y2ro[p + 6], 8);
    SPLICE(out1, y2ro[o1 + 3], 0);
    SPLICE(out2, y2ro[o1 + 7], 0);
    xip[iw] = out1;
    xip2[iw] = out2;
    ++xip;
    ++xip2;
    
    /* double pixels 6 & 7 of the `even' row */
    out1 = 0;
    out2 = 0;
    y2re = dither + UV3;
    y2ro = y2re + 8;
    e2 = PIX2(y1) & 0x3f0;
    p = ((e1 + e2) >> 1) & 0x3f0;
    SPLICE(out1, y2re[p + 0], 24);
    SPLICE(out2, y2re[p + 4], 24);
    SPLICE(out1, y2re[e2 + 1], 16);
    SPLICE(out2, y2re[e2 + 5], 16);
    e1 = PIX3(y1) & 0x3f0;
    p = ((e1 + e2) >> 1) & 0x3f0;
    SPLICE(out1, y2re[p + 2], 8);
    SPLICE(out2, y2re[p + 6], 8);
    SPLICE(out1, y2re[e1 + 3], 0);
    SPLICE(out2, y2re[e1 + 7], 0);
    *xip = out1;
    *xip2 = out2;
    
    /* double pixels 6 & 7 of the `odd' row */
    out1 = 0;
    out2 = 0;
    o2 = PIX2(y1) & 0x3f0;
    p = ((o1 + o2) >> 1) & 0x3f0;
    SPLICE(out1, y2ro[p + 0], 24);
    SPLICE(out2, y2ro[p + 4], 24);
    SPLICE(out1, y2ro[o2 + 1], 16);
    SPLICE(out2, y2ro[o2 + 5], 16);
    o1 = PIX3(y1) & 0x3f0;
    p = ((o1 + o2) >> 1) & 0x3f0;
    SPLICE(out1, y2ro[p + 2], 8);
    SPLICE(out2, y2ro[p + 6], 8);
    SPLICE(out1, y2ro[o1 + 3], 0);
    SPLICE(out2, y2ro[o1 + 7], 0);
    xip[iw] = out1;
    xip2[iw] = out2;
    ++xip;
    ++xip2;
    
    up += 4;
    vp += 4;
    yp += 4;
    
    w -= 8;
    if (w <= 0) {
      register u_int pstride,cstride; 
      w = width;
      pstride = 2 * iw - w;
      cstride = (iw - w) >> 1;
      yp += pstride;
      e1 = (yp[0] & 0xfc) << 2;
      o1 = (yp[iw] & 0xfc) << 2;
      up += cstride;
      vp += cstride;
      xip += (8 * iw - 2 * w) >> 2;
    }
  }
}

/****************************************************************/
/*
 * Dither function for jpeg 411 format (in normal size
 *
 */
/****************************************************************/
void dither_411(config* c,
	   u_int off, u_int x,
	   u_int width, u_int height,char* the_Image)
{
  register u_int iw = c->width;
  XImage* theImage=(XImage *)the_Image;
  register   u_char* yp = c->frm_+ off;
  register   u_char* up =c->frm_+c->framesize_+((off- x)>>2)+(x>> 1);
  register   u_char* vp = up + (c->framesize_ >> 2);
  register u_int* xip = (u_int*)(theImage->data+ off);
  register int w = width;
  register int z = 0;
  register u_char* dither = lut_ + z;
  register int len;
  for (len = width * height; len > 0; len -= 16) {
    register u_int u = *(  u_int*)up;
    register u_int v = *(  u_int*)vp;
    register u_int y1, y2, out1, out2;
    register   u_char* y2r;
    register u_int* xip2 = xip + (iw >> 2);
    
    y1 = (u & 0xf800f800) | ((v & 0xf800f800) >> 5);
    v = ((u & 0x00f800f8) << 5) | (v & 0x00f800f8);
    u = y1;

    y1 = *(  u_int*)yp;
    y2 = *(  u_int*)(yp + iw);
    out1 = 0;
    out2 = 0;
    y2r = dither + UV0;
    SPLICE(out1, y2r[(PIX0(y1) & 0x3f0)+ 0], 24);
    SPLICE(out1, y2r[(PIX1(y1) & 0x3f0)+ 1], 16);
    SPLICE(out2, y2r[(PIX0(y2) & 0x3f0)+ 4], 24);
    SPLICE(out2, y2r[(PIX1(y2) & 0x3f0)+ 5], 16);
    
    y2r = dither + UV1;
    SPLICE(out1, y2r[(PIX2(y1) & 0x3f0)+ 2], 8);
    SPLICE(out1, y2r[(PIX3(y1) & 0x3f0)+ 3], 0);
    SPLICE(out2, y2r[(PIX2(y2) & 0x3f0)+ 6], 8);
    SPLICE(out2, y2r[(PIX3(y2) & 0x3f0)+ 7], 0);
    
    *xip++ = out1;
    *xip2++ = out2;
    
    yp += 4;
    y1 = *(  u_int*)yp;
    y2 = *(  u_int*)(yp + iw);
    out1 = 0;
    out2 = 0;
    y2r = dither + UV2;
    SPLICE(out1, y2r[(PIX0(y1) & 0x3f0)+ 0], 24);
    SPLICE(out1, y2r[(PIX1(y1) & 0x3f0)+ 1], 16);
    SPLICE(out2, y2r[(PIX0(y2) & 0x3f0)+ 4], 24);
    SPLICE(out2, y2r[(PIX1(y2) & 0x3f0)+ 5], 16);
    
    y2r = dither + UV3;
    SPLICE(out1, y2r[(PIX2(y1) & 0x3f0)+ 2], 8);
    SPLICE(out1, y2r[(PIX3(y1) & 0x3f0)+ 3], 0);
    SPLICE(out2, y2r[(PIX2(y2) & 0x3f0)+ 6], 8);
    SPLICE(out2, y2r[(PIX3(y2) & 0x3f0)+ 7], 0);
    
    *xip++ = out1;
    *xip2++ = out2;
    
    yp += 4;
    up += 4;
    vp += 4;
    
    w -= 8;
    if (w <= 0) {
      register int pstride, cstride;
      w = width;
      pstride = 2 * iw - w;
      cstride = (iw - w) >> 1;
      yp += pstride;
      up += cstride;
      vp += cstride;
      xip += (pstride >> 2);
      z = (z + 8) & 0xf;
      dither = lut_ + z;
    }
  }
}
/*********************************************************/

/*
 * render the output of the decoder so it is suitable for displaying 
 * on the X display
 */

void renderer(char *buf,char* c_,char* the_Image)
{
  int y,off,sx,x;
  config* c=c_;
  int wraptime = c->dec_now_ ^ 0x80;
  u_char* ts = c->rvts_;
  int k;
  for (k = c->nblk_; --k >= 0; ++ts) {
    if (*ts == wraptime)
      *ts = c->dec_now_;
  }
  c->dec_now_ = (c->dec_now_ + 1) & 0xff;
  {
    u_char* ts = c->rvts_;
    for (y = 0; y < c->height; y += 8) {
      for (x = 0; x < c->width; ) {
	if (RV_PAST(c->ren_now_, ts[x >> 3])) {
	  x += 8;
	  continue;
	}
	sx = x;
	do {
	  x += 8;
	}while(x<c->width&&!RV_PAST(c->ren_now_, ts[x >> 3]));
	
	/*XXX*/
	off = y * c->width + sx;
	c->dither((char *)c, off, sx, x - sx, 8,the_Image);
      }
      ts += c->width >> 3;
    }
    /*XXX*/
    c->ren_now_ = c->dec_now_;
	}
  c->dec_now_ = (c->dec_now_ + 1) & 0xff;
  
}

void renderer_up(char *buf,config *c,char* the_Image)
{
  int y,off,sx,x;
  int wraptime = c->dec_now_ ^ 0x80;
  u_char* ts = c->rvts_;
  int k;
  for (k = c->nblk_; --k >= 0; ++ts) {
    if (*ts == wraptime)
      *ts = c->dec_now_;
  }
  
  c->dec_now_ = (c->dec_now_ + 1) & 0xff;
  {
    u_char* ts = c->rvts_;
    for (y = 0; y < c->height; y += 8) {
      for (x = 0; x < c->width; ) {
	sx = x;
	do {
	  x += 8;
	}while(x<c->width);
	
	/*XXX*/
	off = y * c->width + sx;
	c->dither((char *)c, off, sx, x - sx, 8,the_Image);
      }
      ts += c->width >> 3;
    }
    /*XXX*/
    c->ren_now_ = c->dec_now_;
	}
  c->dec_now_ = (c->dec_now_ + 1) & 0xff;
  
}


/***************************************************/

void init_JpegDecoder_411(config* c) 
{
  int i,ns,n;
  c->cthresh_=6;
  c->color_=1;
  c->framesize_=c->width*c->height;
  
  /* initialize huffman tables */
  for (i = NUM_HUFF_TBLS; --i >= 0; ) {
    c->dcht_[i] = 0;
    c->acht_[i] = 0;
  }
  memset((char*)c->comp_, 0, sizeof(c->comp_));
  init(c);
  ns = NCC * c->width*c->height / 64;	/*# blocks*/
  if(c->cache_==NULL)
    if((c->cache_ = malloc(ns*sizeof(short)))==NULL)
       message_output("NOt Enough Memory",1);
       
  /* 
   * Initialize to some "large value" so threshold
   * is exceeded on first pass.
   */
  memset(c->cache_, 0x7f, ns * sizeof(*(c->cache_)));
  setlrskips(c);
  n = c->size_ + c->size_ / 2;
  if(c->frm_==NULL)
    if((c->frm_ = malloc(n*sizeof(u_char)))==NULL)	   /*  pixel store */
      message_output("NOt Enough Memory",1);
  
  memset(c->frm_, 0x0, n);
}


int quality_to_qfactor(int v)
{
  if (v < 1)
    v = 5000;
  else if (v < 50)
    v = 5000 / v;
  else if (v < 100)
		v = 200 - v * 2;
  else 
    v = 1;
  
  return (v);
}


/*
 * Table K.1 from JPEG spec.
 */
static   int jpeg_luma_quantizer[64] = {
  16, 11, 10, 16, 24, 40, 51, 61,
  12, 12, 14, 19, 26, 58, 60, 55,
  14, 13, 16, 24, 40, 57, 69, 56,
  14, 17, 22, 29, 51, 87, 80, 62,
  18, 22, 37, 56, 68, 109, 103, 77,
  24, 35, 55, 64, 81, 104, 113, 92,
  49, 64, 78, 87, 103, 121, 120, 101,
  72, 92, 95, 98, 112, 100, 103, 99
};
/*
 * Table K.2 from JPEG spec.
 */
static   int jpeg_chroma_quantizer[64] = {
  17, 18, 24, 47, 99, 99, 99, 99,
  18, 21, 26, 66, 99, 99, 99, 99,
  24, 26, 56, 99, 99, 99, 99, 99,
  47, 66, 99, 99, 99, 99, 99, 99,
  99, 99, 99, 99, 99, 99, 99, 99,
  99, 99, 99, 99, 99, 99, 99, 99,
  99, 99, 99, 99, 99, 99, 99, 99,
  99, 99, 99, 99, 99, 99, 99, 99,
};

void jpeg_qt(int q, int* out,   int* in)
{
  int i,val;
  q = quality_to_qfactor(q);
  for (i = 0; i < 64; i++) {
    val = (q * in[i] + 50) / 100;
    /*
     * Baseline JPEG restricts range to [1,255].
     */
    if (val < 1)
      val = 1;
    else if (val > 255)
      val = 255;
    out[i] = val;
  }
}

void jpeg_luma_qt(int q, int* qt)
{
  jpeg_qt(q, qt, jpeg_luma_quantizer);
}

void jpeg_chroma_qt(int q, int* qt)
{
  jpeg_qt(q, qt, jpeg_chroma_quantizer);
}

static   unsigned char dc_luminance_bits[17] =
{ /* 0-base */ 0, 0, 1, 5, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0 };
static   unsigned char dc_luminance_val[] =
{ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 };

static   unsigned char dc_chrominance_bits[17] =
{ /* 0-base */ 0, 0, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0 };
static   unsigned char dc_chrominance_val[] =
{ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 };

static   unsigned char ac_luminance_bits[17] =
{ /* 0-base */ 0, 0, 2, 1, 3, 3, 2, 4, 3, 5, 5, 4, 4, 0, 0, 1, 0x7d };
static   unsigned char ac_luminance_val[] =
{ 0x01, 0x02, 0x03, 0x00, 0x04, 0x11, 0x05, 0x12,
  0x21, 0x31, 0x41, 0x06, 0x13, 0x51, 0x61, 0x07,
  0x22, 0x71, 0x14, 0x32, 0x81, 0x91, 0xa1, 0x08,
  0x23, 0x42, 0xb1, 0xc1, 0x15, 0x52, 0xd1, 0xf0,
  0x24, 0x33, 0x62, 0x72, 0x82, 0x09, 0x0a, 0x16,
  0x17, 0x18, 0x19, 0x1a, 0x25, 0x26, 0x27, 0x28,
  0x29, 0x2a, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39,
  0x3a, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49,
  0x4a, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59,
  0x5a, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69,
  0x6a, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79,
  0x7a, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89,
  0x8a, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98,
  0x99, 0x9a, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7,
  0xa8, 0xa9, 0xaa, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6,
  0xb7, 0xb8, 0xb9, 0xba, 0xc2, 0xc3, 0xc4, 0xc5,
  0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xd2, 0xd3, 0xd4,
  0xd5, 0xd6, 0xd7, 0xd8, 0xd9, 0xda, 0xe1, 0xe2,
  0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea,
  0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8,
  0xf9, 0xfa };
static   unsigned char ac_chrominance_bits[17] =
{ /* 0-base */ 0, 0, 2, 1, 2, 4, 4, 3, 4, 7, 5, 4, 4, 0, 1, 2, 0x77 };
static   unsigned char ac_chrominance_val[] =
{ 0x00, 0x01, 0x02, 0x03, 0x11, 0x04, 0x05, 0x21,
  0x31, 0x06, 0x12, 0x41, 0x51, 0x07, 0x61, 0x71,
  0x13, 0x22, 0x32, 0x81, 0x08, 0x14, 0x42, 0x91,
  0xa1, 0xb1, 0xc1, 0x09, 0x23, 0x33, 0x52, 0xf0,
  0x15, 0x62, 0x72, 0xd1, 0x0a, 0x16, 0x24, 0x34,
  0xe1, 0x25, 0xf1, 0x17, 0x18, 0x19, 0x1a, 0x26,
  0x27, 0x28, 0x29, 0x2a, 0x35, 0x36, 0x37, 0x38,
  0x39, 0x3a, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48,
  0x49, 0x4a, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58,
  0x59, 0x5a, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68,
  0x69, 0x6a, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78,
  0x79, 0x7a, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,
  0x88, 0x89, 0x8a, 0x92, 0x93, 0x94, 0x95, 0x96,
  0x97, 0x98, 0x99, 0x9a, 0xa2, 0xa3, 0xa4, 0xa5,
  0xa6, 0xa7, 0xa8, 0xa9, 0xaa, 0xb2, 0xb3, 0xb4,
  0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba, 0xc2, 0xc3,
  0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xd2,
  0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9, 0xda,
  0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9,
  0xea, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8,
  0xf9, 0xfa };

void defaults(config* c)
{
  int i;
  c->width = 0;
  c->height = 0;
  
  c->precision = 8;
  c->ncomp = 3;
  /* Y */
  c->comp[0].id = 0;
  c->comp[0].hsf = 2;
  c->comp[0].vsf = 1;	/* RTP type 0, 4:2:2 JPEG */
  c->comp[0].qno = 0;
  /* U */
  c->comp[1].id = 1;
  c->comp[1].hsf = 1;
  c->comp[1].vsf = 1;
  c->comp[1].qno = 1;
  /* V */
  c->comp[2].id = 2;
  c->comp[2].hsf = 1;
  c->comp[2].vsf = 1;
  c->comp[2].qno = 1;
  
  c->dc_huffbits[0] = dc_luminance_bits;
  c->dc_huffval[0] = dc_luminance_val;
  c->dc_huffbits[1] = dc_chrominance_bits;
  c->dc_huffval[1] = dc_chrominance_val;
  c->ac_huffbits[0] = ac_luminance_bits;
  c->ac_huffval[0] = ac_luminance_val;
  c->ac_huffbits[1] = ac_chrominance_bits;
  c->ac_huffval[1] = ac_chrominance_val;
  for (i = 2; i < 4; ++i) {
    c->ac_huffbits[i] = 0;
    c->ac_huffval[i] = 0;
    c->dc_huffbits[i] = 0;
    c->dc_huffval[i] = 0;
  }
  c->comp[0].dc_tbl_no = 0;
  c->comp[0].ac_tbl_no = 0;
  c->comp[1].dc_tbl_no = 1;
  c->comp[1].ac_tbl_no = 1;
  c->comp[2].dc_tbl_no = 1;
  c->comp[2].ac_tbl_no = 1;
  c->cache_ =NULL;
  c->frm_   =NULL;
}

/*
 * Set the quantizer for this configuration.
 * Q is the IJPG quality factor, which has a value in [0,100].
 */
void quantizer(config* c)
{
  jpeg_luma_qt(c->quality, c->qtab[0]);
  jpeg_chroma_qt(c->quality, c->qtab[1]);
  
  memset(c->qtab[2], 0, sizeof(c->qtab[2]));
  memset(c->qtab[3], 0, sizeof(c->qtab[3]));
}

int q_to_thresh(int q)
{
  int s = q / 10;
  if (q <= 30 || q >= 80)
    s += 1;
  return (s);
}

/*
 * init() is called only by the  ructor above
 */
void init(config* c)
{
  int i;
  int maxh = 1;
  int maxv = 1;
  c->rlen_ = 0;
  
  c->size_ = c->width * c->height;	/* output size*/
  memcpy((char*)c->qt_, (char*)c->qtab, sizeof(c->qt_));
  for (i = 0; i < 4; ++i)
    rdct_fold_q(c->qt_[i], c->fqt_[i]);
  
  for (i = c->ncomp; --i >= 0; ) {
    int id = c->comp[i].id;
    c->comp_[id].hsf = c->comp[i].hsf;
    c->comp_[id].vsf = c->comp[i].vsf;
    c->comp_[id].qno = c->comp[i].qno;
    c->comp_[id].dc_tbl_no = c->comp[i].dc_tbl_no;
    c->comp_[id].ac_tbl_no = c->comp[i].ac_tbl_no;
  }
  /*
   * XXX should check if huffman table won't change
   * before reallocating.
   */
  freehufftab(c);
  for (i = 0; i < 4; ++i) {
    if (c->dc_huffval[i] != 0) {
      int id = c->comp[i].id;
      c->dcht_[id] = huffbuild(c->dc_huffbits[i],
			       c->dc_huffval[i]);
    }
    if (c->ac_huffval[i] != 0) {
      int id = c->comp[i].id;
      c->acht_[id] = huffbuild(c->ac_huffbits[i],
			       c->ac_huffval[i]);
    }
  }
  
  for (i = c->ncomp; --i >= 0; ) {
    if (maxh < c->comp[i].hsf)
      maxh = c->comp[i].hsf;
    if (maxv < c->comp[i].vsf)
      maxv = c->comp[i].vsf;
  }
  
  /* # rows and cols in input JPEG image */
  c->ncol_ = (c->width +   BITS_NUM* maxh - 1) / (BITS_NUM  * maxh);
  c->nrow_ = (c->height +   BITS_NUM* maxv - 1) / (BITS_NUM  * maxv);
  
  if (compute_margins(maxh, maxv,c) != 0)  {
    fprintf(stderr, 
	    "JpegDecoder::init: couldn't compute margins\n");
  }
  
}
#define INIT_M {m.top=m.right=m.left=0;\
m.ytopskip=m.uvtopskip=m.marktopskip=0;\
m.ylskip=m.uvlskip=m.marklskip=0;\
m.yrskip=m.uvrskip=m.markrskip=0;\
m.ftopskip=m.flskip=m.frskip=0;}

int compute_margins(int maxh, int maxv,config *c)
{
  int dx,dy,q;
  /*
   * Embed one image size in a bigger one.
   * (extra areas will already be gray from init() )
   */
  margin m;
  INIT_M;
  m.top = m.left = m.right = 0;
  c->lcrop_ = 0;
  c->rcrop_ = c->ncol_;
  c->topcrop_ = 0;
  c->botcrop_ = c->nrow_;
  
  dx = 0;
  dy = 0;
  /*
   * If output size is smaller that intput, crop input image to fit 
   * output size, otherwise try and center the image in the larger 
   * frame, making sure to align on macroblock (16x16) boundaries.
   */
  if (dx < 0) {
    int q;
    dx = -dx;
    /* Bail if can't align on block boundary. */
    if (dx % 8 != 0)
      return (-1);
    
    q = dx / 16;
    if (q % 2 != 0) {
      /* Can't center crop horizontally */
      c->lcrop_=(((dx -16) / 2) + 8 * maxh - 1) / (8 * maxh);
      c->rcrop_ = c->ncol_ - c->lcrop_ + 1;
    } else {
      /* Center crop horizontally */
      c->lcrop_ = ((dx / 2) + 8 * maxh - 1) / (8 * maxh);
      c->rcrop_ = c->ncol_ - c->lcrop_;
    }
  } else {
    if (dx % 8 != 0)
      return (-1);
    
    /* Know dx is multiple of 8 at least */
    if (dx % 16 == 8) {
      /* crop one column and bias dx */
      c->ncol_--;
      dx -= 8;
    }
    
    q = dx / 16;
    if (q % 2 != 0) {
      /* Can't center horizontally */
      m.right = (dx - 16) / 2;
      m.left = (dx + 16) / 2;
    } else {
      /* Center horizontally */
      m.right = m.left = dx / 2;
    }
  }
  
  if (dy < 0) {
    dy = -dy;
    /* Bail if can't align on block boundary. */
    if (dy % 8 != 0)
      return (-1);
    
    q = dy / 16;
    if (q % 2 != 0) {
      /* Can't center crop vertically */
      c->topcrop_ = (((dy - 16) / 2) + 8 * maxv - 1) / 
	(8 * maxv);
      c->botcrop_ = c->nrow_ -c-> topcrop_ + 1;
    } else {
      /* Center crop vertically */
      c->topcrop_ = ((dy / 2) + 8 * maxv - 1) / (8 * maxv);
			c->botcrop_ = c->nrow_ - c->topcrop_;
    }
  } else {
    /* Bail if can't align on block boundary. */
    if (dy % 8 != 0)
      return (-1);
    /* Know dy is multiple of 8 at least */
    if (dy % 16 == 8) {
      /* crop one row and bias dy */
      c->nrow_--;
      dy -= 8;
    }
    q = dy / 16;
    if (q % 2 != 0) {
      /* Can't center vertically */
      m.top = (dy - 16) / 2;
    } else {
      /* Center vertically */
      m.top = dy / 2;
    }
  } 
  return (0);
}

/**********************************************************
 * delete all the resources allocated for the decompression
 **********************************************************/
char* Delete_JpegDecoder(char *config_)
{
  config *config_struct=config_;
  if(config_!=NULL) {
    if(config_struct->rvts_!=NULL)
      free(config_struct->rvts_);
    if(config_struct->cache_!=NULL)
      free(config_struct->cache_);
    if(config_struct->frm_!=NULL)
      free(config_struct->frm_);
    free(config_);
  }
  return NULL;
}
  
  
/***************************************************/
/* Initialization for jpeg 411 code
 * note diaply, window and the_Visual  parameters are added only to 
 * achieve a resemblance with the yuv initalizer
 * return the decoder structure 
 */
char * Init_JpegDecoder(int width, int height,int quality,char* display,
			 int window,char* the_Visual)
{
  config *config_;
  if((config_=(config *)malloc(sizeof(config)))==NULL)
         message_output("NOt Enough Memory",1);


  defaults(config_);
  config_->width=width;
  config_->height=height;
  config_->quality=quality;
  config_->dec_now_=1;
  config_->comp[0].vsf = 2;
  config_->comp[1].hsf = 1;
  config_->comp[1].vsf = 1;
  config_->comp[2].hsf = 1;
  config_->comp[2].vsf = 1;
  quantizer(config_);
  init_JpegDecoder_411(config_);
  config_->thresh_= q_to_thresh(config_->quality);
  config_->nblk_ = (config_->width*config_->height) / 64;
  if((config_->rvts_ = malloc(sizeof(u_char)*config_->nblk_))==NULL)
      message_output("NOt Enough Memory",1);
  memset(config_->rvts_, 0, config_->nblk_);
  return((char *)config_);
  
}

/***************************************************/
void setlrskips(config* c)
{
  margin m;
  INIT_M;
  m.ylskip = m.left;
  m.uvlskip = m.left / 2;
  
  m.yrskip = m.right;
  m.uvrskip = m.right / 2;
  
  m.ytopskip = m.top * c->width;
  m.uvtopskip = m.ytopskip / 256 * 64;
  
  
  m.marktopskip = m.top / 8 * c->width / 8;
  m.marklskip = m.left / 8;
  m.markrskip = m.right / 8;
}

/**************************************************************************/
void freehufftab(config* c)
{
  int i;
  for ( i = 0; i < 4; ++i) {
    if (c->dcht_[i] != 0){
      free(c->dcht_[i]);
      c->dcht_[i]=NULL;
    }
    if (c->acht_[i] != 0){
      free(c->acht_[i]);
      c->acht_[i]=NULL;
    }
  }
}

/**************************************************************************/
#if NCC != 6

@BUG in blkdiff@
#endif
#define ABS(t) ((t) - (((t) >> 31 & (t)) << 1))
int zag_blkdiff(short* blk, short* ref)
{
  int t,d;
  t = blk[0] - ref[0];
  d = ABS(t);
  t = blk[1] - ref[8];
  d += ABS(t);
  t = blk[2] - ref[1];
  d += ABS(t);
  t = blk[3] - ref[2];
  d += ABS(t);
  t = blk[4] - ref[9];
  d += ABS(t);
  t = blk[5] - ref[16];
  d += ABS(t);
  return (d);
}

/**************************************************************************/
int inorder_blkdiff(short* blk, short* ref)
{
  int t,d;
  t = blk[0] - ref[0];
  d = ABS(t);
  t = blk[1] - ref[1];
  d += ABS(t);
  t = blk[2] - ref[2];
  d += ABS(t);
  t = blk[3] - ref[3];
  d += ABS(t);
  t = blk[4] - ref[4];
  d += ABS(t);
  t = blk[5] - ref[5];
  d += ABS(t);
  return (d);
}

/**************************************************************************/
#ifdef notdef
/* XXX this is negligibly faster than above, and has a bug */
int blkdiff(short* blk, short* cache)
{
  uint *p0,*p1,m,tv,d;
  p0 = (u_int*)blk;
  p1 = (u_int*)cache;
  
  m = ~0x80008000;
  t = (p0[0] >> 1) & m;
  t += (~p1[0] >> 1) & m;
  int v = t << 17 >> 17;
  int d = ABS(v);
  t = (t << 1) >> 17;
  d += ABS(v);
  
  t = (p0[1] >> 1) & m;
  t += (~p1[1] >> 1) & m;
  v = t << 17 >> 17;
  d += ABS(v);
  t = (t << 1) >> 17;
  d += ABS(v);
  
  t = (p0[2] >> 1) & m;
  t += (~p1[2] >> 1) & m;
  v = t << 17 >> 17;
  d += ABS(v);
  t = (t << 1) >> 17;
  d += ABS(v);
  
  return (d);
}
#endif

/**************************************************************************/
int rdqt(  u_char* p,config *c)
{
  int len,prec,v,n,qt[64],i,j;
  u_char* ep;
  len = *p++ << 8;
  len |= *p++;
  
  ep = p + len - 2;
  while (p < ep) {
    n = *p++;
    prec = n >> 4;
    n &= 0x0f;
    if (n >= 4) {
      /*XXX illegal number*/
      return (-1);
    }
    for (i = 0; i < 64; i++) {
      v = *p++;
      if (prec)
	v = (v << 8) + *p++;
      qt[i] = v;
    }
    for (i = 0; i < 64; i++) {
      /* Compute new table only if it changed */
      j = ROWZAG[i];
      if (qt[i] != c->qt_[n][j]) {
	for (; i < 64; ++i) {
	  j = ROWZAG[i];
	  c->qt_[n][j] = qt[i];
	}
	rdct_fold_q(c->qt_[n], c->fqt_[n]);
	break;
      }
    }
  }
  return (len);
}

/**************************************************************************/
void restart(config* conf)
{
  int c;
  conf->nbb_ = 0;
  /*XXXwhat if ff is sitting in bit buffer?*/
  /* Scan for next JPEG marker */
  do {
    do {			/* skip any non-FF bytes */
      c = *(conf->inb_++);
    } while (c != 0xFF);
    do {
      /* skip any duplicate FFs */
      /* we don't increment nbytes here since extra FFs are legal */
      c = *(conf->inb_++);
    } while (c == 0xFF);
  } while (c == 0);		/* repeat if it was a stuffed FF/00 */
#ifdef notdef
  if (nbytes != 1)
    WARNMS2(cinfo->emethods,
	    "Corrupt JPEG data: %d extraneous bytes before marker 0x%02x",
	    nbytes-1, c);
  
#endif
#ifdef notdef
  if (c != (RST0 + cinfo->next_restart_num)) {
    /* Uh-oh, the restart markers have been messed up too. */
    /* Let the file-format module try to figure out how to resync. */
    (*cinfo->methods->resync_to_restart) (cinfo, c);
  } else
    TRACEMS1(cinfo->emethods, 2, "RST%d", cinfo->next_restart_num);
#endif
  /* Re-initialize DC predictions to 0 */
  conf->comp_[0].dc = 0;
  conf->comp_[1].dc = 0;
  conf->comp_[2].dc = 0;
#ifdef notdef
  cinfo->next_restart_num = (cinfo->next_restart_num + 1) & 7;
#endif
}

/**************************************************************************/
u_char * parseJFIF(  u_char* in,config *c)
{
  int t;
  while (in < c->end_) {
    if (*in++ != 0xff)
      continue;
    /*XXX need more checks for buffer overflow*/
    switch (*in++) {
      
    default:
      /* Don't know.  Keep looking for SOS. */
      continue;			
    case 0xdb:
      continue;
      
    case 0xdd:
      /* restart interval definition */
      t = *in++ << 8;
      t |= *in++;
      if (t != 4)
	/* XXX bad length */
	;
      c->rlen_ = *in++ << 8;
      c->rlen_ |= *in++;
      c->rcnt_ = 0;
      continue;
      
    case 0xda:
      /* start-of-scan marker */
      if (in + 2 <= c->end_) {
	/* skip over SOS */
	int t = *in++ << 8;
	t |= *in++;
	in += (t - 2);
      }
      return (in);
    }
  }
  /*XXX*/
  return (c->end_);
}

/**************************************************************************/
#define MASK_DECL	u_int mask[2];mask[0]=0;mask[1]=0
#define MASK_VAL	mask[0], mask[1]
#define MASK_REF	mask
void Rdct(int nc, register short *bp, u_int m0, u_int m1,
	  u_char* pixels, int stride, int qno)
{
  if (nc != 0) {
    rdct(bp, m0, m1, pixels, stride, config_->fqt_[qno]);}
}



/**************************************************************************/
/* When redispaying a stream initialize it so that the first frame will be 
 * completely displayed */
void Jpeg_ReInit(char* c, char* theDisp, int window)
{
  int ns;
  config_=(config *)c;
  /*
   * Initialize to some "large value" so threshold
   * is exceeded on first pass.
   */
  ns = NCC * config_->width*config_->height / 64;	/*# blocks*/
  memset(config_->cache_, 0x7f, ns * sizeof(*(config_->cache_)));
}


/**************************************************************************/
void init_jpeg_q(char *c, int q)
{
  config_=(config *)c;
  config_->quality=q;
  quantizer(config_);
  init_JpegDecoder_411(config_);
  config_->thresh_= q_to_thresh(config_->quality);
}  

/**************************************************************************/
/*
 * 411 Decoders
 * decodes a frame saved in "in" and return a pointer to the decoded output
 */

void JpegDecoder_411(u_char* in,int len,int q,char* c,char* the_Image)
{
  u_char *marks;
  int mark;
  int q0,q1,y,x;
  u_char *yp,*vp,*up;
  short *cache,blk[64];
  margin m;
  INIT_M;
  
  config_=(config *)c;
  marks=config_->rvts_;
  mark=config_->dec_now_;
  config_->inb_ = in;
  config_->end_ = in + len;
  config_->nbb_ = 0;
  /*
   * If first symbol is a marker (a not a stuffed ff),
   * assume a jfif header is present and parse it.
   * XXX this could change state that needs to be
   * communicated back to caller.
   */
  if (in[0] == 0xff && in[1] != 0)
    config_->inb_ = parseJFIF(config_->inb_,config_);
  
  huffreset(config_);
  q0 = config_->comp_[0].qno;
  q1 = config_->comp_[1].qno;
  yp = config_->frm_;
  up = yp + config_->size_;
  vp = up + config_->size_ / 4;
  cache = config_->cache_;
  
  /* Skip top */
  yp += m.ytopskip;
  up += m.uvtopskip;
  vp += m.uvtopskip;
  marks+= m.marktopskip;
  cache += m.marktopskip * NCC;
  for (y = 0; y < config_->nrow_; ++y) {
    int ycrop = (y < config_->topcrop_ ||y >= config_->botcrop_);
    if (!ycrop) {
      marks+= m.marklskip;
      yp += m.ylskip;
      up += m.uvlskip;
      vp += m.uvlskip;
      cache += m.marklskip * 2 * NCC;
    }
    for (x = 0; x < config_->ncol_; ++x) {
      int nc,dontskip;
      MASK_DECL;
      if (ycrop ||x<config_->lcrop_||x>=config_->rcrop_) {
	(void)huffskip(&(config_->comp_[0]));
	(void)huffskip(&(config_->comp_[0]));
	(void)huffskip(&(config_->comp_[0]));
	(void)huffskip(&(config_->comp_[0])); 
	(void)huffskip(&(config_->comp_[1]));
	(void)huffskip(&(config_->comp_[2]));
	continue;
      }
      /*
       * If we're handling restart markers,
       * check if we need to resync.
       */
      if (config_->rlen_ != 0 && --config_->rcnt_ <= 0) {
	config_->rcnt_ = config_->rlen_;
	restart(config_);
      }
      nc=huffparse(&(config_->comp_[0]),blk,cache,MASK_REF,0);
      cache += NCC;
      dontskip = nc;
      Rdct(nc, blk, MASK_VAL, yp,config_->width,q0);
      nc = huffparse(&(config_->comp_[0]),blk,cache, 
		     MASK_REF,dontskip);
      cache += NCC;
      dontskip |= nc;
      Rdct(nc,blk,MASK_VAL,yp +8,config_->width,q0);
      nc = huffparse(&(config_->comp_[0]), blk,
		     cache,MASK_REF,dontskip);
      cache += NCC;
      dontskip |= nc;
      Rdct(nc, blk, MASK_VAL, yp + 8 * config_->width, 
	   config_->width, q0);
      nc = huffparse(&(config_->comp_[0]),blk,cache,MASK_REF,
				       dontskip);
      cache += NCC;
      dontskip |= nc;
      Rdct(nc, blk, MASK_VAL,yp+8* config_->width+8,
	   config_->width,q0);
      if (config_->color_ && dontskip) {
	/*
	 * If we found above that the luminance
	 * planes exceeded the threhold, decode
	 * the choma planes unconditionally.
	 * Otherwise, see if they can be
	 * suppressed too.
	 */
	short dummy[6];
	nc = huffparse(&(config_->comp_[1]),
		       blk,dummy,MASK_REF,1);
	Rdct(nc,blk,MASK_VAL,up,config_->width/2,q1);
	nc = huffparse(&(config_->comp_[2]), blk, 
		       dummy, MASK_REF,1);
	Rdct(nc,blk,MASK_VAL,vp,config_->width/2,q1);
      } else { 
	(void)huffskip(&(config_->comp_[1]));
	(void)huffskip(&(config_->comp_[2]));
      }
      if (dontskip) {
	int off;
	marks[0] = mark;
	marks[1] = mark;
	off = config_->width >> 3;
	marks[off] = mark;
	marks[off + 1] = mark;
      }
      marks+= 2;
      yp += 16;
      up += 8;
      vp += 8;
    }
    if (!ycrop) {
      /* Skip over entire macroblock */
      marks+= m.markrskip;
      yp += m.yrskip;
      up += m.uvrskip;
      vp += m.uvrskip;
      cache += m.markrskip * 2 * NCC;
      
      marks+= config_->width >> 3;
      yp -= config_->width;
      up -= config_->width / 2;
      vp -= config_->width / 2;
      
      yp += 2 * 8 * config_->width;
      up += 8 * config_->width / 2;
      vp += 8 * config_->width / 2;
    }
  }
  config_->render((char *)config_->frm_,(char *)config_,the_Image);
  
}




/* Figure F.12: extend sign bit */

#ifdef notdef
#define huff_EXTEND(x,s)  ((x) < extend_test[s] ? (x) + extend_offset[s] : (x))

static   int extend_test[16] =   /* entry n is 2**(n-1) */
{ 0, 0x0001, 0x0002, 0x0004, 0x0008, 0x0010, 0x0020, 0x0040, 0x0080,
  0x0100, 0x0200, 0x0400, 0x0800, 0x1000, 0x2000, 0x4000 };

static   int extend_offset[16] = /* entry n is (-1 << n) + 1 */
{ 0, ((-1)<<1) + 1, ((-1)<<2) + 1, ((-1)<<3) + 1, ((-1)<<4) + 1,
  ((-1)<<5) + 1, ((-1)<<6) + 1, ((-1)<<7) + 1, ((-1)<<8) + 1,
  ((-1)<<9) + 1, ((-1)<<10) + 1, ((-1)<<11) + 1, ((-1)<<12) + 1,
  ((-1)<<13) + 1, ((-1)<<14) + 1, ((-1)<<15) + 1 };
#else
/* is this really faster? */
int huff_EXTEND(int x, int s)
{
  register int b = x >> (s - 1);
  register int m = ((b & 1) - 1) << s;
  return ((x | m) + (~b & 1));
}
#endif

/*
 * Read the next 16 bits off the bit string into the bit buffer.
 * Skip over zero-stuffed ff's but make no attempt to verify
 * that they aren't some other marker (which shouldn't be in the
 * middle of a block anyway).
 */
#define HUFFRQ(bb) \
{ \
    register int v; \
    register  u_char *cp = config_->inb_; \
    \
    bb <<= 16; \
    v = *cp++; \
    if (v == 0xff) ++cp; \
    bb |= v << 8; \
    v = *cp++; \
    if (v == 0xff) ++cp; \
    bb |= v; \
    config_->inb_ = cp; \
    \
}

#define MASK(s) ((1 << (s)) - 1)

#define HUFF_DECODE(ht, nbb, bb,result) { \
  register int s_, v_; \
  \
  if (nbb < 16) { \
		    HUFFRQ(bb); \
		    nbb += 16; \
  } \
  v_ = (bb >> (nbb - 16)) & 0xffff; \
  s_ = (ht)[v_]; \
  nbb -= (s_ >> 8); \
  result = s_ & 0xff; \
}

#define GET_BITS(n, nbb, bb, result) \
{ \
    nbb -= n; \
    if (nbb < 0)  { \
		      HUFFRQ(bb); \
		      nbb += 16; \
    } \
    (result) = ((bb >> nbb) & MASK(n)); \
}

#define SKIP_BITS(n, nbb, bb) \
{ \
    nbb -= n; \
	if (nbb < 0)  { \
			  HUFFRQ(bb); \
			  nbb += 16; \
	} \
}


/*
 * Parse a huffman-encoded 8x8 block.  Blocks are independent
 * of eachother, except for the dc predictor, and blocks can
 * start and end on arbitrary bit boundaries.  No markers should
 * appear in the bit stream, and ff bytes should be zero stuffed
 * (i.e., replaced with ff 00).
 *
 * The block is coded as a sequence of pairs of symbols.  Where the
 * first symbol is a huffman-encoded value <r,n> where r is a four-bit
 * runlength and n is the length of the second symbol (of the pair),
 * which follows verbatim in the bit string.
 */

int huffparse(component* p, short* out, short* ref,
	      u_int* mask, int dontskip)
{
  short in[6];
  register int k = 1;
  register int bb = config_->bb_;
  register int nbb = config_->nbb_;
  u_short* ht = config_->dcht_[p->dc_tbl_no];
  register int s, r;
#ifdef INT_64
  INT_64 m0 = 1;
#else
  u_int m1 = 0;
  u_int m0 = 1;
#endif
  HUFF_DECODE(ht, nbb, bb, s);
  if (s != 0) {
    GET_BITS(s, nbb, bb, r);
    s = huff_EXTEND(r, s);
    /* update predictor */
    s += p->dc;
    p->dc = s;
  } else
    s = p->dc;
  
  in[0] = s;
  in[1] = 0;
  in[2] = 0;
  in[3] = 0;
  in[4] = 0;
  in[5] = 0;
  
  /*
   * First, grab only a few low frequency coefficients.
   * If they aren't sufficiently different from the current
   * block, skip over this block quickly.
   */
  ht = config_->acht_[p->ac_tbl_no];
  
  for (;;) {
    register int v;
    /* Symbol-1 */
    HUFF_DECODE(ht, nbb, bb, v);
    s = v & 15;
    r = v >> 4;
    if (s != 0) {
      k += r;
      if (k >= 6) {
	k -= r;
	break;
      }
      /* Symbol-2 */
      GET_BITS(s, nbb, bb, v);
      s = huff_EXTEND(v, s);
      in[k] = s;
      v = COLZAG[k];
      m0 |= 1 << v;
      
      ++k;
		} else
		  break;
  }
  
  if (!dontskip && inorder_blkdiff(in, ref) < config_->thresh_) {
		/* skip this block */
    for (;;) {
      register int v;
      if (s != 0) {
	k += r;
	/* Symbol-2 */
	SKIP_BITS(s, nbb, bb);
	++k;
      } else {
	if (r != 15)
	  /* end of block */
	  break;
	k += 16;
      }
      if (k >= 64)
	break;
      
      /* Symbol-1 */
      HUFF_DECODE(ht, nbb, bb, v);
      s = v & 15;
      r = v >> 4;
    }
    config_->nbb_ = nbb;
    config_->bb_ = bb;
    return (0);
  }
  
  /* 
   * Need to do this since there may be holes in the loop below and
   * coefficients won't get updated.
   */
  memset((char*)out, 0, 2 * 64);
  
  ref[0] = out[0] = in[0];
  ref[1] = out[8] = in[1];
  ref[2] = out[1] = in[2];
  ref[3] = out[2] = in[3];
  ref[4] = out[9] = in[4];
  ref[5] = out[16] = in[5];
  
  for (;;) {
    register int v;
    if (s != 0) {
      k += r;
      /* Symbol-2 */
      GET_BITS(s, nbb, bb, v);
      s = huff_EXTEND(v, s);
      v = COLZAG[k];
      
      out[v] = s;
#ifdef INT_64
      m0 |= (INT_64)1 << v;
#else
      if (v < 32)
	m0 |= 1 << v;
      else
	m1 |= 1 << (v - 32);
#endif
      ++k;
    } else {
      if (r != 15)
	/* end of block */
	break;
      
      k += 16;
    }
    if (k >= 64)
      break;
    
    /* Symbol-1 */
    HUFF_DECODE(ht, nbb, bb, v);
    s = v & 15;
    r = v >> 4;
  }
  
#ifdef INT_64
  *mask = m0;
#else
  mask[0] = m0;
  mask[1] = m1;
#endif
  config_->nbb_ = nbb;
  config_->bb_ = bb;
  
  return (1);
}

/*
 * Skip over a block.
 */
int huffskip(component* p)
{
  register int k;
  register int bb =config_->bb_;
  register int nbb = config_->nbb_;
  u_short* ht = config_->dcht_[p->dc_tbl_no];
  register int s;
  HUFF_DECODE(ht, nbb, bb, s);
  if (s != 0) {
    int r;
    GET_BITS(s, nbb, bb, r);
    s = huff_EXTEND(r, s);
    /* update predictor */
    s += p->dc;
    p->dc = s;
  }
  ht = config_->acht_[p->ac_tbl_no];
  for (k = 1; k < 64; ) {
    /* Symbol-1 */
    register int v,r;
    HUFF_DECODE(ht, nbb, bb, v);
    s = v & 15;
    r = v >> 4;
    if (s != 0) {
      k += r;
      /* Symbol-2 */
      SKIP_BITS(s, nbb, bb);
      ++k;
    } else {
      if (r != 15)
	/* end of block */
	break;
      k += 16;
    }
  }
  config_->nbb_ = nbb;
  config_->bb_ = bb;
  
  return (0);
}

void huffreset(config* c)
{
  c->nbb_ = 0;
  c->comp_[0].dc = 0;
  c->comp_[1].dc = 0;
  c->comp_[2].dc = 0;
}

/*
 * Build a 64k lookup table from the jpeg huffman table described
 * by the arguments.  The table is indexed by the next 16-bits
 * of the input stream.  The entry in the table tells us the
 * length of the next code and its decoded value.
 */
u_short* huffbuild(  u_char* bits,   u_char* vals)
{
  /* Figure C.1: make table of Huffman code length for each symbol */
  /* Note that this is in code-length order. */
  
  int code = 0;
  int si,p = 0;
  int sym,nsym = 0;
  u_short huffcode[256];
  u_short *ht;
  int huffsize[257];
  int codelen,i;
  for (codelen = 1; codelen <= 16; ++codelen) {
    for (i = 1; i <= bits[codelen]; ++i)
      /*
       * XXX should sanity check that nsym stays
       * below 256.
       */
      huffsize[nsym++] = codelen;
	}
  huffsize[nsym] = 0;
  
  /* Figure C.2: generate the codes themselves */
  /* Note that this is in code-length order. */
  
  code = 0;
  si = huffsize[0];
  p = 0;
  while (p < nsym) {
    while (huffsize[p] == si)
      huffcode[p++] = code++;
    
    code <<= 1;
    ++si;
  }
  /*
   * Build the direct-map lookup table.
	 */
  if((ht = malloc(sizeof(u_short)*65536))==NULL)
    message_output("NOt Enough Memory",1);
  
  memset((char*)ht, 0, 65536 * sizeof(u_short));
  for (sym = 0; sym < nsym; ++sym) {
    int n;
    int codelen = huffsize[sym];
    int nbit = 16 - codelen;
    int code = huffcode[sym] << nbit;
    int map = (codelen << 8) | vals[sym];
    /*
     * The low nbit bits are don't cares.
     * Spin through all possible combos.
     */
    for (n = 1 << nbit; --n >= 0; )
      ht[code | n] = map;
  }
  return (ht);
}

/***********************************************************************/
int Jpeg_AllocColors(char* c,char* the_Visual, char* the_Disp,int scale)
{
  Visual* theVisual;
  config_=(config *)c;
  theVisual=(Visual *)the_Visual;
  if(theVisual->class ==PseudoColor) {
    if(scale<1){
      config_->dither=dither_down_411;
      config_->render=renderer_up;     
    }
    if(scale==1){
      config_->dither=dither_411;
      config_->render=renderer;     
    }
    if(scale>1) {
      config_->dither=dither_411_up;
      config_->render=renderer_up;     
    }      
  }
  if ((theVisual->class == DirectColor)||
      (theVisual->class == TrueColor)) {
    if(scale<1) {
      config_->dither=map_411_down;
      config_->render=renderer_up;     
    }
    if(scale==1){
      config_->dither=map_411;
      config_->render=renderer;     
    }
    if(scale==2){
      config_->dither=map_411_up;
      config_->render=renderer_up;
    }
    return(alloc_true_colors(c,the_Visual, the_Disp));
  }
}





