/*
 *  yuv.c --
 *
 *       This file provides functions related to the YUV compression
 *
 *  (C) 1995 Bernd Deffner, GMD Fokus
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "/usr/include/netinet/in.h"  /* struct sockaddr_in */
#include <arpa/inet.h>   /* inet_ntoa() */
#include <netdb.h>
#include <string.h>
#include <errno.h>
#include <values.h>
#include <assert.h>
#include "xil_capture.h"
#include "types.h"
#include "rtp.h"
#include "tcl.h"
#include "nevit.h"
#include "util.h"
#include <stdarg.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/XShm.h>
#ifdef XIL
#include <xil/xil.h>
#endif
#include <sys/time.h>
#include "yuv.h"

static char rcsid[]  = "$Id: yuv.c,v 1.2 1998/04/06 08:24:37 dor Exp $";


#ifdef XIL

/*
 * delete all resources reserved for the decompression procedure 
 */
char * Delete_YUVDecoder(char *config)
{
  yuv_config  *config_=config;
  
  if(config!=NULL){
    xil_close(config_->state);
    free(config);
  }
  return NULL;
}

/*
 * initialize the YUV decompressor note the q parameter is just added
 * to make this procedure resemble the one for jpeg
 */

char* Init_YUVDecoder(int width, int height, int q,char* display, int window, 
		    char* the_Visual )
{
  Display* xdisplay;
  Window   xwindow;  Visual* theVisual=(Visual *)the_Visual;
  yuv_config  *Config;
  if((Config=(yuv_config *)malloc(sizeof(yuv_config)))==NULL)
   message_output("Not Enough Memory",1);
  xdisplay=(Display*)display;
  xwindow=(Window) window;

  Config->width= width;
  Config->height=height;
  Config->state = xil_open();
  if (Config->state == NULL) {
    fprintf(stderr, "unable to open xil library\n");
    exit(-1);
  }
  Config->cis = xil_cis_create(Config->state,"UYVY" );
  xil_cis_set_attribute((XilCis)Config->cis, "WIDTH",(void*)width);
  xil_cis_set_attribute(Config->cis, "HEIGHT",(void*)height);
  
  xil_cis_set_max_frames(Config->cis, 1);
  Config->imageYCC =
    xil_create(Config->state,Config->width,Config->height,3,XIL_BYTE);
  Config->display = xil_create_from_window(Config->state,xdisplay, xwindow);

  if (theVisual->class ==TrueColor) {
    XilColorspace cspace_ycc601 = 
      xil_colorspace_get_by_name(Config->state, "ycc601");
    XilColorspace cspace_rgb709 = 
      xil_colorspace_get_by_name(Config->state, "rgb709");
    
    xil_set_colorspace(Config->imageYCC, cspace_ycc601);
    xil_set_colorspace(Config->display, cspace_rgb709);
   
  }
  else{
    fprintf(stderr," YUV not supported for Pseudo Color machines ");
    return NULL;
  }
  return((char *)Config);
}

/* redirect the output to another window after starting a redisplay */
void YUV_ReInit(char* decode, char* theDisp, int window)
{
  Display* xdisplay= (Display *)theDisp;
  Window   xwindow=(Window )window;
  yuv_config  *decoder=(yuv_config   *)decode;
  decoder->display = xil_create_from_window(decoder->state,xdisplay,xwindow);
				       
  {  
    /*    XilColorspace cspace_ycc601 = 
      xil_colorspace_get_by_name(decoder->state, "ycc601");*/
    XilColorspace cspace_rgb709 = 
      xil_colorspace_get_by_name(decoder->state, "rgb709");
    xil_set_colorspace(decoder->display, cspace_rgb709);
  }
}

/* pseudo drawing procedure, Drawing is done here using XIL so we only need an
 * empty proc */

void YUV_draw(int arg, ...)
{
  return;
}

/* decompress the YUV frames and display them on the screen */

void  YUVDecoder(u_char* image, int len, int value,char* decode, 
		 char* the_Image)
{
  yuv_config  *decoder=(yuv_config   *)decode;

  xil_cis_put_bits_ptr(decoder->cis,len, 1,image, NULL);
  xil_decompress(decoder->cis, decoder->imageYCC);
  xil_color_convert(decoder->imageYCC,decoder->display);
  xil_toss(decoder->imageYCC);
}

int YUV_AllocColors(char* c,char* the_Visual, char* the_Disp,int scale)
{
  return 1;
}


/* end the XIL related part */
#endif
void YUV_fill_hdr(session_t *s, char *buf)
{
  yuv_hdr_t *yuv_hdr;

  yuv_hdr = (yuv_hdr_t *) buf;
  yuv_hdr->width = s->local->width;
  yuv_hdr->height = s->local->height;
}


/* 
 * fragment and transmit YUV frames, returns size of compressed frame
*/

int YUV_encode(session_t *s, int level)
{
  struct {
    rtp_hdr_t rtp_hdr;
    yuv_hdr_t yuv_hdr;
  } hdr;
  u_int8 *data;
  int i;
  int num_frags;   /* number of fragments */
  int frag_len;    /* length of the actual fragment */
  int left;        /* # payload bytes to transmit  */

  num_frags = s->local->rtp_s[level].nbytes / s->codec.data_limit + 
    ((s->local->rtp_s[level].nbytes % s->codec.data_limit) > 0); 
  left = s->local->rtp_s[level].nbytes;
  data = s->local->image;
  for(i = 0; i < num_frags; i++) 
    {
      
      /*
       * add RTP header and set Marker bit if this is the last
       * fragment of the YUV frame
       */
      RTP_fill_hdr(s, (char *)&hdr.rtp_hdr, (i==num_frags-1), 
		   s->local->rtp_s[level].ts, level);
      
      /* 
       * add YUV header 
       */
      YUV_fill_hdr(s, (char *)&hdr.yuv_hdr);

      /* 
       * send packet 
       */
      frag_len = 
	(i == num_frags - 1) ? left : s->codec.data_limit;

      s->net.write(level, (char *)&hdr, s->codec.hdr_len, data,
			frag_len);
      left -= s->codec.data_limit;
      data += s->codec.data_limit;
    }    

  /* increase timestamp */
  assert(s->grabber.frames > 0);
  s->local->rtp_s[level].ts = RTP_inc_ts(s->local->rtp_s[level].ts, 90000 / s->grabber.frames);
  return(s->local->rtp_s[level].nbytes + num_frags * s->codec.hdr_len);
}


int YUV_init(session_t *s)
{
  if(s->local->payload_type ==RTP_PT_YUV)
    return 0;
  s->local->payload_type = RTP_PT_YUV;
  s->codec.compressor = NEVIT_YUV;
  s->codec.encode = YUV_encode;
  s->codec.hdr_len = sizeof(rtp_hdr_t) + sizeof(yuv_hdr_t);
  s->codec.data_limit = NEVIT_MTU - s->codec.hdr_len;
  return 1;
}
