/*
* ring.h  -- definitions for ring buffer
* $Id: ring.h,v 1.1.1.1 1997/11/12 17:16:33 dor Exp $
*/

#ifndef _ring_h_
#define _ring_h_
#include <sys/types.h>

#define RING_BITS    15   /* 2^RING_BITS bytes in ring */
#define MAX_TALKING  8

typedef int (ring_action_t) (char *buffer, u_int32 count, void *arg1, void *arg);

/*
* Only the receiver uses the ring buffer.
*/

typedef struct {
  size_t size;             /* allocated size in bytes (must be power of 2) */
  u_int32 mask;            /* size mask for buffer (bytes) */
  /*
   * Next sample to be played out. Treated as non-wrap-around
   * (straight counter).
   */
  u_int32 next;
  /*
   * Sample time when last audio sample is going to play.
   * Treated as non-wrap-around (straight counter).
   */
  u_int32 last_sample;

  /*
   * TALKING INDICATION
   * Index of last valid member pointer in list.
   */
  int member_last;
  /*
   * List of currently talking members.
   */
  struct Audio_member_t *member[MAX_TALKING]; /* current talkers */

  /*
   * Mixing buffer.
   */
  char buffer[1];
} ring_t;

/* translate 'at' (measured in frames) to ring buffer location */
#define RB(r,at) (&r->buffer[(at) & r->mask])
#define RBAT(r)  (&r->buffer[r->at & r->mask])

extern void ring_activate(ring_t *r, u_int32 at);
extern ring_t *ring_create(audio_encoding_t e);
extern void ring_cpy(ring_t *r, u_int32 start, u_int32 size,
   void (*cpy)(), char *src, int bpf);
extern int ring_action(ring_t *r, u_int32 start, u_int32 size,
   ring_action_t *func, void *arg1, void *arg2, int bpf);
extern int ring_left(ring_t *r);
extern void ring_clear(ring_t *r, audio_descr_t *ad);
#endif
