/*
channels  ---  convert between stereo and mono.

Copyright 1993 by AT&T Bell Laboratories; all rights reserved
*/

#include "types.h"        /* u_int8 */
#include "audio_descr.h"
#include "dvi_adpcm.h"
#include "pcmu_l.h"
#include <netinet/in.h>   /* ntohs() */
#include <memory.h>       /* memcpy() */

extern u_int8 sum_table[];



/*
* Convert channel count. When converting from mono to stereo or higher
* channel counts, the 'local_audio_channel' variable determines where
* the incoming audio data is placed. If zero, it is spread to all
* channels, otherwise just to the channel named. 'in_bytes' and
* 'out_bytes' are measured in bytes.
*/
int channels(char *in_buf, int in_bytes, audio_descr_t *in_ad,
  char *out_buf, int *out_bytes, audio_descr_t *out_ad, 
  int local_audio_channel)
{
  int16  *is, *os;  /* output buffer for linear encoding */
  u_int8 *ic, *oc;  /* output buffer for mu-law encoding */
  int frames;       /* frames to convert */

  if (in_ad->encoding == AE_L16) {
    is = (int16 *)in_buf;
    os = (int16 *)out_buf;
    ic = 0;
    oc = 0;
  } else {
    is = 0;
    os = 0;
    ic = (u_int8 *)in_buf;
    oc = (u_int8 *)out_buf;
  }

  /* 
   * From mono to stereo: currently, just replicate; should position
   * audio in space.
   */
  if (in_ad->channels == 1 && out_ad->channels == 2) {
    *out_bytes = in_bytes * 2;
    /* Spread over both channels. */
    if (local_audio_channel == 0) {
      if (in_ad->encoding == AE_PCMU) {
        for ( ; in_bytes > 0; in_bytes--, ic++) {
          *oc++ = *ic;
          *oc++ = *ic;
        }
      }
      else if (in_ad->encoding == AE_L16) {
        for (frames = in_bytes/2; frames > 0; frames--, is++) {
          *os++ = *is;
          *os++ = *is;
        }
      }
      else return -1;
    }
    /* if audiochannel given, place mono audio in one stereo channel only */
    else {
      if (local_audio_channel > 2) return -1;
      local_audio_channel--;   /* more convenient */
      if (in_ad->encoding == AE_PCMU) {
        for (frames = in_bytes; frames > 0; frames--, ic++) {
          oc[local_audio_channel] = *ic;
          oc += 2;
        }
      }
      else if (in_ad->encoding == AE_L16) {
        for (frames = in_bytes/4; frames > 0; frames--, is++) {
          os[local_audio_channel] = *is;
          os += 2;
        }
      }
    }
    return 0;
  }

  /* From stereo to mono: mix two channels. */
  else if (in_ad->channels == 2 && out_ad->channels == 1) {
    *out_bytes = in_bytes / 2;
    if (in_ad->encoding == AE_PCMU) {
      for (frames = in_bytes/2; frames > 0; frames--, ic += 2) {
        *oc++ = sum_table[((*ic) << 8) | *(ic+1)] / 2;
      }
    }
    else if (in_ad->encoding == AE_L16) {
      for (frames = in_bytes/4; frames > 0; frames--, is += 2) {
        *os++ = (*is + *(is+1)) / 2;
      }
    }
    else return -1;
    return 0;
  }
  return -1;
} /* channels */
