/*
g711 -- Convert between ulaw PCM (CCITT G.711) and other formats, as specified
        by the audio header.

Copyright 1993 by AT&T Bell Laboratories; all rights reserved
*/

#include "audio_generic.h"
#include "pcmu_l.h"
#include "g711.h"



/*
* Initialize codec.
*/
void *g711_init(void *v, double frame)
{
  return 0;
} /* g711_init */

/*
* Translate from G.711 (mu-law PCM) to output format specified by 'ad'.
* Does not handle stereo/mono conversion.
*/
int g711_decode(void *in_buf, int in_size, audio_descr_t *ad, 
  void *out_buf, int *out_size, void *state)
{
  u_int8 *in_c;
  int16 *s;
  int i;

  in_c = (u_int8 *)in_buf;
  switch(ad->encoding) {
  case AE_L16:
    s = (int16 *)out_buf;
    for (i = 0; i < in_size; i++) s[i] = pcmu_l16(in_c[i]);
    *out_size = 2*in_size;
    break;

  default:
    return -1;
  }
  return 0;
} /* g711_decode */


/*
* Translate from input format described by 'ad' to G.711.
* Does not handle stereo/mono conversion.
*/
int g711_encode(void *in_buf, int in_size, audio_descr_t *ad, 
  void *out_buf, int *out_size, void *state, int header_flag)
{
  int i;
  u_int8 *out_c;

  out_c = (u_int8 *)out_buf;
  switch(ad->encoding) {
    case AE_L16:  /* 16 bit linear */
      in_size /= 2;  /* convert to units */
      for (i = 0; i < in_size; i++)
        out_c[i] = l16_to_pcmu(((int16 *)in_buf)[i]);
      *out_size = in_size;
      break;

    default:
      return -1;
  }
  return 0;
} /* g711_encode */
