/*
unix_audio  --  audio interface through Unix socket

Copyright 1994 by Henning Schulzrinne; all rights reserved
*/

#include "notify.h"
#include <fcntl.h>       /* O_RDWR */
#include <unistd.h>      /* close() */
#include "audio_generic.h"
#include <errno.h>



audio_support_t audio_support = {{0,0}, 0, 0, 0, {0,0,0,0}, 0, 0};

static int au_play = -1;            /* audio data output port */ 
static int au_rec  = -1;            /* audio data record port */

/*
* Open audio hardware in given mode 'iomode' and set file descriptor
* to non-blocking.  Return file descriptor number or -1 on error.
*/
int ahw_open(int iomode)
{
  int fd_flags;  /* fcntl flags */

  /* open audio data stream */
  if (!iomode) return -1;

  /* audio input (recording) */
  if ((iomode & A_RD) && (au_rec < 0)) {
    if ((au_rec = open(AUDIO_DATA, O_RDONLY | O_NDELAY)) < 0) {
      return -1;
    }
    fd_flags = fcntl(au_rec, F_GETFL, 0);
    (void)fcntl(au_rec, F_SETFL, fd_flags|O_NDELAY);
    rec_count = 0;
  }
  /* audio output (playback) */
  if ((iomode & A_WR) && (au_play < 0)) {
    if ((au_play = open(AUDIO_DATA, O_WRONLY | O_NDELAY)) < 0) {
      return -1;
    }
    play_count = 0;
  }

  return au_rec;
} /* ahw_open */


/*
* Close audio hardware. If (iomode & A_WR), close writing.
* If (iomode & A_RD), close reading. Return old (now closed) input
* file descriptor to allow disabling input handlers.
*/
int ahw_close(int iomode)
{
  int au = -1;

  if ((iomode & A_WR) && (au_play >= 0)) {
    close(au_play);
    au_play = -1;
  }
  if ((iomode & A_RD) && (au_rec >= 0)) {
    au = au_rec;
    close(au_rec);
    au_rec = -1;
  }
  return au;
} /* ahw_close */


/*
* Set audio hardware. Audio device does not have to be open.
* Disable polling signal. Return 1 if ok.
* pr     in   0: record, 1: play
* p      in   parameter to set
* value  in   value to be used; -1: don't change
*/
int ahw_set(int pr, audio_set_t p, int value)
{
} /* ahw_set */


/*
* Determine what kind of audio capabilities are supported by the audio
* device and set global variable 'audio_support' correspondingly.  To
* avoid noise in speaker, problems with restoring old audio properties
* and having to open the audio device (which may not be available at
* the time), the routine tries to determine the type of audio device
* (amd, bri) and then sets the known properties of this device. 
* (Precision can only be set with /dev/audio, not /dev/audioctl.)
* Things would more likely work in future Sun hardware versions if we
* just checked the actual settings, but...  Assumes that play
* capabilities are the same as record capabilities.
*/
void ahw_capabilities(void)
{
  /* check if structure has been filled already */
  if (audio_support.sample_rates[0]) return;

  /* basic 8-bit (amd) audio */
  audio_support.sample_rates[0] = 8000;
  audio_support.max_rate        = 8000;
  audio_support.precision[0]    = 8;
  audio_support.encodings       = SET(AE_PCMU);
  audio_support.max_channels    = 1;
  audio_support.rec_ports       = SET(AP_in_microphone);
  audio_support.play_ports      = SET(AP_out_speaker) | SET(AP_out_headphone);
} /* ahw_capabilities */


/*
* Return number of bytes in play queue.
* If the number is negative, an underflow has occurred.
* Return -1 if error.
*/
long ahw_write_queue(void)
{
  return 0;
} /* ahw_write_queue */


/*
* Return occupancy of record queue in bytes.
* Return negative value if error.
*/
long ahw_read_queue(void)
{
  return 0;
} /* ahw_read_queue */


/*
* Get current hardware audio status. If an audio error occurred, return
* as quickly as possible to avoid starving the CPU.
* Return 1 if error.
*/
int ahw_get(audio_state_t *as, u_int32 *record_errors, u_int32 *play_errors)
{
  return 0;
} /* ahw_get */


/*
* Return > 0 if another process is waiting for audio input or output.
*/
int ahw_waiting(void)
{
  return 0;
} /* ahw_waiting */


/*
* Write audio buffer to audio device.
*/
int ahw_write(void *buffer, size_t bytes)
{
  if (bytes == 0) return bytes;
  if (au_play >= 0) {
    bytes = write(au_play, buffer, bytes);
    if (bytes > 0) play_count += bytes;  
  }
  return bytes;
} /* ahw_write */


/*
* Read audio samples from audio device.
* buffer  out   buffer to receive audio data
* bytes   in    number of audio bytes to read
*/
int ahw_read(void *buffer, size_t bytes)
{
  bytes = read(au_rec, buffer, bytes);
  if (bytes > 0) rec_count += bytes;  
  return bytes;
} /* ahw_read */
