/*
init  --  initialization

Copyright 1994 by Henning Schulzrinne; all rights reserved
*/

#include <stdlib.h>     /* getenv(), srand() */
#include "nevot.h"
#include "audio_generic.h"
#include "ring.h"
#include "pcmu_l.h"
#include "bit.h"
#include "debug.h"
#include "mix.h"

#include <unistd.h>     /* getpid() */
#include <signal.h>     /* SIG_IGN */
#include <time.h>       /* time() */
#include <ansi.h>


nevot_global_t nevot;


/*
* Initialize global data structures.
* Return 0 on success, -1 on error.
*/
Audio_errno_t Audio_init(Audio_handler_t handler)
{
  nevot.event      = handler;
  nevot.audio.tick = 0.02;  /* seconds */

  /* ignore certain signals */
  signal(SIGPIPE, SIG_IGN);    /* broken pipe (peer dies) */

  /* Allow for 16-bit stereo at 48000 Hz. */
  nevot.maxlen = m_create(MAX_CACHE + 14, 48000*2 * MAX_PACKETIZATION + 200);

  srand(getpid());    /* for periodic control messages */
 
  /* get own addresses; add calls here for hosts with multiple addresses */
  nevot.tsap_count = 1; 
  tsap_gethostaddress(AF_INET, &nevot.tsap[0]);

  debug_file("");  /* default: stderr */
  debug_mask(0);
  pcmu_linear_init();
  audio_stats_init();
  mix_init();
  tx_init();
  rx_init();
  tx_packet_init();
  vat_init(0); rtp_init(0);
  audio_vu(0, 0.);
  audio_vu(1, 0.);
  return Audio_OK;
} /* Audio_init */

/* initialize the local address for multihomed hosts
* Return 0 on success, -1 on error.
*/
Audio_errno_t Audio_local(char *host)
{
  tsap_gethostbyname(host,&nevot.tsap[0],1);
}
