/* nevot_init.c */

static char rcsid[] = "$Id: nevot_init.c,v 1.3 1998/07/30 11:48:11 dor Exp $";

#include <tk.h>
#include <stdlib.h>
#include "debug.h"   /* debug() */
#include "util.h"    /* source_file() */
#include <sys/types.h>
#include <stdlib.h>
#include "nevot.h"

/* bitmaps */
#include "bitmaps/arrow_down"
#include "bitmaps/arrow_up"
#include "bitmaps/nevot_icon"
#include "bitmaps/nevot_mask"

/* C-versions of Tcl files */

char audioctl_tcl[];
extern char cmd_audioctl();
#include "blist.c"
#include "field.c"
#include "fileselect_f.c"
#include "hscale.c"
#include "nevot.c"
#include "nevot_defs.c"
#include "setup.c"
#include "stack.c"
#include "button_inc.c"
#include "getopt.c"
#include "tkerror_n.c"

extern Tcl_Interp *interp2;

static struct {
  char *name;
  char *var;
  char *def;
} libraries[] = {
  {"NEVOT_LIBRARY",      "nevot_library",      "."},
  {0,0}
};

static int vu_size = 0;

static struct {
  char *color;
  float start;
  float pixel;
} bar[] = {
  {"green",  0.0000, 0.0},
  {"yellow", 0.8333, 0.6},
  {"red",    0.9833, 0.9},
  {0,        1.0,    1.0}
};

/*
* Return right-most pixel position of VU value.
*/
static int vu_x(int i, float v)
{
  return ((v - bar[i].start) / (bar[i+1].start - bar[i].start) 
    * (bar[i+1].pixel - bar[i].pixel) + bar[i].pixel) * vu_size;
} /* vu_pos */

/*
* Display current VU 'value' (0..1) in VU meter 'm' (0: record, 1:
* play) as a horizontal bar; called by routine when update is needed. 
* Distinguishes green, yellow and red zone (tag names) with different
* scales.
* color   dB range         fraction    display
* Green   -60 dB .. -10 dB ..0.833     0.6
* Yellow  -10 dB .. -1 dB  ..0.983     0.9
* Red     -1  dB ..  0 dB  ..1         1.0
*/
void vu_show(int m, float value)
{
  char cmd[200];
  float v;              /* value for each bar */
  int y = m ? 35 : 5;   /* y coordinate */
  int i;

  /* Go through bars. */
  for (i = 0; i < 3; i++) {
    v = value;
    if (v > bar[i+1].start) v = bar[i+1].start;
    else if (v < bar[i].start) v = bar[i].start;
    sprintf(cmd, ".base.vu coords %s_%s %d %d %d %d",
      m ? "out" : "in", bar[i].color, 
      (int)(bar[i].pixel * vu_size), /* x: left start */
      y,                             /* y: left start */
      vu_x(i, v),                    /* x: right end */
      y);                            /* y: right end */
    Tcl_Eval(interp2, cmd);
  }
} /* vu_show */


/*
* Show maximum line.
*/
void vu_show_max(int m, float value)
{
  char cmd[200];
  int y = m ? 35 : 5;   /* y coordinate */
  int x;
  int i;

  for (i = 0; i < 3; i++) {  
    if (value < bar[i+1].start) {
      x = vu_x(i, value);
      sprintf(cmd, ".base.vu coords max %d %d %d %d",
        x, y-5, x, y+5);
      Tcl_Eval(interp2, cmd);
      sprintf(cmd, ".base.vu itemconfig max -fill %s",
        bar[i].color);
      Tcl_Eval(interp2, cmd);
      break;
    }
  }
} /* vu_show_max */


int Nevot_Init(Tcl_Interp *interp)
{
  extern char getopt_tcl[];
  extern char button_inc_tcl[];
  extern char control_tcl[];
  extern char control_udp_tcl[];
  extern char control_sio_tcl[];
  extern char control_tcp_tcl[];
  extern char tkerror_n_tcl[];
   
  extern char *prog_name;
  extern int Tcl_SetSockBuffer_cmd(ClientData clientData, Tcl_Interp *interp,                                     int argc, char **argv);
  int i;
  
  Tcl_SetVar(interp, "prog_name",prog_name, TCL_GLOBAL_ONLY);
  
  
   /* bitmaps */
  Tk_DefineBitmap(interp, Tk_GetUid("arrow_down"), arrow_down_bits,
    arrow_down_width, arrow_down_height);
  Tk_DefineBitmap(interp, Tk_GetUid("arrow_up"), arrow_up_bits,
    arrow_up_width, arrow_up_height);
  Tk_DefineBitmap(interp, Tk_GetUid("nevot_icon"), nevot_icon_bits,
    nevot_icon_width, nevot_icon_height);
  Tk_DefineBitmap(interp, Tk_GetUid("nevot_mask"), nevot_mask_bits,
    nevot_mask_width, nevot_mask_height);

  /* set up library directories */
  for (i = 0; libraries[i].name; i++) {
    char *lib_dir = getenv(libraries[i].name);
    if (lib_dir == NULL) lib_dir = libraries[i].def;
    Tcl_SetVar(interp, libraries[i].var, lib_dir, TCL_GLOBAL_ONLY);
  }

  /* set up bitmaps */
  Tk_DefineBitmap(interp, Tk_GetUid("nevot_icon"), nevot_icon_bits,
    nevot_icon_width, nevot_icon_height);
  Tk_DefineBitmap(interp, Tk_GetUid("nevot_mask"), nevot_mask_bits,
    nevot_mask_width, nevot_mask_height);

  /* link variables */
  Tcl_LinkVar(interp, "vu_size", (char *)&vu_size, TCL_LINK_INT);

  /*
   * Call Tcl_CreateCommand for application-specific commands, if
   * they weren't already created by the init procedures called above.
   */

  Tcl_CreateCommand(interp, "Tcl_SetSockBuffer", Tcl_SetSockBuffer_cmd, 0, 0);
  Tcl_CreateCommand(interp, "audioctl_tcl", cmd_audioctl, 0, 0);

  if (source_file(interp, "$nevot_library/nevot.tcl") < 0) {
    /* pull in C-versions of Tcl files */

    if (Tcl_Eval(interp, blist_tcl)       != TCL_OK) return TCL_ERROR;
    if (Tcl_Eval(interp, button_inc_tcl)  != TCL_OK) return TCL_ERROR;
    if (Tcl_Eval(interp, control_tcl)     != TCL_OK) return TCL_ERROR;
    if (Tcl_Eval(interp, control_sio_tcl) != TCL_OK) return TCL_ERROR;
    if (Tcl_Eval(interp, control_tcp_tcl) != TCL_OK) return TCL_ERROR;
    if (Tcl_Eval(interp, control_udp_tcl) != TCL_OK) return TCL_ERROR;
    if (Tcl_Eval(interp, field_tcl)       != TCL_OK) return TCL_ERROR;
    if (Tcl_Eval(interp, getopt_tcl)      != TCL_OK) return TCL_ERROR;
    if (Tcl_Eval(interp, fileselect_f_tcl)!= TCL_OK) return TCL_ERROR;
    if (Tcl_Eval(interp, hscale_tcl)      != TCL_OK) return TCL_ERROR;
    if (Tcl_Eval(interp, setup_tcl)       != TCL_OK) return TCL_ERROR;
    if (Tcl_Eval(interp, stack_tcl)       != TCL_OK) return TCL_ERROR;
    if (Tcl_Eval(interp, tkerror_n_tcl)   != TCL_OK) return TCL_ERROR;
    if (Tcl_Eval(interp, nevot_defs_tcl)  != TCL_OK) return TCL_ERROR;
    if (Tcl_Eval(interp, audioctl_tcl)    != TCL_OK) return TCL_ERROR;
    if (Tcl_Eval(interp, nevot_tcl)       != TCL_OK) return TCL_ERROR;
    
  }

  /*
   * User-specific startup file to invoke: None.
   */
  return TCL_OK;
}
