/***********************************************************************************
 *
 * sipd based on the draft: draft-ietf-mmusic-sip-02
 *
 * partially based on cd by Christian Zahl
 *
 * Copyright 1997 by Stefan Hoffmann
 * GMD Fokus
 *
 ***********************************************************************************/



#include <netdb.h>
#include <netinet/in.h>
#include <pwd.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/utsname.h>
#include <tcl.h>
#include <unistd.h>
#include "misc.h"
#include "notify.h"
#include "debug.h"
#include "defines.h"
#include "event.h"
#include "list.h"
#include "timer.h"


extern char      *optarg;
extern int       optind;
int              debugLevel = 1;
int              logFacillity = LOG_LOCAL0;
char             myAddr[17]; /* IP address of the local host*/
int              tcp_server_socket, udp_server_socket; /* server sockets for incoming SIP requests */
int              isc_socket;  /* local multicast socket to isc */
int              isc_connections = 0; /* number of connections to isc */
int              sip_port = -1; /* SIP port */
inv_list_t       *invitation_list = NULL; /* list of active invitations */

Tcl_Interp *interp;

char             alias_db[255];
char             ls_db[255];
char             sipd_bin_path[255];
char             isc_bin_path[255];
char             isc_audio_path[255];
char             ConfigFile[255];

char             LocationService[255];
char             ExpandService[255];
char             ForwardService[255];
char             StartIscService[255];


/* Exit *****************************************************************************
 *
 *  exits the program
 *  deletes event handlers of server sockets
 *
 ***********************************************************************************/
void Exit ()
{

  notify_set_input_func(0, NOTIFY_FUNC_INPUT_NULL, tcp_server_socket);
  notify_set_input_func(0, NOTIFY_FUNC_INPUT_NULL, udp_server_socket);
  notify_stop();

  while (invitation_list != NULL) 
    close_invitation(invitation_list);
  if (isc_connections > 0)
    close(isc_socket);
    
  close (tcp_server_socket);
  close (udp_server_socket);
  closelog ();
  DEBUG (2, syslog (LOG_DEBUG, "sipd exiting");)
  exit (1);
} /* Exit */



/* get_configuration ***************************************************************
 *
 *  reads ~/.sipdrc configuration file
 *
 ***********************************************************************************/
void get_configuration ()
{
  FILE *config_file;
  char line[300];
  struct passwd *pwEntry;

  if (*ConfigFile == NULL) {
    /* get home directory */
    if ((pwEntry = getpwnam(cuserid(NULL))) != NULL) {
      sprintf(ConfigFile, "%s%s", pwEntry->pw_dir, "/.sipdrc"); 
    }
  }

  if ((config_file = fopen(ConfigFile, "r")) != NULL) {
    while ((fgets(line, 299, config_file)) != NULL) {
      line[strlen(line)-1] = '\0';
      if (strstr(line, "AliasDataBase:") == line && *alias_db == NULL) {
	strcpy(alias_db, skip_white_spaces(strstr(line, ":")+1));
	continue;
      }
      if (strstr(line, "SipdBinaryPath:") == line && *sipd_bin_path == NULL) {
	strcpy(sipd_bin_path, skip_white_spaces(strstr(line, ":")+1));
    	continue;
      }
      if (strstr(line, "ExpandService:") == line && *ExpandService == NULL) {
	strcpy(ExpandService, skip_white_spaces(strstr(line, ":")+1));
   	continue;
      }
      if (strstr(line, "SessionController:") == line && *isc_bin_path == NULL) {
	strcpy(isc_bin_path, skip_white_spaces(strstr(line, ":")+1));
    	continue;
      }
      if (strstr(line, "IscAudioPath:") == line && *isc_audio_path == NULL) {
    	strcpy(isc_audio_path, skip_white_spaces(strstr(line, ":")+1));
    	continue;
      }
      if (strstr(line, "LocationServiceDataBase:") == line && *ls_db == NULL) {
	strcpy(ls_db, skip_white_spaces(strstr(line, ":")+1));
     	continue;
      }
      if (strstr(line, "LocationService:") == line && *LocationService == NULL) {
	strcpy(LocationService, skip_white_spaces(strstr(line, ":")+1));
    	continue;
      }
      if (strstr(line, "StartIscService:") == line && *StartIscService == NULL) {
	strcpy(StartIscService, skip_white_spaces(strstr(line, ":")+1));
     	continue;
      }
      if (strstr(line, "ForwardService:") == line && *ForwardService == NULL) {
	strcpy(ForwardService, skip_white_spaces(strstr(line, ":")+1));
    	continue;
      }
    }
    fclose(config_file);
  }
} /* get_configuration */



void usage ()
{
  printf("Usage: %s\n [-a AliasDataBase]\n [-b SipdBinaryPath]\n [-d debug level]\n [-f ConfigFile]\n [-i SessionController]\n [-I IscAudioPath]\n [-l LocationServiceDataBase]\n [-p sip port]\n\n [-E ExpandService]\n [-F ForwardService]\n [-L LocationService]\n [-S SartIscService]\n", PROG_NAME);
  Exit ();
} /* usage */



void parse_options (int argc, char *argv[])
{
  int c;
  
  while ((c = getopt (argc, argv, "a:b:d:E:f:F:i:I:l:L:p:S:?")) != -1)
    switch (c) {
    case 'a':
      strcpy(alias_db, optarg);
      break;
    case 'b':
      strcpy(sipd_bin_path, optarg);
      break;
    case 'd':
      debugLevel = atoi (optarg);
      break;
    case 'E':
      strcpy(ExpandService, optarg);
      break;
    case 'f':
      strcpy(ConfigFile, optarg);
      break;
    case 'F':
      strcpy(ForwardService, optarg);
      break;
    case 'i':
      strcpy(isc_bin_path, optarg);
      break;
   case 'I':
      strcpy(isc_audio_path, optarg);
      break;
    case 'l':
      strcpy(ls_db, optarg);
      break;
    case 'L':
      strcpy(LocationService, optarg);
      break;
    case 'p':
      sip_port = atoi (optarg);
      break;
    case 'S':
      strcpy(StartIscService, optarg);
      break;
    case '?':
      DEBUG (1, syslog (LOG_ERR, "Bad option [%c];", c);)
      usage ();
    }
  if (argc != optind)
    usage ();
} /* parse_option */



/* check_option ********************************************************************
 *
 *  checks if the specified options are valid
 *
 ***********************************************************************************/
void check_options()
{

  if (*ExpandService != NULL && *ExpandService != '\0') {
    if (access(ExpandService, X_OK)) {
      printf("You have specified the ExpandService option but the setting is not valid.\nPlease modify the settings in the .sipdrc file or the command line options and restart sipd.\n");
      Exit();
    }
  }
  else {
    *ExpandService = NULL;
    if (*sipd_bin_path != NULL && *sipd_bin_path != '\0') {
      if (access(sipd_bin_path, X_OK)) {
	printf("You have specified the SipdBinaryPath option but the setting is not valid.\nPlease modify the settings in the .sipdrc file or the command line options and restart sipd.\n");
      Exit();
      }
    }
    else {
      printf("No settings specified for option SipdBinaryPath.\nPlease modify the settings in the .sipdrc file or the command line options and restart sipd.\n"); 
    Exit();
    }
    if (*alias_db == NULL || *alias_db == '\0' || access(alias_db, R_OK)) {
      printf("Warning! The setting for option AliasDataBase is not valid!\n");
    }
  }

  if (*LocationService != NULL && *LocationService != '\0') {
    if (access(LocationService, X_OK)) {
      printf("You have specified the LocationService option but the setting is not valid.\nPlease modify the settings in the .sipdrc file or the command line options and restart sipd.\n");
      Exit();
    }
  }
  else {
    *LocationService = NULL;
    if (*ls_db != NULL && *ls_db != '\0') {
      if (access(ls_db, R_OK)) {
	printf("You have specified the LocationServiceDataBase option but the setting is not valid.\nPlease modify the settings in the .sipdrc file or the command line options and restart sipd.\n");
      Exit();
      }
    }
    else {
      printf("No settings specified for option LocationServiceDataBase.\nPlease modify the settings in the .sipdrc file or the command line options and restart sipd.\n"); 
      Exit();
    }
  }

  if (*ForwardService != NULL && *ForwardService != '\0') {
    if (access(ForwardService, X_OK)) {
      printf("You have specified the ForwardService option but the setting is not valid.\nPlease modify the settings in the .sipdrc file or the command line options and restart sipd.\n");
      Exit();
    }
  }
  else {
    *ForwardService = NULL;
  }

  if (*StartIscService != NULL && *StartIscService != '\0') {
    if (access(StartIscService, X_OK)) {
      printf("You have specified the StartIscService option but the setting is not valid.\nPlease modify the settings in the .sipdrc file or the command line options and restart sipd.\n");
      Exit();
    }
  }
  else {
    *StartIscService = NULL;
    if (*isc_bin_path != NULL && *isc_bin_path != '\0') {
      if (access(isc_bin_path, X_OK)) {
	printf("You have specified the SessionController option but the setting is not valid.\nPlease modify the settings in the .sipdrc file or the command line options and restart sipd.\n");
      Exit();
      }
    }
    else {
      printf("No settings specified for option IscBinaryPath.\nPlease modify the settings in the .sipdrc file or the command line options and restart sipd.\n"); 
      Exit();
    }
    if (*isc_audio_path == NULL || *isc_audio_path == '\0' || access(isc_audio_path, F_OK)) {
      printf("Warning! The setting for option IscAudioPath is not valid!\n");
      *isc_audio_path = NULL; 
    }
  }
} /* check_options */



/* create_server_socket **********************************************************************************
 * 
 *  allocate & bind server sockets for TCP or UDP
 *  
 *  in:  
 *  - protocol: type of socket, "udp" or "tcp"
 *  - qlen: length queue for pending connections for tcp
 *
 *  returns:
 *  - socket
 *  - -1 on failure
 *
 ***********************************************************************************/
int create_server_socket (char *protocol, int qlen)
{
  static char          func[] = "create_server_socket";
  struct servent       *servEntry;
  struct sockaddr_in   sin;
  int                  serverSocket;
  int                  type;
  int                  size = SOCK_BUF_SIZE;
  int                  on = 1;


  /***********************************************************************************
   * lookup for our sip_port 
   ***********************************************************************************/
  if (sip_port != -1) {
    DEBUG (2, syslog (LOG_DEBUG, "%s: using portnumber %d as specified", func, sip_port););
  }
  else if ((servEntry = getservbyname (SIP_SERVICE_NAME, protocol)) == NULL) {
    DEBUG (1, syslog (LOG_WARNING, "%s: Warning, service %s not defiend, using default %d/tcp", func, SIP_SERVICE_NAME, SIP_PORT_NO);)
    sip_port = SIP_PORT_NO;
  }
  else 
    sip_port = htons(servEntry->s_port);
    
  if (strcmp(protocol, "udp") == 0)
    type = SOCK_DGRAM;
  else
    type = SOCK_STREAM;
  
  /***********************************************************************************
   * get the socket 
   ***********************************************************************************/
  if ((serverSocket = socket (AF_INET, type, 0)) == -1) {
    DEBUG (1, syslog (LOG_ERR, "%s: socket: %m", func);)
    Exit ();
  }

  if (setsockopt (serverSocket, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof (on)) == -1) {
    DEBUG (1, syslog (LOG_ERR, "%s: setsockopt: %m", func);)
    close (serverSocket);
    Exit ();
  }

  if (setsockopt (serverSocket, SOL_SOCKET, SO_SNDBUF,(char *)&size, sizeof(int)) < 0) {
    close (serverSocket);
    Exit ();
  }

  if (setsockopt (serverSocket, SOL_SOCKET, SO_RCVBUF,(char *)&size, sizeof(int)) < 0) {
    close (serverSocket);
    Exit ();
  }

 /***********************************************************************************
   * bind the socket 
   ***********************************************************************************/
  memset (&sin, 0, sizeof (sin));
  sin.sin_family = AF_INET;
  sin.sin_addr.s_addr = INADDR_ANY;
  sin.sin_port = htons(sip_port);
  if (bind (serverSocket, (struct sockaddr *)&sin, sizeof (sin)) == -1) {
    DEBUG (1, syslog (LOG_ERR, "%s: bind: %m", func);)
    close (serverSocket); 
    Exit ();
  }

  if (type == SOCK_STREAM)
    if (listen (serverSocket, qlen) == -1) {
      DEBUG (1, syslog (LOG_ERR, "%s: listen: %m", func);)
      close (serverSocket);
      Exit ();
    };

  return serverSocket;
} /* create_server_socket */



/* server_main_loop ****************************************************************
 *
 * initialize the daemon mode
 * get tcp and udp server sockets
 * install event handlers for server sockets
 * start event handler loop
 *
 ***********************************************************************************/
void server_main_loop ()
{
  static char func[] = "server_main_loop";
  
  /***********************************************************************************
   * initialize the daemon mode
   ***********************************************************************************/
  switch (fork()) {      
    case -1:      
      DEBUG (1, syslog (LOG_ERR, "%s: fork: %m", func);)      
      closelog ();      
      exit (-1);      
    case 0:     
       break;      
    default:     
      closelog ();      
      exit (0);      
  }      


  setpgrp();    
  chdir ("/");    
  umask (0);    
  DEBUG (2, syslog (LOG_DEBUG, "entering server mode...");)

  signal(SIGINT, Exit);
  signal(SIGKILL, Exit);
  signal(SIGTERM, Exit);

  /***********************************************************************************
   * get tcp and udp server sockets
   ***********************************************************************************/
  tcp_server_socket = create_server_socket("tcp",5);
  udp_server_socket = create_server_socket("udp",0);
    
  /***********************************************************************************
   * install event handlers for server sockets
   ***********************************************************************************/
  notify_set_input_func(0, socket_event, tcp_server_socket);
  notify_set_input_func(0, socket_event, udp_server_socket);
  notify_start();
} /* server_main_loop */



/* main ****************************************************************************
 *
 ***********************************************************************************/
main (int argc, char *argv[])
{
  struct utsname utsname;
  struct hostent *ip;

  interp = Tcl_CreateInterp();

   /* initialize all options to NULL */
  *alias_db = NULL;
  *ls_db = NULL;
  *sipd_bin_path = NULL;
  *isc_bin_path = NULL;
  *isc_audio_path = NULL;
  *LocationService = NULL;
  *ExpandService = NULL;
  *ForwardService = NULL;
  *StartIscService = NULL;
  *ConfigFile = NULL;
  
  parse_options (argc, argv);
  get_configuration();
  check_options();
  
  openlog (PROG_NAME, LOG_PID, logFacillity);
  DEBUG (1, syslog (LOG_DEBUG, "Started...");)

  uname (&utsname);
  /* get own Internet number */
  if ((ip = gethostbyname(utsname.nodename)) && ip->h_addrtype == AF_INET) {
    struct in_addr in;
    in.s_addr = *(u_long *)(*ip->h_addr_list);
    strcpy(myAddr, (char *)inet_ntoa(in));
  }

  server_main_loop();
} /* main */

